/*
Copyright (C) 2009, 2010 Alex Ignácio da Silva

This file is part of Haundrix Games.

Haundrix Games is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License version 3 as published by the
Free Software Foundation, with additional permission under section 7 to combine
it with independent works licensed under selected Free/Libre Open Source Software licenses.
Please see the full text of the license that is distributed along with this file.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
*/
import org.eclipse.swt.widgets.Display;
import org.jdesktop.animation.timing.TimingEventListener;
import org.jdesktop.animation.timing.TimingSource;

import com.haundrix.animation.SwtTimingSource;

public class SwtTimingSourceDemo2 {
    
    public static void main(String[] args) {
        System.out.printf("                                  measured\n");
        System.out.printf("timer delay   iterations   total time   per-delay\n");
        Display display = new Display();
        TimerThread timerThread = new TimerThread(display);
        timerThread.start();
        while(timerThread.isAlive()) {
            if(!display.readAndDispatch()) {
                display.sleep();
            }
        }
        display.dispose();
    }
    
    private static class TimerThread extends Thread {

        private Display display;
        
        private volatile int timerIteration;
        private volatile long startTime;

        public TimerThread(Display display) {
            this.display = display;
        }

        @Override
        public void run() {
            for(int resolution = 0; resolution <= 20; ++resolution) {
                final int nIterations = (resolution == 0) ? 1000 : (1000 / resolution);
                timerIteration = 1;
                final SwtTimingSource timingSource = new SwtTimingSource(display, resolution);
                final int theResolution = resolution;  // we need a final variable to use inside the inner class
                timingSource.addEventListener(new TimingEventListener() {
                    @Override
                    public void timingSourceEvent(TimingSource timingSource) {
                        if(++timerIteration > nIterations) {
                            timingSource.stop();
                            timerIteration = 0;
                            long endTime = System.nanoTime();
                            long totalTime = (endTime - startTime) / 1000000;
                            float calculatedDelayTime = totalTime / (float) nIterations;
                            System.out.printf("  %2d          %5d         %5d        %5.2f\n", 
                                    theResolution, nIterations, totalTime, calculatedDelayTime);
                        }
                    }
                });
                startTime = System.nanoTime();
                timingSource.start();
                while(timerIteration > 0) {
                    try {
                        Thread.sleep(1000);
                    } catch (Exception e) {}
                }
            }
            // let's post one last event to the GUI thread to guarantee
            // it doesn't get stuck in the display.sleep method call
            display.asyncExec(new Runnable() {
                @Override
                public void run() {
                    System.out.println("Over!");
                }
            });
        }
        
    }
    
}
