/*
Copyright (C) 2009, 2010 Alex Ignácio da Silva

This file is part of Haundrix Games.

Haundrix Games is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License version 3 as published by the
Free Software Foundation, with additional permission under section 7 to combine
it with independent works licensed under selected Free/Libre Open Source Software licenses.
Please see the full text of the license that is distributed along with this file.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
*/
import java.util.HashMap;
import java.util.Map;

import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.jdesktop.animation.timing.interpolation.Evaluator;

import com.haundrix.animation.SwtPointEvaluator;
import com.haundrix.games.chess.view.ChessPieceType;
import com.haundrix.games.controller.TestBoardController;
import com.haundrix.games.view.FilePieceImageDescriptor;
import com.haundrix.games.view.IPieceImageDescriptor;
import com.haundrix.games.view.PieceSet;
import com.haundrix.games.view.RectangularBoardControl;

public class TestBoardControllerDemo {
    
    private static final String PIECE_SET_DIRECTORY = "pieceSets/condal/";

    public static void main(String[] args) {
        Evaluator.register(Point.class, SwtPointEvaluator.class);
        
        Display display = new Display();
        Shell shell = new Shell(display);
        shell.setLayout(new FillLayout());

        Map<ChessPieceType, IPieceImageDescriptor> pieceImages = new HashMap<ChessPieceType, IPieceImageDescriptor>();
        pieceImages.put(ChessPieceType.BLACK_ROOK, new FilePieceImageDescriptor(PIECE_SET_DIRECTORY + "blackRook.png"));
        pieceImages.put(ChessPieceType.BLACK_KNIGHT, new FilePieceImageDescriptor(PIECE_SET_DIRECTORY + "blackKnight.png"));
        pieceImages.put(ChessPieceType.BLACK_BISHOP, new FilePieceImageDescriptor(PIECE_SET_DIRECTORY + "blackBishop.png"));
        pieceImages.put(ChessPieceType.BLACK_QUEEN, new FilePieceImageDescriptor(PIECE_SET_DIRECTORY + "blackQueen.png"));
        pieceImages.put(ChessPieceType.BLACK_KING, new FilePieceImageDescriptor(PIECE_SET_DIRECTORY + "blackKing.png"));
        pieceImages.put(ChessPieceType.BLACK_PAWN, new FilePieceImageDescriptor(PIECE_SET_DIRECTORY + "blackPawn.png"));
        pieceImages.put(ChessPieceType.WHITE_ROOK, new FilePieceImageDescriptor(PIECE_SET_DIRECTORY + "whiteRook.png"));
        pieceImages.put(ChessPieceType.WHITE_KNIGHT, new FilePieceImageDescriptor(PIECE_SET_DIRECTORY + "whiteKnight.png"));
        pieceImages.put(ChessPieceType.WHITE_BISHOP, new FilePieceImageDescriptor(PIECE_SET_DIRECTORY + "whiteBishop.png"));
        pieceImages.put(ChessPieceType.WHITE_QUEEN, new FilePieceImageDescriptor(PIECE_SET_DIRECTORY + "whiteQueen.png"));
        pieceImages.put(ChessPieceType.WHITE_KING, new FilePieceImageDescriptor(PIECE_SET_DIRECTORY + "whiteKing.png"));
        pieceImages.put(ChessPieceType.WHITE_PAWN, new FilePieceImageDescriptor(PIECE_SET_DIRECTORY + "whitePawn.png"));
        PieceSet<ChessPieceType> pieceSetDescriptor = new PieceSet<ChessPieceType>(pieceImages);
        
        RectangularBoardControl<ChessPieceType> board = new RectangularBoardControl<ChessPieceType>(shell, 8, 8, pieceSetDescriptor);
        board.setPiece(ChessPieceType.BLACK_ROOK, 0, 0);
        board.setPiece(ChessPieceType.BLACK_KNIGHT, 0, 1);
        board.setPiece(ChessPieceType.BLACK_BISHOP, 0, 2);
        board.setPiece(ChessPieceType.BLACK_QUEEN, 0, 3);
        board.setPiece(ChessPieceType.BLACK_KING, 0, 4);
        board.setPiece(ChessPieceType.BLACK_BISHOP, 0, 5);
        board.setPiece(ChessPieceType.BLACK_KNIGHT, 0, 6);
        board.setPiece(ChessPieceType.BLACK_ROOK, 0, 7);
        for(int column = 0; column < 8; column++) {
            board.setPiece(ChessPieceType.BLACK_PAWN, 1, column);
        }
        board.setPiece(ChessPieceType.WHITE_ROOK, 7, 0);
        board.setPiece(ChessPieceType.WHITE_KNIGHT, 7, 1);
        board.setPiece(ChessPieceType.WHITE_BISHOP, 7, 2);
        board.setPiece(ChessPieceType.WHITE_QUEEN, 7, 3);
        board.setPiece(ChessPieceType.WHITE_KING, 7, 4);
        board.setPiece(ChessPieceType.WHITE_BISHOP, 7, 5);
        board.setPiece(ChessPieceType.WHITE_KNIGHT, 7, 6);
        board.setPiece(ChessPieceType.WHITE_ROOK, 7, 7);
        for(int column = 0; column < 8; column++) {
            board.setPiece(ChessPieceType.WHITE_PAWN, 6, column);
        }
        TestBoardController boardController = new TestBoardController(board);
        board.addBoardListener(boardController);

        shell.pack();
        shell.setSize(800, 600);
        shell.open();
        while(!shell.isDisposed()) {
            if(!display.readAndDispatch())
                display.sleep();
        }
        display.dispose();
    }

}
