/*
Copyright (C) 2009, 2010 Alex Ignácio da Silva

This file is part of Haundrix Games.

Haundrix Games is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License version 3 as published by the
Free Software Foundation, with additional permission under section 7 to combine
it with independent works licensed under selected Free/Libre Open Source Software licenses.
Please see the full text of the license that is distributed along with this file.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
*/
package com.haundrix.games.chess.model;

import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.DataProvider;

import com.haundrix.games.model.Move;
import com.haundrix.games.model.MoveNotationTest;
import com.haundrix.games.model.Square;

public class ChessAlgebraicMoveNotationDefaultOptionsTest extends MoveNotationTest {
    
    private final Move[] moves1 = {
        new Move(new Square(6, 4), new Square(4, 4)),  // e4
        new Move(new Square(1, 3), new Square(3, 3)),  //            d5
        new Move(new Square(4, 4), new Square(3, 4)),  // e5
        new Move(new Square(1, 5), new Square(3, 5)),  //            f5
        new Move(new Square(3, 4), new Square(2, 5)),  // exf6 e.p.
        new Move(new Square(0, 1), new Square(2, 2)),  //            Nc6
        new Move(new Square(7, 6), new Square(5, 5)),  // Nf3
        new Move(new Square(0, 2), new Square(4, 6)),  //            Bg4
        new Move(new Square(7, 5), new Square(4, 2)),  // Bc4
        new Move(new Square(0, 3), new Square(2, 3)),  //            Qd6
        new Move(new Square(7, 4), new Square(7, 6)),  // 0-0
        new Move(new Square(0, 4), new Square(0, 2)),  //            0-0-0
        new Move(new Square(2, 5), new Square(1, 6)),  // fxg7
        new Move(new Square(3, 3), new Square(4, 2)),  //            dxc4
        new Move(new Square(6, 1), new Square(4, 1)),  // b4
        new Move(new Square(0, 6), new Square(2, 5)),  //            Nf6
        new PromotionMove(new Square(1, 6), new Square(0, 5), Queen.class),  // gxf8=Q
        new Move(new Square(0, 7), new Square(0, 5)),  //            Rhxf8
        new Move(new Square(6, 3), new Square(5, 3)),  // d3
        new Move(new Square(2, 3), new Square(5, 3)),  //            Qxd3
        new Move(new Square(7, 1), new Square(6, 3)),  // Nbd2
        new Move(new Square(4, 6), new Square(5, 5)),  //            Bxf3
        new Move(new Square(7, 3), new Square(5, 5)),  // Qxf3
        new Move(new Square(5, 3), new Square(7, 5)),  //            Qxf1+
        new Move(new Square(6, 3), new Square(7, 5)),  // Nxf1
        new Move(new Square(2, 2), new Square(4, 1)),  //            Nxb4
        new Move(new Square(5, 5), new Square(3, 5)),  // Qf5+
        new Move(new Square(0, 2), new Square(0, 1)),  //            Kb8
        new Move(new Square(7, 5), new Square(5, 6)),  // Ng3
        new Move(new Square(0, 5), new Square(0, 4)),  //            Rfe8
        new Move(new Square(5, 6), new Square(4, 4)),  // Ne4
        new Move(new Square(0, 3), new Square(7, 3)),  //            Rd1++
    };
    
    private final String[] moveDescriptions1 = {
        "e4", "d5",
        "e5", "f5",
        "exf6 e.p.", "Nc6",
        "Nf3", "Bg4",
        "Bc4", "Qd6",
        "0-0", "0-0-0",
        "fxg7", "dxc4",
        "b4", "Nf6",
        "gxf8=Q", "Rhxf8",
        "d3", "Qxd3",
        "Nbd2", "Bxf3",
        "Qxf3", "Qxf1+",
        "Nxf1", "Nxb4",
        "Qf5+", "Kb8",
        "Ng3", "Rfe8",
        "Ne4", "Rd1++",
    };
    
    private final String[][] invalidMoveDescriptions1 = {
        {"garbage", "e5", "e4+", "e4++"},  // e4
        {"asfsadf", "dx5", "exd5", "d5 e.p."},  // d5
        {"e6", "xe5", "e5=Q", "e5@"},  // e5
        {"f7", "exf5", "f5+", "aaa"},  // f5
        {"exf6", "exf6 e.p", "exf6 ep", "exf6 e.p.=Q", "exf6 e.p.+", "f6 e.p."},  // exf6 e.p.
        {"Nbc6", "N8c6", "Nb8c6", "Nxc6"},  // Nc6
        {"Nf4", "Nxf3", "Nf3+"},  // Nf3
        {"Bcg4", "Bxg4", "Bg4++"},  // Bg4
        {"Bz4", "Yc4", "bc4", "BBc4"},  // Bc4
        {"d6", "Qxd5", "Qxd6", "0-0-0"},  // Qd6
        {"O-O", "O-O-O", "0-0-0", "0-0+", "0-0 e.p."},  // 0-0
        {"0-0", "O-O-O", "0-0-0++", "0-0-0#", "0-0-0=Q"},  // 0-0-0
        {"xg7", "g7", "fxg7 e.p."},  // fxg7
        {"Pxc4", "Pc4", "c4", "dxc4+"},  // dxc4
        {"b5", "bb4", "2b4"},  // b4
        {"Nf5", "Nxf6", "nf6"},  // Nf6
        {"gxf8 e.p.=Q", "f8=Q", "gxf8=K", "gxf8=P", "gxf8=Q+"},  // gxf8=Q
        {"Rh8xf8", "R8xf8", "Rxe8"},  // Rhxf8
        {"d3+", "Pd3", "dd3", "2d3"},  // d3
        {"Q6d3", "d6d3"},  // Qxd3
        {"N1d2", "Nb1d2", "Nbxd2"},  // Nbd2
        {"Rxf3", "Nxf3", "Kxf3"},  // Bxf3
        {"Qxf3++", "gxf3 e.p.", "Qa8"},  // Qxf3
        {"Qxf1++", "Qxf1#"},  // Qxf1+
        {"nxf1", "Qxf1", "Nxf1+"},  // Nxf1
        {"Ncxb4", "N6xb4", "Nc6xb4", "cxb4"},  // Nxb4
        {"Qxf5+", "Qf5++", "Qf5#"},  // Qf5+
        {"b8", "Kb8lsfjsaliecv", "Kxb8"},  // Kb8
        {"Nxg3", "Ng3+", "Bg3", "Pg3"},  // Ng3
        {"Rf8e8", "Re8", "Rfxe8"},  // Rfe8
        {"Qxy8", "Ne9"},  // Ne4
        {"rd1++", "Rd1+", "Rd1=Q++"},  // Rd1++
    };

    private final Move[] moves2 = {
        new Move(new Square(6, 3), new Square(4, 3)),  // d4
        new Move(new Square(1, 4), new Square(3, 4)),  //            e5
        new Move(new Square(4, 3), new Square(3, 4)),  // dxe5
        new Move(new Square(1, 3), new Square(3, 3)),  //            d5
        new Move(new Square(3, 4), new Square(2, 3)),  // exd6 e.p.
        new Move(new Square(0, 5), new Square(2, 3)),  //            Bxd6
        new Move(new Square(6, 4), new Square(4, 4)),  // e4
        new Move(new Square(1, 5), new Square(3, 5)),  //            f5
        new Move(new Square(4, 4), new Square(3, 5)),  // exf5
        new Move(new Square(0, 2), new Square(3, 5)),  //            Bxf5
        new Move(new Square(7, 1), new Square(5, 2)),  // Nc3
        new Move(new Square(2, 3), new Square(4, 1)),  //            Bb4
        new Move(new Square(7, 6), new Square(6, 4)),  // Ne2 (no need to disambiguate, since the other knight is pinned
        new Move(new Square(0, 3), new Square(5, 3)),  //            Qd3
        new Move(new Square(7, 3), new Square(5, 3)),  // Qxd3
        new Move(new Square(4, 1), new Square(3, 2)),  //            Bc5
        new Move(new Square(5, 2), new Square(4, 4)),  // Ne4
        new Move(new Square(0, 6), new Square(2, 5)),  //            Nf6
        new Move(new Square(5, 3), new Square(5, 4)),  // Qe3
        new Move(new Square(0, 1), new Square(1, 3)),  //            Nbd7
        new Move(new Square(4, 4), new Square(5, 2)),  // N4c3+ (still have to disambiguate the move, despite the fact that only one knight can give check)
        new Move(new Square(3, 2), new Square(5, 4)),  // Bxe3
        new Move(new Square(6, 5), new Square(5, 4)),  // fxe3
        new Move(new Square(0, 4), new Square(1, 4)),  // Ke7
        new Move(new Square(5, 4), new Square(4, 4)),  // e4
        new Move(new Square(1, 4), new Square(2, 4)),  // Ke6
        new Move(new Square(4, 4), new Square(3, 5)),  // exf5+
    };
    
    private final String[] moveDescriptions2 = {
        "d4", "e5",
        "dxe5", "d5",
        "exd6 e.p.", "Bxd6",
        "e4", "f5",
        "exf5", "Bxf5",
        "Nc3", "Bb4",
        "Ne2", "Qd3",
        "Qxd3", "Bc5",
        "Ne4", "Nf6",
        "Qe3", "Nbd7",
        "N4c3+", "Bxe3",
        "fxe3", "Ke7",
        "e4", "Ke6",
        "exf5+",
    };
    
    private final String[][] invalidMoveDescriptions2 = {
        {},  // d4
        {},  // e5
        {},  // dxe5
        {},  // d5
        {},  // exd6 e.p.
        {},  // Bxd6
        {},  // e4
        {},  // f5
        {},  // exf5
        {},  // Bxf5
        {},  // Nc3
        {},  // Bb4
        {"Nge2", "N1e2", "Ng1e2"},  // Ne2 (no need to disambiguate, since the other knight is pinned
        {},  // Qd3
        {},  // Qxd3
        {},  // Bc5
        {},  // Ne4
        {},  // Nf6
        {},  // Qe3
        {},  // Nbd7
        {"Nc3", "Nc3+", "Nec3+", "Nec3"},  // N4c3+ (still have to disambiguate the move, despite the fact that only one knight can give check)
        {},  // Bxe3
        {},  // fxe3
        {},  // Ke7
        {},  // e4
        {},  // Ke6
        {},  // exf5+
    };
    
    @BeforeMethod
    public void setUp() {
        game = new ChessGame();
        notation = new ChessAlgebraicMoveNotation();
    }
    
    @AfterMethod
    public void tearDown() {
        game = null;
        notation = null;
    }
    
    @DataProvider(name=MoveNotationTest.MOVE_BY_MOVE_TEST_DATA_PROVIDER)
    public Object[][] getMoveByMoveTestData() {
        return new Object[][] {
            {moves1, moveDescriptions1, invalidMoveDescriptions1},
            {moves2, moveDescriptions2, invalidMoveDescriptions2}
        };
    }
    
    @DataProvider(name=MoveNotationTest.ALL_MOVES_TEST_DATA_PROVIDER)
    public Object[][] getAllMovesTestData() {
        return new Object[][] {
            {moves1, moveDescriptions1},
            {moves2, moveDescriptions2}
        };
    }
    
}
