/*
Copyright (C) 2009, 2010 Alex Ignácio da Silva

This file is part of Haundrix Games.

Haundrix Games is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License version 3 as published by the
Free Software Foundation, with additional permission under section 7 to combine
it with independent works licensed under selected Free/Libre Open Source Software licenses.
Please see the full text of the license that is distributed along with this file.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
*/
package com.haundrix.games.chess.model;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.apache.commons.lang.ArrayUtils;
import org.testng.Assert;
import org.testng.annotations.Test;

import com.haundrix.games.model.Move;
import com.haundrix.games.model.Square;
import com.haundrix.games.model.Piece.PieceColor;
import com.haundrix.tests.HaundrixAssert;

public class ChessGameMovesTest extends ChessTest {
    
    @Test
    public void testRookMovement() {
        ChessPiece[][] pieces = {
    // columns :  0     1     2     3     4     5     6     7
    /* row 0 */ { k  , null,  n1 , null, null, null, null,  r1 },  // row 0
    /* row 1 */ {null, null, null, null, null, null, null, null},  // row 1
    /* row 2 */ {null, null, null, null, null, null, null,  p2 },  // row 2
    /* row 3 */ { p1 , null, null, null, null, null, null, null},  // row 3
    /* row 4 */ { b1 , null,  R1 , null, null,  B1 , null, null},  // row 4
    /* row 5 */ {null, null, null, null, null, null, null, null},  // row 5
    /* row 6 */ { K  , null,  P1 , null, null, null,  P2 ,  R2 },  // row 6
    /* row 7 */ {null, null,  q1 , null, null, null, null, null},  // row 7
    // columns :  0     1     2     3     4     5     6     7
        };
        setUpGame(pieces);
        gameState.setEnPassantSquare(new Square(2, 0));
        
        Square[] whiteRookSquares = {new Square(4, 2), new Square(6, 7)};
        Move[] allLegalRookMoves = {
                new Move(new Square(4, 2), new Square(4, 0)),
                new Move(new Square(4, 2), new Square(4, 1)),
                new Move(new Square(4, 2), new Square(4, 3)),
                new Move(new Square(4, 2), new Square(4, 4)),
                new Move(new Square(4, 2), new Square(0, 2)),
                new Move(new Square(4, 2), new Square(1, 2)),
                new Move(new Square(4, 2), new Square(2, 2)),
                new Move(new Square(4, 2), new Square(3, 2)),
                new Move(new Square(4, 2), new Square(5, 2)),
                
                new Move(new Square(6, 7), new Square(2, 7)),
                new Move(new Square(6, 7), new Square(3, 7)),
                new Move(new Square(6, 7), new Square(4, 7)),
                new Move(new Square(6, 7), new Square(5, 7)),
                new Move(new Square(6, 7), new Square(7, 7)),
        };
        testAllLegalMovesFrom(whiteRookSquares, allLegalRookMoves);
    }

    @Test
    public void testKnightMovement() {
        ChessPiece[][] pieces = {
    // columns :  0     1     2     3     4     5     6     7
    /* row 0 */ { k  , null, null, null, null, null, null, null},  // row 0
    /* row 1 */ {null, null, null, null, null, null, null, null},  // row 1
    /* row 2 */ {null, null,  p4 , null,  P4 , null,  b1 ,  P6 },  // row 2
    /* row 3 */ { p2 , null,  p3 , null, null, null,  P5 ,  P7 },  // row 3
    /* row 4 */ { p1 , null, null,  N3 , null,  p6 , null,  N4 },  // row 4
    /* row 5 */ { P1 ,  N2 , null,  p5 , null,  R2 , null,  P8 },  // row 5
    /* row 6 */ { R1 ,  P2 , null, null,  P3 , null, null, null},  // row 6
    /* row 7 */ { N1 , null, null, null, null, null, null,  K  },  // row 7
    // columns :  0     1     2     3     4     5     6     7
        };
        setUpGame(pieces);
        gameState.setEnPassantSquare(new Square(2, 0));
        
        Square[] whiteKnightSquares = {new Square(7, 0), new Square(5, 1), new Square(4, 3), new Square(4, 7)};
        Move[] allLegalKnightMoves = {
                new Move(new Square(7, 0), new Square(6, 2)),

                new Move(new Square(5, 1), new Square(3, 0)),
                new Move(new Square(5, 1), new Square(3, 2)),
                new Move(new Square(5, 1), new Square(6, 3)),
                new Move(new Square(5, 1), new Square(7, 2)),
                
                new Move(new Square(4, 3), new Square(6, 2)),
                new Move(new Square(4, 3), new Square(3, 1)),
                new Move(new Square(4, 3), new Square(2, 2)),
                new Move(new Square(4, 3), new Square(3, 5)),
                
                new Move(new Square(4, 7), new Square(6, 6)),
                new Move(new Square(4, 7), new Square(3, 5)),
                new Move(new Square(4, 7), new Square(2, 6)),
        };
        testAllLegalMovesFrom(whiteKnightSquares, allLegalKnightMoves);
    }
    
    @Test
    public void testBishopMovement() {
        ChessPiece[][] pieces = {
    // columns :  0     1     2     3     4     5     6     7
    /* row 0 */ {null, null, null, null,  k  , null, null, null},  // row 0
    /* row 1 */ { p1 , null, null,  p2 , null, null, null, null},  // row 1
    /* row 2 */ {null, null, null, null, null,  p3 , null, null},  // row 2
    /* row 3 */ {null, null, null, null, null, null,  p4 , null},  // row 3
    /* row 4 */ { B1 , null, null,  B2 , null, null, null, null},  // row 4
    /* row 5 */ {null,  P2 , null, null, null, null, null, null},  // row 5
    /* row 6 */ {null,  P1 , null, null, null,  P3 , null, null},  // row 6
    /* row 7 */ { R1 , null, null, null, null, null, null,  K  },  // row 7
    // columns :  0     1     2     3     4     5     6     7
        };
        setUpGame(pieces);
        gameState.setEnPassantSquare(new Square(2, 6));
        
        Square[] whiteBishopSquares = {new Square(4, 0), new Square(4, 3)};
        Move[] allLegalBishopMoves = {
                new Move(new Square(4, 0), new Square(3, 1)),
                new Move(new Square(4, 0), new Square(2, 2)),
                new Move(new Square(4, 0), new Square(1, 3)),
                
                new Move(new Square(4, 3), new Square(5, 2)),
                new Move(new Square(4, 3), new Square(3, 2)),
                new Move(new Square(4, 3), new Square(2, 1)),
                new Move(new Square(4, 3), new Square(1, 0)),
                new Move(new Square(4, 3), new Square(3, 4)),
                new Move(new Square(4, 3), new Square(2, 5)),
                new Move(new Square(4, 3), new Square(5, 4)),
        };
        testAllLegalMovesFrom(whiteBishopSquares, allLegalBishopMoves);
    }
    
    @Test
    public void testQueenMovement() {
        ChessPiece[][] pieces = {
    // columns :  0     1     2     3     4     5     6     7
    /* row 0 */ { r1 , null, null,  q1 ,  k  , null, null,  r2 },  // row 0
    /* row 1 */ { N1 , null, null,  p2 , null,  p3 ,  p4 , null},  // row 1
    /* row 2 */ { p1 , null, null, null, null, null, null,  p5 },  // row 2
    /* row 3 */ {null, null, null, null, null, null, null,  Q3 },  // row 3
    /* row 4 */ {null, null, null,  Q2 , null, null, null, null},  // row 4
    /* row 5 */ {null, null, null, null, null,  B1 , null, null},  // row 5
    /* row 6 */ {null,  P1 , null, null, null, null, null,  P2 },  // row 6
    /* row 7 */ { Q1 , null, null, null,  K  , null, null,  R1 },  // row 7
    // columns :  0     1     2     3     4     5     6     7
        };
        setUpGame(pieces);
        
        Square[] whiteQueenSquares = {new Square(7, 0), new Square(4, 3), new Square(3, 7)};
        Move[] allLegalQueenMoves = {
                new Move(new Square(7, 0), new Square(6, 0)),
                new Move(new Square(7, 0), new Square(5, 0)),
                new Move(new Square(7, 0), new Square(4, 0)),
                new Move(new Square(7, 0), new Square(3, 0)),
                new Move(new Square(7, 0), new Square(2, 0)),
                new Move(new Square(7, 0), new Square(7, 1)),
                new Move(new Square(7, 0), new Square(7, 2)),
                new Move(new Square(7, 0), new Square(7, 3)),
                
                new Move(new Square(4, 3), new Square(4, 0)),
                new Move(new Square(4, 3), new Square(4, 1)),
                new Move(new Square(4, 3), new Square(4, 2)),
                new Move(new Square(4, 3), new Square(4, 4)),
                new Move(new Square(4, 3), new Square(4, 5)),
                new Move(new Square(4, 3), new Square(4, 6)),
                new Move(new Square(4, 3), new Square(4, 7)),
                new Move(new Square(4, 3), new Square(7, 3)),
                new Move(new Square(4, 3), new Square(6, 3)),
                new Move(new Square(4, 3), new Square(5, 3)),
                new Move(new Square(4, 3), new Square(3, 3)),
                new Move(new Square(4, 3), new Square(2, 3)),
                new Move(new Square(4, 3), new Square(1, 3)),
                new Move(new Square(4, 3), new Square(5, 2)),
                new Move(new Square(4, 3), new Square(3, 4)),
                new Move(new Square(4, 3), new Square(2, 5)),
                new Move(new Square(4, 3), new Square(1, 6)),
                new Move(new Square(4, 3), new Square(3, 2)),
                new Move(new Square(4, 3), new Square(2, 1)),
                new Move(new Square(4, 3), new Square(5, 4)),
                new Move(new Square(4, 3), new Square(6, 5)),
                new Move(new Square(4, 3), new Square(7, 6)),

                new Move(new Square(3, 7), new Square(3, 0)),
                new Move(new Square(3, 7), new Square(3, 1)),
                new Move(new Square(3, 7), new Square(3, 2)),
                new Move(new Square(3, 7), new Square(3, 3)),
                new Move(new Square(3, 7), new Square(3, 4)),
                new Move(new Square(3, 7), new Square(3, 5)),
                new Move(new Square(3, 7), new Square(3, 6)),
                new Move(new Square(3, 7), new Square(5, 7)),
                new Move(new Square(3, 7), new Square(4, 7)),
                new Move(new Square(3, 7), new Square(2, 7)),
                new Move(new Square(3, 7), new Square(4, 6)),
                new Move(new Square(3, 7), new Square(2, 6)),
                new Move(new Square(3, 7), new Square(1, 5)),
        };
        testAllLegalMovesFrom(whiteQueenSquares, allLegalQueenMoves);
    }
    
    @Test
    public void testKingMovement0() {
        ChessPiece[][] pieces = {
    // columns :  0     1     2     3     4     5     6     7
    /* row 0 */ { k  , null, null, null, null, null, null, null},  // row 0
    /* row 1 */ {null, null, null, null, null, null, null, null},  // row 1
    /* row 2 */ {null, null, null, null, null, null, null, null},  // row 2
    /* row 3 */ {null, null, null, null, null,  p1 , null, null},  // row 3
    /* row 4 */ {null, null, null, null, null, null, null, null},  // row 4
    /* row 5 */ {null, null, null, null, null, null, null, null},  // row 5
    /* row 6 */ {null,  P1 , null, null, null, null, null, null},  // row 6
    /* row 7 */ { K  , null, null, null, null, null, null, null},  // row 7
    // columns :  0     1     2     3     4     5     6     7
        };
        setUpGame(pieces);
        gameState.setEnPassantSquare(new Square(2, 5));
        
        Square[] whiteKingSquares = {new Square(7, 0)};
        Move[] allLegalKingMoves = {
                new Move(new Square(7, 0), new Square(6, 0)),
                new Move(new Square(7, 0), new Square(7, 1)),
        };
        testAllLegalMovesFrom(whiteKingSquares, allLegalKingMoves);
    }

    @Test
    public void testKingMovement1() {
        ChessPiece[][] pieces = {
    // columns :  0     1     2     3     4     5     6     7
    /* row 0 */ { k  , null, null, null, null, null, null, null},  // row 0
    /* row 1 */ {null, null, null, null, null, null, null, null},  // row 1
    /* row 2 */ {null, null, null,  n1 ,  n2 , null, null, null},  // row 2
    /* row 3 */ {null, null,  P1 ,  K  , null, null, null,  p1 },  // row 3
    /* row 4 */ {null, null,  P2 ,  P3 ,  P4 , null, null, null},  // row 4
    /* row 5 */ {null, null, null, null, null, null, null, null},  // row 5
    /* row 6 */ {null, null, null, null, null, null, null, null},  // row 6
    /* row 7 */ {null, null, null, null, null, null, null, null},  // row 7
    // columns :  0     1     2     3     4     5     6     7
        };
        setUpGame(pieces);
        gameState.setEnPassantSquare(new Square(2, 7));
        
        Square[] whiteKingSquares = {new Square(3, 3)};
        Move[] allLegalKingMoves = {
                new Move(new Square(3, 3), new Square(3, 4)),
                new Move(new Square(3, 3), new Square(2, 2)),
                new Move(new Square(3, 3), new Square(2, 3)),
                new Move(new Square(3, 3), new Square(2, 4)),
        };
        testAllLegalMovesFrom(whiteKingSquares, allLegalKingMoves);
    }
    
    @Test
    public void testKingMovement2() {
        ChessPiece[][] pieces = {
    // columns :  0     1     2     3     4     5     6     7
    /* row 0 */ { k  , null, null, null, null, null, null, null},  // row 0
    /* row 1 */ {null, null, null, null, null, null, null, null},  // row 1
    /* row 2 */ {null, null, null, null, null, null, null, null},  // row 2
    /* row 3 */ {null, null, null, null, null, null,  b1 ,  P1 },  // row 3
    /* row 4 */ {null, null, null, null, null, null, null,  K  },  // row 4
    /* row 5 */ {null, null, null, null, null, null, null, null},  // row 5
    /* row 6 */ {null, null, null, null, null, null, null, null},  // row 6
    /* row 7 */ {null, null, null, null, null, null, null, null},  // row 7
    // columns :  0     1     2     3     4     5     6     7
        };
        setUpGame(pieces);
        
        Square[] whiteKingSquares = {new Square(4, 7)};
        Move[] allLegalKingMoves = {
                new Move(new Square(4, 7), new Square(5, 6)),
                new Move(new Square(4, 7), new Square(5, 7)),
                new Move(new Square(4, 7), new Square(4, 6)),
                new Move(new Square(4, 7), new Square(3, 6)),
        };
        testAllLegalMovesFrom(whiteKingSquares, allLegalKingMoves);
    }

    @Test
    public void testWhitePawnMovement() {
        ChessPiece[][] pieces = {
    // columns :  0     1     2     3     4     5     6     7
    /* row 0 */ {null, null, null, null,  k  , null, null,  r2 },  // row 0
    /* row 1 */ {null,  p2 ,  p3 ,  p4 , null, null,  P9 , null},  // row 1
    /* row 2 */ { r1 , null, null, null, null, null, null, null},  // row 2
    /* row 3 */ {null,  n1 , null, null,  P5 ,  p6 , null, null},  // row 3
    /* row 4 */ {null, null,  P3 ,  N1 ,  p5 , null,  p7 , null},  // row 4
    /* row 5 */ { P1 , null, null, null,  n2 ,  N2 , null,  p8 },  // row 5
    /* row 6 */ { p1 ,  P2 , null,  P4 , null,  P6 ,  P7 ,  P8 },  // row 6
    /* row 7 */ {null,  R1 , null, null,  K  , null, null, null},  // row 7
    // columns :  0     1     2     3     4     5     6     7
        };
        setUpGame(pieces);
        
        Square[] whitePawnSquares = {new Square(5, 0), new Square(6, 1), new Square(4, 2),
                new Square(6, 3), new Square(3, 4), new Square(6, 5), new Square(6, 6), new Square(1, 6)};
        Move[] allLegalWhitePawnMoves = {
                new Move(new Square(5, 0), new Square(4, 0)),
                
                new Move(new Square(6, 1), new Square(5, 1)),
                new Move(new Square(6, 1), new Square(4, 1)),
                
                new Move(new Square(4, 2), new Square(3, 1)),
                new Move(new Square(4, 2), new Square(3, 2)),
                
                new Move(new Square(6, 3), new Square(5, 3)),
                new Move(new Square(6, 3), new Square(5, 4)),
                
                new Move(new Square(3, 4), new Square(2, 4)),
                
                new Move(new Square(6, 5), new Square(5, 4)),
                
                new Move(new Square(6, 6), new Square(5, 6)),
                new Move(new Square(6, 6), new Square(5, 7)),
                
                new PromotionMove(new Square(1, 6), new Square(0, 6), Queen.class),
                new PromotionMove(new Square(1, 6), new Square(0, 6), Rook.class),
                new PromotionMove(new Square(1, 6), new Square(0, 6), Bishop.class),
                new PromotionMove(new Square(1, 6), new Square(0, 6), Knight.class),
                new PromotionMove(new Square(1, 6), new Square(0, 7), Queen.class),
                new PromotionMove(new Square(1, 6), new Square(0, 7), Rook.class),
                new PromotionMove(new Square(1, 6), new Square(0, 7), Bishop.class),
                new PromotionMove(new Square(1, 6), new Square(0, 7), Knight.class),
        };
        testAllLegalMovesFrom(whitePawnSquares, allLegalWhitePawnMoves);
    }

    @Test
    public void testBlackPawnMovement() {
        ChessPiece[][] pieces = {
    // columns :  0     1     2     3     4     5     6     7
    /* row 0 */ {null, null, null, null,  k  , null, null,  r2 },  // row 0
    /* row 1 */ {null,  p2 ,  p3 ,  p4 , null, null,  P9 , null},  // row 1
    /* row 2 */ { r1 , null, null, null, null, null, null, null},  // row 2
    /* row 3 */ {null,  n1 , null, null,  P5 ,  p6 , null, null},  // row 3
    /* row 4 */ {null, null,  P3 ,  N1 ,  p5 , null,  p7 , null},  // row 4
    /* row 5 */ { P1 , null, null, null,  n2 ,  N2 , null,  p8 },  // row 5
    /* row 6 */ { p1 ,  P2 , null,  P4 , null,  P6 ,  P7 ,  P8 },  // row 6
    /* row 7 */ {null,  R1 , null, null,  K  , null, null, null},  // row 7
    // columns :  0     1     2     3     4     5     6     7
        };
        setUpGame(pieces);
        gameState.setColorToMove(PieceColor.BLACK);
        
        Square[] blackPawnSquares = {new Square(1, 1), new Square(1, 2), new Square(1, 3),
                new Square(4, 4), new Square(3, 5), new Square(4, 6), new Square(5, 7),
                new Square(6, 0)};
        Move[] allLegalBlackPawnMoves = {
                new Move(new Square(1, 1), new Square(2, 1)),
                
                new Move(new Square(1, 2), new Square(2, 2)),
                new Move(new Square(1, 2), new Square(3, 2)),
                
                new Move(new Square(1, 3), new Square(2, 3)),
                new Move(new Square(1, 3), new Square(3, 3)),
                
                new Move(new Square(4, 4), new Square(5, 5)),
                
                new Move(new Square(3, 5), new Square(4, 5)),
                
                new Move(new Square(4, 6), new Square(5, 5)),
                new Move(new Square(4, 6), new Square(5, 6)),
                
                new Move(new Square(5, 7), new Square(6, 6)),
                
                new PromotionMove(new Square(6, 0), new Square(7, 0), Queen.class),
                new PromotionMove(new Square(6, 0), new Square(7, 0), Rook.class),
                new PromotionMove(new Square(6, 0), new Square(7, 0), Bishop.class),
                new PromotionMove(new Square(6, 0), new Square(7, 0), Knight.class),
                new PromotionMove(new Square(6, 0), new Square(7, 1), Queen.class),
                new PromotionMove(new Square(6, 0), new Square(7, 1), Rook.class),
                new PromotionMove(new Square(6, 0), new Square(7, 1), Bishop.class),
                new PromotionMove(new Square(6, 0), new Square(7, 1), Knight.class),
        };
        testAllLegalMovesFrom(blackPawnSquares, allLegalBlackPawnMoves);
    }
    
    @Test
    public void testEnPassantMovement() {
        ChessPiece[][] pieces = {
    // columns :  0     1     2     3     4     5     6     7
    /* row 0 */ {null, null, null, null,  k  , null, null, null},  // row 0
    /* row 1 */ {null, null, null,  p2 , null, null, null, null},  // row 1
    /* row 2 */ {null, null, null, null, null,  p3 , null, null},  // row 2
    /* row 3 */ {null, null,  P2 , null,  P3 , null, null, null},  // row 3
    /* row 4 */ { p1 , null, null, null, null, null,  p4 , null},  // row 4
    /* row 5 */ {null, null, null, null, null, null, null, null},  // row 5
    /* row 6 */ {null,  P1 , null, null, null,  P4 , null, null},  // row 6
    /* row 7 */ {null, null, null, null,  K  , null, null, null},  // row 7
    // columns :  0     1     2     3     4     5     6     7
        };
        setUpGame(pieces);

        game.move(new Move(new Square(6, 1), new Square(4, 1)));
        Move enPassantMove0 = new Move(new Square(4, 0), new Square(5, 1));
        assertLegalMove(enPassantMove0);
        Assert.assertTrue(ArrayUtils.contains(game.getLegalDestinationsFrom(new Square(4, 0)), new Square(5, 1)));
        
        game.move(new Move(new Square(1, 3), new Square(3, 3)));
        Move enPassantMove1 = new Move(new Square(3, 2), new Square(2, 3));
        Move enPassantMove2 = new Move(new Square(3, 4), new Square(2, 3));
        assertLegalMove(enPassantMove1);
        assertLegalMove(enPassantMove2);
        Assert.assertTrue(ArrayUtils.contains(game.getLegalDestinationsFrom(new Square(3, 2)), new Square(2, 3)));
        Assert.assertTrue(ArrayUtils.contains(game.getLegalDestinationsFrom(new Square(3, 4)), new Square(2, 3)));

        game.move(enPassantMove1);
        Assert.assertNull(board.getPiece(3, 3), "En passant captured piece has not been removed from the board.");
        assertIllegalMove(enPassantMove0);  // too late for en passant capture
        Assert.assertFalse(ArrayUtils.contains(game.getLegalDestinationsFrom(new Square(4, 0)), new Square(5, 1)));
        
        game.move(new Move(new Square(2, 5), new Square(3, 5)));
        Move invalidEnPassantMove = new Move(new Square(3, 4), new Square(2, 5));
        assertIllegalMove(invalidEnPassantMove);  // invalid en passant capture
        Assert.assertFalse(ArrayUtils.contains(game.getLegalDestinationsFrom(new Square(3, 4)), new Square(2, 5)));
        
        game.move(new Move(new Square(6, 5), new Square(4, 5)));
        Move enPassantMove3 = new Move(new Square(4, 6), new Square(5, 5));
        assertLegalMove(enPassantMove3);
        Assert.assertTrue(ArrayUtils.contains(game.getLegalDestinationsFrom(new Square(4, 6)), new Square(5, 5)));
        
        game.move(enPassantMove3);
        Assert.assertNull(board.getPiece(4, 5), "En passant captured piece has not been removed from the board.");
    }

    @Test
    public void testIsKingSafeAfterMovement0() {
        ChessPiece[][] pieces = {
    // columns :  0     1     2     3     4     5     6     7
    /* row 0 */ {null, null,  b2 , null, null, null, null, null},  // row 0
    /* row 1 */ {null, null, null,  N1 , null,  p3 ,  k  , null},  // row 1
    /* row 2 */ {null, null, null,  p2 , null, null,  n1 , null},  // row 2
    /* row 3 */ { r1 ,  p1 ,  P2 , null, null,  K  , null, null},  // row 3
    /* row 4 */ {null, null, null, null, null,  R1 , null, null},  // row 4
    /* row 5 */ {null,  q1 , null, null, null, null, null, null},  // row 5
    /* row 6 */ {null, null,  P1 , null, null, null, null, null},  // row 6
    /* row 7 */ {null,  b1 , null, null, null,  r2 , null, null},  // row 7
    // columns :  0     1     2     3     4     5     6     7
        };
        setUpGame(pieces);
        gameState.setEnPassantSquare(new Square(2, 1));
        
        Square[] whitePieceSquares = {new Square(6, 2), new Square(3, 2), new Square(1, 3),
                new Square(4, 5), new Square(3, 5)};
        Move[] allLegalMoves = {
                new Move(new Square(3, 2), new Square(2, 2)),
                new Move(new Square(3, 2), new Square(2, 3)),
                
                new Move(new Square(4, 5), new Square(5, 5)),
                new Move(new Square(4, 5), new Square(6, 5)),
                new Move(new Square(4, 5), new Square(7, 5)),
                
                new Move(new Square(3, 5), new Square(3, 6)),
                new Move(new Square(3, 5), new Square(4, 4)),
                new Move(new Square(3, 5), new Square(4, 6)),
        };
        testAllLegalMovesFrom(whitePieceSquares, allLegalMoves);
    }

    @Test
    public void testIsKingSafeAfterMovement1() {
        ChessPiece[][] pieces = {
    // columns :  0     1     2     3     4     5     6     7
    /* row 0 */ { B1 , null,  n1 , null, null, null, null,  r2 },  // row 0
    /* row 1 */ {null, null,  r1 , null, null, null, null, null},  // row 1
    /* row 2 */ {null, null, null, null, null, null, null,  p1 },  // row 2
    /* row 3 */ {null, null,  N1 ,  q1 , null, null,  P2 , null},  // row 3
    /* row 4 */ { Q1 ,  b1 , null, null,  k  ,  P1 ,  K  , null},  // row 4
    /* row 5 */ {null, null, null, null, null, null, null, null},  // row 5
    /* row 6 */ {null, null, null, null, null, null, null, null},  // row 6
    /* row 7 */ {null, null, null, null, null,  b2 , null, null},  // row 7
    // columns :  0     1     2     3     4     5     6     7
        };
        setUpGame(pieces);
        gameState.setColorToMove(PieceColor.BLACK);
        
        Square[] blackPieceSquares = {new Square(4, 1), new Square(1, 2), new Square(0, 2),
                new Square(3, 3), new Square(4, 4), new Square(7, 5), new Square(2, 7), new Square(0, 7)};
        Move[] allLegalMoves = {
                new Move(new Square(1, 2), new Square(3, 2)),
                
                new Move(new Square(4, 4), new Square(5, 4)),
                new Move(new Square(4, 4), new Square(4, 3)),
        };
        testAllLegalMovesFrom(blackPieceSquares, allLegalMoves);
    }
    
    @Test
    public void testBasicCastleMovement() {
        ChessPiece[][] pieces = {
    // columns :  0     1     2     3     4     5     6     7
    /* row 0 */ { r1 , null, null, null,  k  , null, null,  r2 },  // row 0
    /* row 1 */ {null, null, null, null, null, null, null, null},  // row 1
    /* row 2 */ {null, null, null, null, null, null, null, null},  // row 2
    /* row 3 */ {null, null,  p1 , null, null, null, null, null},  // row 3
    /* row 4 */ {null, null, null, null, null, null, null, null},  // row 4
    /* row 5 */ {null, null, null, null, null, null, null, null},  // row 5
    /* row 6 */ {null, null, null, null, null, null, null, null},  // row 6
    /* row 7 */ { R1 , null, null, null,  K  , null, null,  R2 },  // row 7
    // columns :  0     1     2     3     4     5     6     7
        };
        r1.setMoved(false);
        r2.setMoved(false);
        k.setMoved(false);
        R1.setMoved(false);
        R2.setMoved(false);
        K.setMoved(false);
        setUpGame(pieces);
        gameState.setEnPassantSquare(new Square(2, 2));
        
        Square[] whiteKingSquares = {new Square(7, 4)};
        Move[] allLegalWhiteKingMoves = {
                new Move(new Square(7, 4), new Square(7, 2)),
                new Move(new Square(7, 4), new Square(7, 3)),
                new Move(new Square(7, 4), new Square(6, 3)),
                new Move(new Square(7, 4), new Square(6, 4)),
                new Move(new Square(7, 4), new Square(6, 5)),
                new Move(new Square(7, 4), new Square(7, 5)),
                new Move(new Square(7, 4), new Square(7, 6)),
        };
        testAllLegalMovesFrom(whiteKingSquares, allLegalWhiteKingMoves);
        
        game.move(new Move(new Square(7, 4), new Square(6, 4)));

        Square[] blackKingSquares = {new Square(0, 4)};
        Move[] allLegalBlackKingMoves = {
                new Move(new Square(0, 4), new Square(0, 2)),
                new Move(new Square(0, 4), new Square(0, 3)),
                new Move(new Square(0, 4), new Square(1, 3)),
                new Move(new Square(0, 4), new Square(1, 4)),
                new Move(new Square(0, 4), new Square(1, 5)),
                new Move(new Square(0, 4), new Square(0, 5)),
                new Move(new Square(0, 4), new Square(0, 6)),
        };
        testAllLegalMovesFrom(blackKingSquares, allLegalBlackKingMoves);
        
        game.move(new Move(new Square(0, 7), new Square(6, 7)));
        game.move(new Move(new Square(6, 4), new Square(7, 4)));
        game.move(new Move(new Square(6, 7), new Square(0, 7)));
        
        whiteKingSquares = new Square[] {new Square(7, 4)};
        allLegalWhiteKingMoves = new Move[] {
                new Move(new Square(7, 4), new Square(7, 3)),
                new Move(new Square(7, 4), new Square(6, 3)),
                new Move(new Square(7, 4), new Square(6, 4)),
                new Move(new Square(7, 4), new Square(6, 5)),
                new Move(new Square(7, 4), new Square(7, 5)),
        };
        testAllLegalMovesFrom(whiteKingSquares, allLegalWhiteKingMoves);
        
        game.move(new Move(new Square(7, 4), new Square(6, 3)));
        
        blackKingSquares = new Square[] {new Square(0, 4)};
        allLegalBlackKingMoves = new Move[] {
                new Move(new Square(0, 4), new Square(0, 2)),
                new Move(new Square(0, 4), new Square(0, 3)),
                new Move(new Square(0, 4), new Square(1, 3)),
                new Move(new Square(0, 4), new Square(1, 4)),
                new Move(new Square(0, 4), new Square(1, 5)),
                new Move(new Square(0, 4), new Square(0, 5)),
        };
        testAllLegalMovesFrom(blackKingSquares, allLegalBlackKingMoves);
        
        game.move(new Move(new Square(0, 4), new Square(0, 2)));
        Assert.assertNull(board.getPiece(0, 0));
        Assert.assertSame(board.getPiece(0, 3), r1);
    }
    
    @Test
    public void testCantCastleInCheck() {
        ChessPiece[][] pieces = {
    // columns :  0     1     2     3     4     5     6     7
    /* row 0 */ {null, null, null, null,  k  , null, null, null},  // row 0
    /* row 1 */ {null, null, null, null, null, null, null, null},  // row 1
    /* row 2 */ {null, null, null, null, null, null, null, null},  // row 2
    /* row 3 */ {null, null, null, null, null, null, null, null},  // row 3
    /* row 4 */ {null, null, null, null, null, null, null, null},  // row 4
    /* row 5 */ {null, null, null,  n1 , null, null, null, null},  // row 5
    /* row 6 */ {null, null, null, null, null, null, null, null},  // row 6
    /* row 7 */ { R1 , null, null, null,  K  , null, null,  R2 },  // row 7
    // columns :  0     1     2     3     4     5     6     7
        };
        R1.setMoved(false);
        R2.setMoved(false);
        K.setMoved(false);
        setUpGame(pieces);
        
        Square[] whiteKingSquares = {new Square(7, 4)};
        Move[] allLegalWhiteKingMoves = {
                new Move(new Square(7, 4), new Square(7, 3)),
                new Move(new Square(7, 4), new Square(6, 3)),
                new Move(new Square(7, 4), new Square(6, 4)),
                new Move(new Square(7, 4), new Square(7, 5)),
        };
        testAllLegalMovesFrom(whiteKingSquares, allLegalWhiteKingMoves);
    }

    @Test
    public void testCantCastleWithBlockingPiece() {
        ChessPiece[][] pieces =  {
    // columns :  0     1     2     3     4     5     6     7
    /* row 0 */ { r1 , null,  b1 , null,  k  ,  b2 , null,  r2 },  // row 0
    /* row 1 */ {null, null, null, null, null, null, null, null},  // row 1
    /* row 2 */ {null, null, null, null, null, null, null, null},  // row 2
    /* row 3 */ {null, null, null, null, null, null, null, null},  // row 3
    /* row 4 */ {null, null, null, null, null, null, null, null},  // row 4
    /* row 5 */ {null, null, null, null, null, null, null, null},  // row 5
    /* row 6 */ {null, null, null, null, null, null, null, null},  // row 6
    /* row 7 */ { R1 ,  n1 , null, null,  K  , null,  N1 ,  R2 },  // row 7
    // columns :  0     1     2     3     4     5     6     7
        };
        r1.setMoved(false);
        r2.setMoved(false);
        k.setMoved(false);
        R1.setMoved(false);
        R2.setMoved(false);
        K.setMoved(false);
        setUpGame(pieces);
        
        Square[] whiteKingSquares = {new Square(7, 4)};
        Move[] allLegalWhiteKingMoves = {
                new Move(new Square(7, 4), new Square(7, 3)),
                new Move(new Square(7, 4), new Square(6, 4)),
                new Move(new Square(7, 4), new Square(6, 5)),
                new Move(new Square(7, 4), new Square(7, 5)),
        };
        testAllLegalMovesFrom(whiteKingSquares, allLegalWhiteKingMoves);
        
        game.move(new Move(new Square(7, 6), new Square(5, 5)));

        Square[] blackKingSquares = {new Square(0, 4)};
        Move[] allLegalBlackKingMoves = {
                new Move(new Square(0, 4), new Square(0, 3)),
                new Move(new Square(0, 4), new Square(1, 3)),
                new Move(new Square(0, 4), new Square(1, 4)),
                new Move(new Square(0, 4), new Square(1, 5)),
        };
        testAllLegalMovesFrom(blackKingSquares, allLegalBlackKingMoves);
        
        game.move(new Move(new Square(0, 2), new Square(1, 3)));
        
        game.move(new Move(new Square(7, 4), new Square(7, 6)));
        Assert.assertNull(board.getPiece(7, 7));
        Assert.assertSame(board.getPiece(7, 5), R2);
        
        game.move(new Move(new Square(0, 4), new Square(0, 2)));
        Assert.assertNull(board.getPiece(0, 0));
        Assert.assertSame(board.getPiece(0, 3), r1);
    }

    @Test
    public void testCantCastleThroughAttackedSquares() {
        ChessPiece[][] pieces = {
    // columns :  0     1     2     3     4     5     6     7
    /* row 0 */ { r1 , null, null, null,  k  , null, null,  r2 },  // row 0
    /* row 1 */ {null, null, null, null, null, null, null, null},  // row 1
    /* row 2 */ {null, null, null, null, null, null, null, null},  // row 2
    /* row 3 */ {null, null, null, null, null, null, null, null},  // row 3
    /* row 4 */ {null, null,  Q1 , null, null, null, null, null},  // row 4
    /* row 5 */ {null, null, null, null, null, null, null, null},  // row 5
    /* row 6 */ {null, null, null, null,  p1 , null, null, null},  // row 6
    /* row 7 */ { R1 , null, null, null,  K  , null, null,  R2 },  // row 7
    // columns :  0     1     2     3     4     5     6     7
        };
        r1.setMoved(false);
        r2.setMoved(false);
        k.setMoved(false);
        R1.setMoved(false);
        R2.setMoved(false);
        K.setMoved(false);
        setUpGame(pieces);
        
        Square[] whiteKingSquares = {new Square(7, 4)};
        Move[] allLegalWhiteKingMoves = {
                new Move(new Square(7, 4), new Square(6, 3)),
                new Move(new Square(7, 4), new Square(6, 4)),
                new Move(new Square(7, 4), new Square(6, 5)),
        };
        testAllLegalMovesFrom(whiteKingSquares, allLegalWhiteKingMoves);
        
        gameState.setColorToMove(PieceColor.BLACK);

        Square[] blackKingSquares = {new Square(0, 4)};
        Move[] allLegalBlackKingMoves = {
                new Move(new Square(0, 4), new Square(0, 3)),
                new Move(new Square(0, 4), new Square(1, 3)),
                new Move(new Square(0, 4), new Square(1, 4)),
                new Move(new Square(0, 4), new Square(0, 5)),
        };
        testAllLegalMovesFrom(blackKingSquares, allLegalBlackKingMoves);
    }

    @Test
    public void testGetLegalMovesTo() {
        ChessPiece[][] pieces = {
    // columns :  0     1     2     3     4     5     6     7
    /* row 0 */ { r1 ,  n1 , null, null,  k  , null, null,  r2 },  // row 0
    /* row 1 */ { p1 ,  p2 , null, null,  p5 , null,  P7 ,  p8 },  // row 1
    /* row 2 */ {null, null, null,  p4 ,  q1 , null,  p7 , null},  // row 2
    /* row 3 */ { b1 , null,  p3 ,  P4 , null,  p6 ,  N2 , null},  // row 3
    /* row 4 */ {null, null, null, null,  P5 , null, null, null},  // row 4
    /* row 5 */ {null, null, null, null, null,  N1 , null, null},  // row 5
    /* row 6 */ { P1 ,  P2 ,  P3 ,  Q1 , null,  P6 , null,  P8 },  // row 6
    /* row 7 */ { R1 , null, null, null,  K  , null, null,  R2 },  // row 7
    // columns :  0     1     2     3     4     5     6     7
        };
        r1.setMoved(false);
        r2.setMoved(false);
        k.setMoved(false);
        R1.setMoved(false);
        R2.setMoved(false);
        K.setMoved(false);
        setUpGame(pieces);
        gameState.setEnPassantSquare(new Square(2, 2));
        
        Square destination = new Square(7, 6);
        Move[] legalMovesTo = {
                new Move(new Square(7, 4), new Square(7, 6)),
                new Move(new Square(5, 5), new Square(7, 6)),
                new Move(new Square(7, 7), new Square(7, 6)),
        };
        testAllLegalMovesTo(destination, legalMovesTo);
        
        destination = new Square(7, 2);
        legalMovesTo = new Move[] {
                new Move(new Square(7, 4), new Square(7, 2)),
                new Move(new Square(7, 0), new Square(7, 2)),
        };
        testAllLegalMovesTo(destination, legalMovesTo);
        
        destination = new Square(2, 2);
        legalMovesTo = new Move[] {
                new Move(new Square(3, 3), new Square(2, 2)),
        };
        testAllLegalMovesTo(destination, legalMovesTo);
        
        destination = new Square(2, 4);
        legalMovesTo = new Move[] {
                new Move(new Square(3, 3), new Square(2, 4)),
                new Move(new Square(3, 6), new Square(2, 4)),
        };
        testAllLegalMovesTo(destination, legalMovesTo);
        
        destination = new Square(3, 5);
        legalMovesTo = new Move[0];
        testAllLegalMovesTo(destination, legalMovesTo);
        
        destination = new Square(0, 7);
        legalMovesTo = new Move[] {
                new PromotionMove(new Square(1, 6), new Square(0, 7), Queen.class),
                new PromotionMove(new Square(1, 6), new Square(0, 7), Rook.class),
                new PromotionMove(new Square(1, 6), new Square(0, 7), Bishop.class),
                new PromotionMove(new Square(1, 6), new Square(0, 7), Knight.class),
        };
        testAllLegalMovesTo(destination, legalMovesTo);
        
        gameState.setColorToMove(PieceColor.BLACK);
        gameState.setEnPassantSquare(null);
        
        destination = new Square(0, 6);
        legalMovesTo = new Move[] {
                new Move(new Square(0, 7), new Square(0, 6)),
                new Move(new Square(2, 4), new Square(0, 6)),
        };
        testAllLegalMovesTo(destination, legalMovesTo);
        
        destination = new Square(0, 2);
        legalMovesTo = new Move[] {
                new Move(new Square(2, 4), new Square(0, 2)),
        };
        testAllLegalMovesTo(destination, legalMovesTo);
        
        destination = new Square(1, 5);
        legalMovesTo = new Move[] {
                new Move(new Square(2, 4), new Square(1, 5)),
        };
        testAllLegalMovesTo(destination, legalMovesTo);

        destination = new Square(0, 3);
        legalMovesTo = new Move[] {
                new Move(new Square(0, 4), new Square(0, 3)),
                new Move(new Square(3, 0), new Square(0, 3)),
        };
        testAllLegalMovesTo(destination, legalMovesTo);
    }

    private void assertLegalMove(Move legalMove) {
        String errorMessage = "Expected this move to be legal: " + legalMove.toString() + "."; 
        Assert.assertTrue(game.isLegal(legalMove), errorMessage);
    }
    
    private void assertLegalMoves(Move[] legalMoves) {
        for(Move legalMove : legalMoves) {
            assertLegalMove(legalMove);
        }
    }

    private void assertIllegalMove(Move illegalMove) {
        String errorMessage = "Expected this move to be illegal: " + illegalMove.toString() + "."; 
        Assert.assertFalse(game.isLegal(illegalMove), errorMessage);
    }
    
    private void assertIllegalMoves(Move[] illegalMoves) {
        for(Move illegalMove : illegalMoves) {
            assertIllegalMove(illegalMove);
        }
    }

    private void assertMoveAndUnmove(Move[] moves) {
        for(Move move : moves) {
            ChessGameState originalGameState = gameState.clone();
            
            ChessPiece piece = (ChessPiece) board.getPiece(move.getOrigin());
            piece.move(move);
            Assert.assertNull(board.getPiece(move.getOrigin()));
            Assert.assertNotNull(board.getPiece(move.getDestination()));
            
            // the moved piece will not necessarily be the same as the moving piece due to promotion
            ChessPiece movedPiece = (ChessPiece) board.getPiece(move.getDestination());
            movedPiece.unmove(move);
            Assert.assertEquals(gameState, originalGameState);
        }
    }
    
    private Move[] getAllIllegalMovesFrom(Square[] originSquares, Move[] allLegalMoves) {
        List<Move> allIllegalMoves = new ArrayList<Move>();
        for(Square origin : originSquares) {
            for(Square destination : board) {
                allIllegalMoves.add(new Move(origin, destination));
            }
        }
        for(Move legalMove : allLegalMoves) {
            allIllegalMoves.remove(legalMove);
        }
        return allIllegalMoves.toArray(new Move[0]);
    }

    private Move[] getAllIllegalMovesTo(Square destination, Move[] allLegalMoves) {
        List<Move> allIllegalMoves = new ArrayList<Move>();
        for(Square origin : board) {
            allIllegalMoves.add(new Move(origin, destination));
        }
        for(Move legalMove : allLegalMoves) {
            allIllegalMoves.remove(legalMove);
        }
        return allIllegalMoves.toArray(new Move[0]);
    }
    
    private void assertAllLegalDestinationsFrom(Square[] origins, Move[] allLegalMoves) {
        for(Square origin : origins) {
            Set<Square> allLegalDestinationsFromOrigin = new HashSet<Square>();
            for(Move move : allLegalMoves) {
                if(move.getOrigin().equals(origin)) {
                    allLegalDestinationsFromOrigin.add(move.getDestination());
                }
            }
            HaundrixAssert.areEqualIgnoringOrder(game.getLegalDestinationsFrom(origin),
                    allLegalDestinationsFromOrigin.toArray(new Square[0]));
        }
    }

    private void assertAllLegalOriginsTo(Square destination, Move[] allLegalMoves) {
        Set<Square> allLegalOriginsToDestination = new HashSet<Square>();
        for(Move move : allLegalMoves) {
            allLegalOriginsToDestination.add(move.getOrigin());
        }
        HaundrixAssert.areEqualIgnoringOrder(game.getLegalOriginsTo(destination),
                allLegalOriginsToDestination.toArray(new Square[0]));
    }

    private void testAllLegalMovesFrom(Square[] origins, Move[] allLegalMoves) {
        assertMoveAndUnmove(allLegalMoves);
        assertLegalMoves(allLegalMoves);
        assertIllegalMoves(getAllIllegalMovesFrom(origins, allLegalMoves));
        assertAllLegalDestinationsFrom(origins, allLegalMoves);
    }
    
    private void testAllLegalMovesTo(Square destination, Move[] allLegalMoves) {
        assertMoveAndUnmove(allLegalMoves);
        assertLegalMoves(allLegalMoves);
        assertIllegalMoves(getAllIllegalMovesTo(destination, allLegalMoves));
        assertAllLegalOriginsTo(destination, allLegalMoves);
    }

}
