/*
Copyright (C) 2009, 2010 Alex Ignácio da Silva

This file is part of Haundrix Games.

Haundrix Games is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License version 3 as published by the
Free Software Foundation, with additional permission under section 7 to combine
it with independent works licensed under selected Free/Libre Open Source Software licenses.
Please see the full text of the license that is distributed along with this file.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
*/
package com.haundrix.games.chess.model;

import org.testng.Assert;
import org.testng.annotations.Test;

import com.haundrix.games.model.Move;
import com.haundrix.games.model.Square;

public class ChessGameNavigationTest {

    @Test
    public void testNavigation() {
        Move[] moves = {
            new Move(new Square(6, 4), new Square(4, 4)),  // e4
            new Move(new Square(1, 3), new Square(3, 3)),  //            d5
            new Move(new Square(4, 4), new Square(3, 4)),  // e5
            new Move(new Square(1, 5), new Square(3, 5)),  //            f5
            new Move(new Square(3, 4), new Square(2, 5)),  // exf6 e.p.
            new Move(new Square(0, 1), new Square(2, 2)),  //            Nc6
            new Move(new Square(7, 6), new Square(5, 5)),  // Nf3
            new Move(new Square(0, 2), new Square(4, 6)),  //            Bg4
            new Move(new Square(7, 5), new Square(4, 2)),  // Bc4
            new Move(new Square(0, 3), new Square(2, 3)),  //            Qd6
            new Move(new Square(7, 4), new Square(7, 6)),  // 0-0
            new Move(new Square(0, 4), new Square(0, 2)),  //            0-0-0
            new Move(new Square(2, 5), new Square(1, 6)),  // fxg7
            new Move(new Square(3, 3), new Square(4, 2)),  //            dxc4
            new Move(new Square(6, 1), new Square(4, 1)),  // b4
            new Move(new Square(0, 6), new Square(2, 5)),  //            Nf6
            new PromotionMove(new Square(1, 6), new Square(0, 5), Queen.class),  // gxf8Q
            new Move(new Square(0, 7), new Square(0, 5)),  //            Rhxf8
            new Move(new Square(6, 3), new Square(5, 3)),  // d3
            new Move(new Square(2, 3), new Square(5, 3)),  //            Qxd3
            new Move(new Square(7, 1), new Square(6, 3)),  // Nbd2
            new Move(new Square(4, 6), new Square(5, 5)),  //            Bxf3
            new Move(new Square(7, 3), new Square(5, 5)),  // Qxf3
            new Move(new Square(5, 3), new Square(7, 5)),  //            Qxf1+
            new Move(new Square(6, 3), new Square(7, 5)),  // Nxf1
            new Move(new Square(2, 2), new Square(4, 1)),  //            Nxb4
            new Move(new Square(5, 5), new Square(3, 5)),  // Qf5+
            new Move(new Square(0, 2), new Square(0, 1)),  //            Kb8
            new Move(new Square(7, 5), new Square(5, 6)),  // Ng3
            new Move(new Square(0, 5), new Square(0, 4)),  //            Rfe8
            new Move(new Square(5, 6), new Square(4, 4)),  // Ne4
            new Move(new Square(0, 3), new Square(7, 3)),  //            Rd1++
        };
        ChessGameState[] gameStates = new ChessGameState[moves.length + 1];
        
        ChessGame game = new ChessGame();
        for(int i = 0; i < moves.length; i++) {
            gameStates[i] = game.getState();
            game.move(moves[i]);
        }
        gameStates[moves.length] = game.getState();
        
        game.goToBegin();
        Assert.assertEquals(game.getCurrentPosition(), 0);
        Assert.assertEquals(game.getState(), gameStates[0]);
        
        int[] positions = {29, 32, 7, 18, 19, 2, 23, 15, 10, 30, 8, 0, 31, 12, 21};
        for(int position : positions) {
            game.goTo(position);
            Assert.assertEquals(game.getCurrentPosition(), position);
            Assert.assertEquals(game.getState(), gameStates[position]);
        }
        
        game.goToEnd();
        Assert.assertEquals(game.getCurrentPosition(), moves.length);
        Assert.assertEquals(game.getState(), gameStates[moves.length]);
    }
    
}
