/*
Copyright (C) 2009, 2010 Alex Ignácio da Silva

This file is part of Haundrix Games.

Haundrix Games is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License version 3 as published by the
Free Software Foundation, with additional permission under section 7 to combine
it with independent works licensed under selected Free/Libre Open Source Software licenses.
Please see the full text of the license that is distributed along with this file.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
*/
package com.haundrix.games.chess.model;

import org.testng.Assert;
import org.testng.annotations.Test;

import com.haundrix.games.model.GameOutcome;
import com.haundrix.games.model.Move;
import com.haundrix.games.model.Square;
import com.haundrix.games.model.Piece.PieceColor;

public class ChessGameResultTest extends ChessTest {

    @Test
    public void testCheckMate1() {
        ChessPiece[][] pieces = {
    // columns :  0     1     2     3     4     5     6     7
    /* row 0 */ {null, null, null, null,  k  , null, null, null},  // row 0
    /* row 1 */ {null, null, null, null, null, null, null, null},  // row 1
    /* row 2 */ {null, null, null, null, null, null, null, null},  // row 2
    /* row 3 */ {null, null, null, null, null, null, null, null},  // row 3
    /* row 4 */ {null, null, null, null, null, null, null, null},  // row 4
    /* row 5 */ {null, null, null, null, null, null, null, null},  // row 5
    /* row 6 */ {null, null, null, null, null,  P1 ,  P2 ,  P3 },  // row 6
    /* row 7 */ { r1 , null, null, null, null, null,  K  , null},  // row 7
    // columns :  0     1     2     3     4     5     6     7
        };
        setUpGame(pieces);

        testGameResult(new ChessGameResult(GameOutcome.BLACK_WINS, ChessGameResultReason.CHECKMATE));
    }

    @Test
    public void testCheckMate2() {
        ChessPiece[][] pieces = {
    // columns :  0     1     2     3     4     5     6     7
    /* row 0 */ {null, null, null, null,  k  , null, null, null},  // row 0
    /* row 1 */ {null, null, null, null, null, null, null,  Q1 },  // row 1
    /* row 2 */ {null, null, null, null,  K  , null, null, null},  // row 2
    /* row 3 */ {null, null, null, null, null, null, null, null},  // row 3
    /* row 4 */ {null, null, null, null, null, null, null, null},  // row 4
    /* row 5 */ {null, null, null, null, null, null, null, null},  // row 5
    /* row 6 */ {null, null, null, null, null, null, null, null},  // row 6
    /* row 7 */ {null, null, null, null, null, null, null, null},  // row 7
    // columns :  0     1     2     3     4     5     6     7
        };
        setUpGame(pieces);

        testGameResult(null);
        
        game.move(new Move(new Square(1, 7), new Square(1, 4)));
        testGameResult(new ChessGameResult(GameOutcome.WHITE_WINS, ChessGameResultReason.CHECKMATE));
    }
    
    @Test
    public void testStaleMate1() {
        ChessPiece[][] pieces = {
    // columns :  0     1     2     3     4     5     6     7
    /* row 0 */ { k  , null, null, null, null, null, null, null},  // row 0
    /* row 1 */ {null, null, null, null, null, null, null, null},  // row 1
    /* row 2 */ { K  , null, null, null, null, null, null, null},  // row 2
    /* row 3 */ {null, null, null, null, null, null, null, null},  // row 3
    /* row 4 */ {null, null, null, null, null, null, null, null},  // row 4
    /* row 5 */ {null, null, null, null, null, null, null, null},  // row 5
    /* row 6 */ {null, null, null, null, null, null, null, null},  // row 6
    /* row 7 */ {null,  Q1 , null, null, null, null, null, null},  // row 7
    // columns :  0     1     2     3     4     5     6     7
        };
        gameState.setColorToMove(PieceColor.BLACK);
        setUpGame(pieces);

        testGameResult(new ChessGameResult(GameOutcome.DRAW, ChessGameResultReason.STALEMATE));
    }

    @Test
    public void testStaleMate2() {
        ChessPiece[][] pieces = {
    // columns :  0     1     2     3     4     5     6     7
    /* row 0 */ {null, null,  b1 ,  k  , null, null, null, null},  // row 0
    /* row 1 */ {null, null, null,  B1 , null, null, null, null},  // row 1
    /* row 2 */ {null, null, null,  K  , null, null, null, null},  // row 2
    /* row 3 */ {null, null, null, null, null, null, null, null},  // row 3
    /* row 4 */ {null, null, null, null, null, null, null, null},  // row 4
    /* row 5 */ {null, null, null, null, null, null, null, null},  // row 5
    /* row 6 */ {null, null, null, null, null, null, null, null},  // row 6
    /* row 7 */ { Q1 , null, null, null, null, null, null, null},  // row 7
    // columns :  0     1     2     3     4     5     6     7
        };
        setUpGame(pieces);

        testGameResult(null);
        
        game.move(new Move(new Square(7, 0), new Square(0, 0)));
        testGameResult(new ChessGameResult(GameOutcome.DRAW, ChessGameResultReason.STALEMATE));
    }
    
    @Test
    public void testDeadPosition1() {
        ChessPiece[][] pieces = {
    // columns :  0     1     2     3     4     5     6     7
    /* row 0 */ {null, null, null, null, null, null, null, null},  // row 0
    /* row 1 */ {null, null, null, null, null, null, null, null},  // row 1
    /* row 2 */ {null, null, null,   k , null, null, null, null},  // row 2
    /* row 3 */ {null, null, null, null, null, null, null, null},  // row 3
    /* row 4 */ {null,  K  , null, null, null, null, null, null},  // row 4
    /* row 5 */ {null, null, null, null, null, null, null, null},  // row 5
    /* row 6 */ {null, null, null, null, null, null, null, null},  // row 6
    /* row 7 */ {null, null, null, null, null, null, null, null},  // row 7
    // columns :  0     1     2     3     4     5     6     7
        };
        setUpGame(pieces);

        testGameResult(new ChessGameResult(GameOutcome.DRAW, ChessGameResultReason.DEAD_POSITION));
    }

    @Test
    public void testDeadPosition2() {
        ChessPiece[][] pieces = {
    // columns :  0     1     2     3     4     5     6     7
    /* row 0 */ {null, null, null, null, null, null,  k  , null},  // row 0
    /* row 1 */ {null, null, null, null,  p1 , null, null, null},  // row 1
    /* row 2 */ {null, null, null, null, null, null, null, null},  // row 2
    /* row 3 */ {null, null, null, null, null,  N1 , null, null},  // row 3
    /* row 4 */ {null, null, null, null, null, null, null, null},  // row 4
    /* row 5 */ {null, null,  K  , null, null, null, null, null},  // row 5
    /* row 6 */ {null, null, null, null, null, null, null, null},  // row 6
    /* row 7 */ {null, null, null, null, null, null, null, null},  // row 7
    // columns :  0     1     2     3     4     5     6     7
        };
        setUpGame(pieces);

        testGameResult(null);
        
        game.move(new Move(new Square(3, 5), new Square(1, 4)));
        testGameResult(new ChessGameResult(GameOutcome.DRAW, ChessGameResultReason.DEAD_POSITION));
    }

    @Test
    public void testDeadPosition3() {
        ChessPiece[][] pieces = {
    // columns :  0     1     2     3     4     5     6     7
    /* row 0 */ {null, null, null, null, null, null, null, null},  // row 0
    /* row 1 */ {null, null, null, null, null, null, null, null},  // row 1
    /* row 2 */ {null, null, null, null, null,  k  , null, null},  // row 2
    /* row 3 */ {null, null, null, null, null, null, null, null},  // row 3
    /* row 4 */ { K  , null, null, null, null, null, null, null},  // row 4
    /* row 5 */ {null, null, null, null, null, null, null, null},  // row 5
    /* row 6 */ {null, null, null, null, null, null, null, null},  // row 6
    /* row 7 */ {null, null, null, null,  B1 , null, null, null},  // row 7
    // columns :  0     1     2     3     4     5     6     7
        };
        setUpGame(pieces);

        testGameResult(new ChessGameResult(GameOutcome.DRAW, ChessGameResultReason.DEAD_POSITION));
    }

    @Test
    public void testDeadPosition4() {
        ChessPiece[][] pieces = {
    // columns :  0     1     2     3     4     5     6     7
    /* row 0 */ {null, null, null, null, null, null, null, null},  // row 0
    /* row 1 */ {null, null, null, null, null, null, null, null},  // row 1
    /* row 2 */ {null, null, null,  k  , null, null, null, null},  // row 2
    /* row 3 */ {null, null, null, null, null,  b1 ,  b2 , null},  // row 3
    /* row 4 */ {null, null, null, null, null,  K  , null, null},  // row 4
    /* row 5 */ {null, null, null, null, null, null, null, null},  // row 5
    /* row 6 */ {null, null, null, null, null, null, null, null},  // row 6
    /* row 7 */ { B1 , null, null, null, null, null, null, null},  // row 7
    // columns :  0     1     2     3     4     5     6     7
        };
        setUpGame(pieces);
        
        testGameResult(null);
        
        game.move(new Move(new Square(4, 5), new Square(3, 5)));
        testGameResult(new ChessGameResult(GameOutcome.DRAW, ChessGameResultReason.DEAD_POSITION));
    }

    @Test
    public void testDeadPosition5() {
        ChessPiece[][] pieces = {
    // columns :  0     1     2     3     4     5     6     7
    /* row 0 */ {null, null, null, null, null, null, null, null},  // row 0
    /* row 1 */ {null, null, null, null, null, null, null, null},  // row 1
    /* row 2 */ {null, null, null,  k  , null, null, null, null},  // row 2
    /* row 3 */ {null, null, null, null, null,  b1 ,  b2 , null},  // row 3
    /* row 4 */ {null, null, null, null, null,  K  , null, null},  // row 4
    /* row 5 */ {null, null, null, null, null, null, null, null},  // row 5
    /* row 6 */ {null, null, null, null, null, null, null, null},  // row 6
    /* row 7 */ { B1 , null, null, null, null, null, null, null},  // row 7
    // columns :  0     1     2     3     4     5     6     7
        };
        setUpGame(pieces);

        testGameResult(null);
        
        game.move(new Move(new Square(4, 5), new Square(3, 6)));
        testGameResult(null);
    }

    @Test
    public void testDeadPosition6() {
        ChessPiece[][] pieces = {
    // columns :  0     1     2     3     4     5     6     7
    /* row 0 */ {null, null, null, null, null, null, null,  k  },  // row 0
    /* row 1 */ {null, null, null, null, null, null,  B2 ,  B1 },  // row 1
    /* row 2 */ {null, null, null, null, null, null,  B3 , null},  // row 2
    /* row 3 */ {null, null, null, null, null,  B4 , null, null},  // row 3
    /* row 4 */ {null, null, null, null, null, null, null, null},  // row 4
    /* row 5 */ {null,  K  , null, null, null, null, null, null},  // row 5
    /* row 6 */ {null, null, null, null, null, null, null, null},  // row 6
    /* row 7 */ {null, null, null, null, null, null, null, null},  // row 7
    // columns :  0     1     2     3     4     5     6     7
        };
        gameState.setColorToMove(PieceColor.BLACK);
        setUpGame(pieces);
        
        testGameResult(null);
        
        game.move(new Move(new Square(0, 7), new Square(1, 6)));
        testGameResult(new ChessGameResult(GameOutcome.DRAW, ChessGameResultReason.DEAD_POSITION));
    }

    @Test
    public void testDeadPosition7() {
        ChessPiece[][] pieces = {
    // columns :  0     1     2     3     4     5     6     7
    /* row 0 */ { b1 , null, null, null, null, null, null,  k  },  // row 0
    /* row 1 */ {null, null, null, null, null, null, null,  B1 },  // row 1
    /* row 2 */ {null, null, null, null, null, null,  B2 , null},  // row 2
    /* row 3 */ {null, null, null, null, null,  B3 , null, null},  // row 3
    /* row 4 */ {null, null, null, null, null, null, null, null},  // row 4
    /* row 5 */ {null,  K  , null, null, null, null, null, null},  // row 5
    /* row 6 */ {null, null, null, null, null, null, null, null},  // row 6
    /* row 7 */ {null, null, null, null, null, null, null,  b2 },  // row 7
    // columns :  0     1     2     3     4     5     6     7
        };
        setUpGame(pieces);
        
        testGameResult(new ChessGameResult(GameOutcome.DRAW, ChessGameResultReason.DEAD_POSITION));
    }
    
    @Test
    public void testPositionRepetition1() {
        ChessPiece[][] pieces = {
    // columns :  0     1     2     3     4     5     6     7
    /* row 0 */ { r1 , null,  b1 ,  q1 ,  k  , null, null,  r2 },  // row 0
    /* row 1 */ { p1 ,  p2 ,  p3 , null, null, null,  p6 ,  p7 },  // row 1
    /* row 2 */ {null, null, null, null, null,  p5 , null, null},  // row 2
    /* row 3 */ {null, null, null,  P4 ,  p4 , null, null, null},  // row 3
    /* row 4 */ {null, null, null, null, null, null, null, null},  // row 4
    /* row 5 */ {null, null, null, null, null, null, null, null},  // row 5
    /* row 6 */ { P1 ,  P2 ,  P3 , null, null,  P5 ,  P6 ,  P7 },  // row 6
    /* row 7 */ { R1 , null, null,  Q1 ,  K  , null,  N1 ,  R2 },  // row 7
    // columns :  0     1     2     3     4     5     6     7
        };
        setUpGame(pieces);
        game = new ChessGame(gameState, 0, 0, true, true);
        
        String[] moves = {
                "Qd4", "Qd6",
                "Nf3", "Bg4", "Ng1", "Bc8",
                "Nf3", "Bg4", "Ng1", "Bc8",
        };
        
        testIsDrawAfterLastMove(moves, ChessGameResultReason.POSITION_REPETITION);
    }

    @Test
    public void testPositionRepetition2() {
        ChessPiece[][] pieces = {
    // columns :  0     1     2     3     4     5     6     7
    /* row 0 */ { r1 , null,  b1 ,  q1 ,  k  , null, null,  r2 },  // row 0
    /* row 1 */ { p1 ,  p2 ,  p3 , null, null, null,  p6 ,  p7 },  // row 1
    /* row 2 */ {null, null, null, null, null,  p5 , null, null},  // row 2
    /* row 3 */ {null, null, null,  P4 ,  p4 , null, null, null},  // row 3
    /* row 4 */ {null, null, null, null, null, null, null, null},  // row 4
    /* row 5 */ {null, null, null, null, null, null, null, null},  // row 5
    /* row 6 */ { P1 ,  P2 ,  P3 , null, null,  P5 ,  P6 ,  P7 },  // row 6
    /* row 7 */ { R1 , null, null,  Q1 ,  K  , null,  N1 ,  R2 },  // row 7
    // columns :  0     1     2     3     4     5     6     7
        };
        setUpGame(pieces);
        game = new ChessGame(gameState, 0, 0, true, true);
        
        String[] moves = {
                "Nf3", "Bg4", "Ng1", "Bc8",
                "Qd4", "Qd6",
                "Nf3", "Bg4", "Ng1", "Bc8",
                "Qd1", "Qd8",
        };
        
        testIsDrawAfterLastMove(moves, ChessGameResultReason.POSITION_REPETITION);
    }
    
    // The rule says the same position must be repeated with the same color to move.
    @Test
    public void testPositionRepetition3() {
        ChessPiece[][] pieces = {
    // columns :  0     1     2     3     4     5     6     7
    /* row 0 */ { r1 , null,  b1 ,  q1 ,  k  , null, null,  r2 },  // row 0
    /* row 1 */ { p1 ,  p2 ,  p3 , null, null, null,  p6 ,  p7 },  // row 1
    /* row 2 */ {null, null, null, null, null,  p5 , null, null},  // row 2
    /* row 3 */ {null, null, null,  P4 ,  p4 , null, null, null},  // row 3
    /* row 4 */ {null, null, null, null, null, null, null, null},  // row 4
    /* row 5 */ {null, null, null, null, null, null, null, null},  // row 5
    /* row 6 */ { P1 ,  P2 ,  P3 , null, null,  P5 ,  P6 ,  P7 },  // row 6
    /* row 7 */ { R1 , null, null,  Q1 ,  K  , null,  N1 ,  R2 },  // row 7
    // columns :  0     1     2     3     4     5     6     7
        };
        setUpGame(pieces);
        game = new ChessGame(gameState, 0, 0, true, true);
        
        String[] moves = {
                "Ke2", "Ke7", "Ke1", "Ke8",
                "Kd2", "Ke7", "Ke2", "Ke8", "Ke1",
                "Kd7", "Ke2", "Ke7", "Ke1", "Ke8",
        };
        
        testIsDrawAfterLastMove(moves, ChessGameResultReason.POSITION_REPETITION);
    }
    
    // The rule says the same position must be repeated with the same castling rights.
    @Test
    public void testPositionRepetition4() {
        ChessPiece[][] pieces = {
    // columns :  0     1     2     3     4     5     6     7
    /* row 0 */ { r1 , null,  b1 ,  q1 ,  k  , null, null,  r2 },  // row 0
    /* row 1 */ { p1 ,  p2 ,  p3 , null, null, null,  p6 ,  p7 },  // row 1
    /* row 2 */ {null, null, null, null, null,  p5 , null, null},  // row 2
    /* row 3 */ {null, null, null,  P4 ,  p4 , null, null, null},  // row 3
    /* row 4 */ {null, null, null, null, null, null, null, null},  // row 4
    /* row 5 */ {null, null, null, null, null, null, null, null},  // row 5
    /* row 6 */ { P1 ,  P2 ,  P3 , null, null,  P5 ,  P6 ,  P7 },  // row 6
    /* row 7 */ { R1 , null, null,  Q1 ,  K  , null,  N1 ,  R2 },  // row 7
    // columns :  0     1     2     3     4     5     6     7
        };
        k.setMoved(false);
        K.setMoved(false);
        r1.setMoved(false);
        r2.setMoved(false);
        R1.setMoved(false);
        R2.setMoved(false);
        setUpGame(pieces);
        game = new ChessGame(gameState, 0, 0, true, true);
        
        String[] moves = {
                "Rb1", "Ke7", "Ra1", "Ke8",
                "Rb1", "Ke7", "Ra1", "Ke8",
                "Rb1", "Ke7",
        };
        
        testIsDrawAfterLastMove(moves, ChessGameResultReason.POSITION_REPETITION);
    }
    
    // The rule says the same position must be repeated with the same en passant square status.
    @Test
    public void testPositionRepetition5() {
        ChessPiece[][] pieces = {
    // columns :  0     1     2     3     4     5     6     7
    /* row 0 */ { r1 , null,  b1 ,  q1 ,  k  , null, null,  r2 },  // row 0
    /* row 1 */ { p1 ,  p2 ,  p3 , null, null, null,  p6 ,  p7 },  // row 1
    /* row 2 */ {null, null, null, null, null,  p5 , null, null},  // row 2
    /* row 3 */ {null, null, null,  P4 ,  p4 , null, null, null},  // row 3
    /* row 4 */ {null, null, null, null, null, null, null, null},  // row 4
    /* row 5 */ {null, null, null, null, null, null, null, null},  // row 5
    /* row 6 */ { P1 ,  P2 ,  P3 , null, null,  P5 ,  P6 ,  P7 },  // row 6
    /* row 7 */ { R1 , null, null,  Q1 ,  K  , null,  N1 ,  R2 },  // row 7
    // columns :  0     1     2     3     4     5     6     7
        };
        setUpGame(pieces);
        gameState.setColorToMove(PieceColor.BLACK);
        game = new ChessGame(gameState, 0, 0, true, true);
        
        String[] moves = {
                "a5",
                "Qd2", "Qd7", "Qd1", "Qd8",
                "Qd2", "Qd7", "Qd1", "Qd8",
                "Qd2",
        };
        
        testIsDrawAfterLastMove(moves, ChessGameResultReason.POSITION_REPETITION);
    }
    
    @Test
    public void test50MoveRule1() {
        ChessPiece[][] pieces = {
    // columns :  0     1     2     3     4     5     6     7
    /* row 0 */ { r1 , null,  b1 ,  q1 ,  k  , null, null,  r2 },  // row 0
    /* row 1 */ { p1 ,  p2 ,  p3 , null, null, null,  p6 ,  p7 },  // row 1
    /* row 2 */ {null, null, null, null, null,  p5 , null, null},  // row 2
    /* row 3 */ {null, null, null,  P4 ,  p4 , null, null, null},  // row 3
    /* row 4 */ {null, null, null, null, null, null, null, null},  // row 4
    /* row 5 */ {null, null, null, null, null, null, null, null},  // row 5
    /* row 6 */ { P1 ,  P2 ,  P3 , null, null,  P5 ,  P6 ,  P7 },  // row 6
    /* row 7 */ { R1 , null, null,  Q1 ,  K  , null,  N1 ,  R2 },  // row 7
    // columns :  0     1     2     3     4     5     6     7
        };
        k.setMoved(false);
        K.setMoved(false);
        r2.setMoved(false);
        R2.setMoved(false);
        setUpGame(pieces);
        game = new ChessGame(gameState, 36, 5, true, true);
        
        String[] moves = {
                "Nf3", "Bg4", "h3", "Bxf3", "gxf3", "h6",
                // now the fun starts
                "Qd2", "Qd7", "Rb1", "Rb8", "Rc1", "Rc8", "Rb1", "Rb8", "Ra1", "Ra8",
                "Kd1", "Kd8", "Rb1", "Rb8", "Rc1", "Rc8", "Rb1", "Rb8", "Ra1", "Ra8",
                "Qd3", "Qd6", "Rb1", "Rb8", "Rc1", "Rc8", "Rb1", "Rb8", "Ra1", "Ra8",
                "Qa3", "Qa6", "Rb1", "Rb8", "Rc1", "Rc8", "Rb1", "Rb8", "Ra1", "Ra8",
                "Qa4", "Qa5", "Rb1", "Rb8", "Rc1", "Rc8", "Rb1", "Rb8", "Ra1", "Ra8",
                "Qb3", "Qb6", "Rb1", "Rb8", "Rc1", "Rc8", "Rb1", "Rb8", "Ra1", "Ra8",
                "Qb4", "Qb5", "Rb1", "Rb8", "Rc1", "Rc8", "Rb1", "Rb8", "Ra1", "Ra8",
                "Qc3", "Qc6", "Rb1", "Rb8", "Rc1", "Rc8", "Rb1", "Rb8", "Ra1", "Ra8",
                "Qc4", "Qc5", "Rb1", "Rb8", "Rc1", "Rc8", "Rb1", "Rb8", "Ra1", "Ra8",
                "Qd4", "Qd6", "Rb1", "Rb8", "Rc1", "Rc8", "Rb1", "Rb8", "Ra1", "Ra8",
        };
        
        testIsDrawAfterLastMove(moves, ChessGameResultReason.FIFTY_MOVE_RULE);
    }
    
    @Test
    public void test50MoveRule2() {
        ChessPiece[][] pieces = {
    // columns :  0     1     2     3     4     5     6     7
    /* row 0 */ { r1 , null,  b1 ,  q1 ,  k  , null, null,  r2 },  // row 0
    /* row 1 */ { p1 ,  p2 ,  p3 , null, null, null,  p6 ,  p7 },  // row 1
    /* row 2 */ {null, null, null, null, null,  p5 , null, null},  // row 2
    /* row 3 */ {null, null, null,  P4 ,  p4 , null, null, null},  // row 3
    /* row 4 */ {null, null, null, null, null, null, null, null},  // row 4
    /* row 5 */ {null, null, null, null, null, null, null, null},  // row 5
    /* row 6 */ { P1 ,  P2 ,  P3 , null, null,  P5 ,  P6 ,  P7 },  // row 6
    /* row 7 */ { R1 , null, null,  Q1 ,  K  , null,  N1 ,  R2 },  // row 7
    // columns :  0     1     2     3     4     5     6     7
        };
        k.setMoved(false);
        K.setMoved(false);
        r2.setMoved(false);
        R2.setMoved(false);
        setUpGame(pieces);
        game = new ChessGame(gameState, 36, 5, true, true);
        
        String[] moves = {
                "Nf3", "Bg4", "Rb1", "Rb8", "Rc1", "Rc8", "Rb1", "Rb8", "Ra1", "Ra8",
                "Qd2", "Qd7", "Rb1", "Rb8", "Rc1", "Rc8", "Rb1", "Rb8", "Ra1", "Ra8",
                "Kd1", "Kd8", "Rb1", "Rb8", "Rc1", "Rc8", "Rb1", "Rb8", "Ra1", "Ra8",
                "Qd3", "Qd6", "Rb1", "Rb8", "Rc1", "Rc8", "Rb1", "Rb8", "Ra1", "Ra8",
                "Qa3", "Qa6", "Rb1", "Rb8", "Rc1", "Rc8", "Rb1", "Rb8", "Ra1", "Ra8",
                "Qa4", "Qa5", "Rb1", "Rb8", "Rc1", "Rc8", "Rb1", "Rb8", "Ra1", "Ra8",
                "Qb3", "Qb6", "Rb1", "Rb8", "Rc1", "Rc8", "Rb1", "Rb8", "Ra1", "Ra8",
                "Qb4", "Qb5", "Rb1", "Rb8", "Rc1", "Rc8", "Rb1", "Rb8", "Ra1", "Ra8",
                "Qc3", "Qc6", "Rb1", "Rb8", "Rc1", "Rc8", "Rb1", "Rb8", "Ra1", "Ra8",
                "Qc4", "Qc5", "Rb1", "Rb8", "Rc1",
        };
        
        testIsDrawAfterLastMove(moves, ChessGameResultReason.FIFTY_MOVE_RULE);
    }
    
    private void testGameResult(ChessGameResult expectedResult) {
        if(expectedResult == null) {
            Assert.assertFalse(game.isFinished());
            Assert.assertNull(game.getResult());
        } else {
            Assert.assertTrue(game.isFinished());
            
            ChessGameResult actualResult = game.getResult();
            Assert.assertNotNull(actualResult);
            Assert.assertEquals(actualResult.getOutcome(), expectedResult.getOutcome());
            Assert.assertEquals(actualResult.getResultReason(), expectedResult.getResultReason());
        }
    }
    
    private void testIsDrawAfterLastMove(String[] moves, ChessGameResultReason resultReason) {
        ChessAlgebraicMoveNotation notation = new ChessAlgebraicMoveNotation();
        for(int i = 0; i < moves.length; i++) {
            game.move(notation.getMove(moves[i], game));
            if(i != moves.length - 1) {
                testGameResult(null);
            } else {
                testGameResult(new ChessGameResult(GameOutcome.DRAW, resultReason));
            }
        }
    }
    
}
