/*
Copyright (C) 2009, 2010 Alex Ignácio da Silva

This file is part of Haundrix Games.

Haundrix Games is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License version 3 as published by the
Free Software Foundation, with additional permission under section 7 to combine
it with independent works licensed under selected Free/Libre Open Source Software licenses.
Please see the full text of the license that is distributed along with this file.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
*/
package com.haundrix.games.chess.model;

import org.testng.Assert;
import org.testng.annotations.Test;

import com.haundrix.games.model.Move;
import com.haundrix.games.model.Square;
import com.haundrix.games.model.Piece.PieceColor;

public class ChessGameSpecialMovesTest extends ChessTest {

    @Test
    public void testIsCaptureMove() {
        ChessPiece[][] pieces = {
    // columns :  0     1     2     3     4     5     6     7
    /* row 0 */ { r1 , null, null, null, null,  r2 ,  k  , null},  // row 0
    /* row 1 */ {null,  b1 ,  p3 ,  q1 ,  P3 ,  p5 , null,  p7 },  // row 1
    /* row 2 */ {null,  p2 ,  n1 ,  p4 , null, null, null, null},  // row 2
    /* row 3 */ { p1 ,  B1 , null, null, null,  P4 ,  p6 , null},  // row 3
    /* row 4 */ {null,  P1 , null, null, null, null, null, null},  // row 4
    /* row 5 */ {null, null, null, null,  B2 ,  N2 , null, null},  // row 5
    /* row 6 */ {null,  N1 ,  P2 , null, null,  n2 ,  P5 ,  P6 },  // row 6
    /* row 7 */ { R1 , null, null,  Q1 ,  K  , null, null,  R2 },  // row 7
    // columns :  0     1     2     3     4     5     6     7
        };
        setUpGame(pieces);
        K.setMoved(false);
        R2.setMoved(false);
        gameState.setEnPassantSquare(new Square(2, 6));

        class TestMove {
            
            private Move move;
            private boolean isCaptureMove;

            public TestMove(Move move, boolean isCaptureMove) {
                this.move = move;
                this.isCaptureMove = isCaptureMove;
            }
            
        }
        
        TestMove[] testMoves = {
            // pawn moves
            new TestMove(new Move(new Square(4, 1), new Square(3, 0)), true),
            new TestMove(new PromotionMove(new Square(1, 4), new Square(0, 4), Bishop.class), false),
            new TestMove(new PromotionMove(new Square(1, 4), new Square(0, 5), Knight.class), true),
            new TestMove(new Move(new Square(3, 5), new Square(2, 6)), true),
            new TestMove(new Move(new Square(6, 7), new Square(5, 7)), false),
            new TestMove(new Move(new Square(6, 7), new Square(4, 7)), false),
            
            // rook moves
            new TestMove(new Move(new Square(7, 0), new Square(3, 0)), true),
            new TestMove(new Move(new Square(7, 7), new Square(7, 5)), false),
            
            // knight moves
            new TestMove(new Move(new Square(6, 1), new Square(4, 2)), false),
            new TestMove(new Move(new Square(5, 5), new Square(3, 6)), true),
            
            // bishop moves
            new TestMove(new Move(new Square(3, 1), new Square(2, 2)), true),
            new TestMove(new Move(new Square(3, 1), new Square(6, 4)), false),
            
            // queen moves
            new TestMove(new Move(new Square(7, 3), new Square(2, 3)), true),
            new TestMove(new Move(new Square(7, 3), new Square(6, 4)), false),
            
            // king moves
            new TestMove(new Move(new Square(7, 4), new Square(6, 5)), true),
            new TestMove(new Move(new Square(7, 4), new Square(7, 6)), false),
        };
        
        for(TestMove testMove : testMoves) {
            Assert.assertEquals(game.isCaptureMove(testMove.move), testMove.isCaptureMove);
        }
    }
    
    @Test
    public void testIsEnPassantMove() {
        ChessPiece[][] pieces = {
    // columns :  0     1     2     3     4     5     6     7
    /* row 0 */ { r1 , null, null, null, null,  r2 ,  k  , null},  // row 0
    /* row 1 */ {null,  b1 ,  p3 ,  q1 ,  P3 ,  p5 , null,  p7 },  // row 1
    /* row 2 */ {null,  p2 ,  n1 , null, null, null, null, null},  // row 2
    /* row 3 */ { p1 ,  B1 , null, null, null,  P4 ,  p6 , null},  // row 3
    /* row 4 */ {null,  P1 , null,  p4 , null, null, null, null},  // row 4
    /* row 5 */ {null, null, null, null,  B2 ,  N2 , null, null},  // row 5
    /* row 6 */ {null,  N1 ,  P2 , null, null,  n2 ,  P5 ,  P6 },  // row 6
    /* row 7 */ { R1 , null, null,  Q1 ,  K  , null, null,  R2 },  // row 7
    // columns :  0     1     2     3     4     5     6     7
        };
        setUpGame(pieces);
        K.setMoved(false);
        R2.setMoved(false);
        gameState.setEnPassantSquare(new Square(2, 6));

        class TestMove {
            
            private Move move;
            private boolean isEnPassantMove;

            public TestMove(Move move, boolean isEnPassantMove) {
                this.move = move;
                this.isEnPassantMove = isEnPassantMove;
            }
            
        }
        
        TestMove[] testMoves = {
            // pawn moves
            new TestMove(new Move(new Square(4, 1), new Square(3, 0)), false),
            new TestMove(new PromotionMove(new Square(1, 4), new Square(0, 4), Bishop.class), false),
            new TestMove(new PromotionMove(new Square(1, 4), new Square(0, 5), Knight.class), false),
            new TestMove(new Move(new Square(3, 5), new Square(2, 6)), true),
            new TestMove(new Move(new Square(6, 7), new Square(5, 7)), false),
            new TestMove(new Move(new Square(6, 7), new Square(4, 7)), false),
            
            // assorted moves
            new TestMove(new Move(new Square(7, 0), new Square(3, 0)), false),
            new TestMove(new Move(new Square(3, 1), new Square(6, 4)), false),
            new TestMove(new Move(new Square(7, 4), new Square(7, 6)), false),
        };
        
        for(TestMove testMove : testMoves) {
            Assert.assertEquals(game.isEnPassantMove(testMove.move), testMove.isEnPassantMove);
        }
        
        game.move(new Move(new Square(6, 2), new Square(4, 2)));
        
        testMoves = new TestMove[] {
            // pawn moves
            new TestMove(new Move(new Square(3, 0), new Square(4, 1)), false),
            new TestMove(new Move(new Square(4, 3), new Square(5, 2)), true),
            new TestMove(new Move(new Square(1, 7), new Square(2, 7)), false),
            new TestMove(new Move(new Square(1, 7), new Square(3, 7)), false),
            
            // assorted moves
            new TestMove(new Move(new Square(0, 0), new Square(0, 4)), false),
            new TestMove(new Move(new Square(1, 3), new Square(3, 5)), false),
            new TestMove(new Move(new Square(0, 6), new Square(1, 6)), false),
        };
        
        for(TestMove testMove : testMoves) {
            Assert.assertEquals(game.isEnPassantMove(testMove.move), testMove.isEnPassantMove);
        }
    }
    
    @Test
    public void testIsCheckMove() {
        ChessPiece[][] pieces = {
    // columns :  0     1     2     3     4     5     6     7
    /* row 0 */ {null, null, null,  r1 , null, null,  k  ,  n2 },  // row 0
    /* row 1 */ {null,  b1 ,  p1 , null,  P2 ,  p3 , null,  p4 },  // row 1
    /* row 2 */ { r1 , null, null, null, null, null,  B2 , null},  // row 2
    /* row 3 */ {null, null, null, null,  N1 ,  N2 , null, null},  // row 3
    /* row 4 */ {null,  P1 , null,  p2 , null, null, null, null},  // row 4
    /* row 5 */ {null,  n1 , null, null,  B1 , null,  Q1 , null},  // row 5
    /* row 6 */ {null, null, null, null, null, null,  P3 ,  P4 },  // row 6
    /* row 7 */ { R1 , null, null, null,  K  , null, null,  R2 },  // row 7
    // columns :  0     1     2     3     4     5     6     7
        };
        setUpGame(pieces);
        K.setMoved(false);
        R2.setMoved(false);

        class TestMove {
            
            private Move move;
            private boolean isCheckMove;
            private boolean isCheckMateMove;

            public TestMove(Move move, boolean isCheckMove, boolean isCheckMateMove) {
                this.move = move;
                this.isCheckMove = isCheckMove;
                this.isCheckMateMove = isCheckMateMove;
            }
            
        }
        
        TestMove[] testMoves = {
            new TestMove(new PromotionMove(new Square(1, 4), new Square(0, 3), Rook.class), true, true),
            new TestMove(new PromotionMove(new Square(1, 4), new Square(0, 4), Queen.class), true, false),
            new TestMove(new Move(new Square(2, 6), new Square(1, 5)), true, true),
            new TestMove(new Move(new Square(2, 6), new Square(1, 7)), true, false),
            new TestMove(new Move(new Square(2, 6), new Square(3, 7)), true, false),
            new TestMove(new Move(new Square(3, 5), new Square(2, 7)), true, false),
            new TestMove(new Move(new Square(3, 5), new Square(2, 3)), false, false),
            new TestMove(new Move(new Square(7, 0), new Square(2, 0)), false, false),
            new TestMove(new Move(new Square(7, 4), new Square(7, 6)), false, false),
            new TestMove(new Move(new Square(4, 1), new Square(3, 1)), false, false),
        };
        
        for(TestMove testMove : testMoves) {
            Assert.assertEquals(game.isCheckMove(testMove.move), testMove.isCheckMove);
            Assert.assertEquals(game.isCheckmateMove(testMove.move), testMove.isCheckMateMove);
        }
        
        game.move(new Move(new Square(7, 4), new Square(7, 3)));
        
        testMoves = new TestMove[] {
            new TestMove(new Move(new Square(2, 0), new Square(7, 0)), true, false),
            new TestMove(new Move(new Square(4, 3), new Square(5, 4)), true, false),
            new TestMove(new Move(new Square(4, 3), new Square(5, 3)), false, false),
            new TestMove(new Move(new Square(1, 1), new Square(5, 5)), true, false),
            new TestMove(new Move(new Square(5, 1), new Square(6, 3)), false, false),
            new TestMove(new Move(new Square(0, 3), new Square(3, 3)), false, false),
        };
        
        for(TestMove testMove : testMoves) {
            Assert.assertEquals(game.isCheckMove(testMove.move), testMove.isCheckMove);
            Assert.assertEquals(game.isCheckmateMove(testMove.move), testMove.isCheckMateMove);
        }
    }

    @Test
    public void testIsCastleMove() {
        ChessPiece[][] pieces = {
    // columns :  0     1     2     3     4     5     6     7
    /* row 0 */ { r1 , null, null, null,  k  , null, null,  r2 },  // row 0
    /* row 1 */ {null,  p2 , null,  q1 , null,  p5 ,  p6 ,  p7 },  // row 1
    /* row 2 */ {null, null, null, null, null,  n1 , null, null},  // row 2
    /* row 3 */ {null, null,  p3 ,  P4 ,  p4 , null,  B1 , null},  // row 3
    /* row 4 */ {null, null, null,  P3 , null, null,  b1 , null},  // row 4
    /* row 5 */ {null, null,  N1 , null,  Q1 ,  N2 , null, null},  // row 5
    /* row 6 */ { p1 ,  P1 ,  P2 , null, null,  P5 ,  P6 ,  P7 },  // row 6
    /* row 7 */ {null,  R1 , null, null,  K  , null, null,  R2 },  // row 7
    // columns :  0     1     2     3     4     5     6     7
        };
        setUpGame(pieces);
        r1.setMoved(false);
        k.setMoved(false);
        r2.setMoved(false);
        R1.setMoved(true);
        K.setMoved(false);
        R2.setMoved(false);
        gameState.setEnPassantSquare(new Square(2, 2));
        
        class TestMove {
            
            private Move move;
            private boolean isQueensideCastleMove;
            private boolean isKingsideCastleMove;

            public TestMove(Move move, boolean isQueensideCastleMove, boolean isKingsideCastleMove) {
                this.move = move;
                this.isQueensideCastleMove = isQueensideCastleMove;
                this.isKingsideCastleMove = isKingsideCastleMove;
            }
            
            public boolean isCastleMove() {
                return isQueensideCastleMove || isKingsideCastleMove;
            }
            
        }
        
        TestMove[] testMoves = {
            new TestMove(new Move(new Square(7, 4), new Square(7, 6)), false, true),
            new TestMove(new Move(new Square(7, 4), new Square(7, 3)), false, false),
            new TestMove(new Move(new Square(3, 3), new Square(2, 2)), false, false),
            new TestMove(new Move(new Square(4, 3), new Square(3, 4)), false, false),
            new TestMove(new Move(new Square(3, 6), new Square(2, 5)), false, false),
        };
        
        for(TestMove testMove : testMoves) {
            Assert.assertEquals(game.isQueensideCastleMove(testMove.move), testMove.isQueensideCastleMove);
            Assert.assertEquals(game.isKingsideCastleMove(testMove.move), testMove.isKingsideCastleMove);
            Assert.assertEquals(game.isCastleMove(testMove.move), testMove.isCastleMove());
        }
        
        game.move(new Move(new Square(7, 4), new Square(7, 6)));
        
        testMoves = new TestMove[] {
                new TestMove(new Move(new Square(0, 4), new Square(0, 2)), true, false),
                new TestMove(new Move(new Square(0, 4), new Square(0, 6)), false, true),
                new TestMove(new Move(new Square(0, 4), new Square(0, 5)), false, false),
                new TestMove(new PromotionMove(new Square(6, 0), new Square(7, 1), Bishop.class), false, false),
                new TestMove(new Move(new Square(1, 3), new Square(3, 3)), false, false),
        };
        
        for(TestMove testMove : testMoves) {
            Assert.assertEquals(game.isQueensideCastleMove(testMove.move), testMove.isQueensideCastleMove);
            Assert.assertEquals(game.isKingsideCastleMove(testMove.move), testMove.isKingsideCastleMove);
            Assert.assertEquals(game.isCastleMove(testMove.move), testMove.isCastleMove());
        }
    }

    @Test
    public void testIsKnightAttackMove1() {
        ChessPiece[][] pieces = {
    // columns :  0     1     2     3     4     5     6     7
    /* row 0 */ {null, null, null, null, null, null, null,  k  },  // row 0
    /* row 1 */ {null, null, null, null,  r1 , null, null, null},  // row 1
    /* row 2 */ {null, null, null, null, null, null, null,  N2 },  // row 2
    /* row 3 */ {null, null, null, null, null, null, null, null},  // row 3
    /* row 4 */ {null, null, null, null, null,  r2 , null, null},  // row 4
    /* row 5 */ {null, null,  N1 , null,  q1 , null, null, null},  // row 5
    /* row 6 */ {null, null, null, null, null, null, null, null},  // row 6
    /* row 7 */ { K  , null, null, null, null, null, null, null},  // row 7
    // columns :  0     1     2     3     4     5     6     7
        };
        setUpGame(pieces);

        class TestMove {
            
            private Move move;
            private boolean isKnightAttackMove;
            
            public TestMove(Move move, boolean isKnightAttackMove) {
                this.move = move;
                this.isKnightAttackMove = isKnightAttackMove;
            }

            public Move getMove() {
                return move;
            }

            public boolean isKnightAttackMove() {
                return isKnightAttackMove;
            }
            
        }
        
        TestMove[] testMoves = {
                new TestMove(new Move(new Square(5, 2), new Square(3, 3)), true),
                new TestMove(new Move(new Square(5, 2), new Square(7, 3)), false),
                new TestMove(new Move(new Square(5, 2), new Square(6, 4)), false),
                new TestMove(new Move(new Square(5, 2), new Square(7, 1)), false),
                new TestMove(new Move(new Square(2, 7), new Square(1, 5)), true),
                new TestMove(new Move(new Square(2, 7), new Square(3, 5)), true),
                new TestMove(new Move(new Square(2, 7), new Square(4, 6)), false),
                new TestMove(new Move(new Square(7, 0), new Square(6, 1)), false),
        };
        
        for(TestMove testMove : testMoves) {
            Assert.assertEquals(game.isKnightAttackMove(testMove.getMove()), testMove.isKnightAttackMove());
        }
    }
    
    @Test
    public void testIsKnightAttackMove2() {
        ChessPiece[][] pieces = {
    // columns :  0     1     2     3     4     5     6     7
    /* row 0 */ {null, null, null, null, null, null, null, null},  // row 0
    /* row 1 */ {null, null, null, null, null, null, null, null},  // row 1
    /* row 2 */ {null, null, null, null, null, null, null, null},  // row 2
    /* row 3 */ {null,  Q2 , null,  Q1 , null, null, null, null},  // row 3
    /* row 4 */ {null, null, null,  B1 , null, null, null, null},  // row 4
    /* row 5 */ {null, null, null, null, null, null, null, null},  // row 5
    /* row 6 */ {null, null, null, null, null, null, null, null},  // row 6
    /* row 7 */ { k  , null, null,  n1 , null,  K  , null, null},  // row 7
    // columns :  0     1     2     3     4     5     6     7
        };
        setUpGame(pieces);
        gameState.setColorToMove(PieceColor.BLACK);

        Move testMove = new Move(new Square(7, 3), new Square(5, 2));
        Assert.assertFalse(game.isKnightAttackMove(testMove));
    }
    
    @Test
    public void testIsKnightAttackMove3() {
        ChessPiece[][] pieces = {
    // columns :  0     1     2     3     4     5     6     7
    /* row 0 */ {null, null, null, null, null, null, null, null},  // row 0
    /* row 1 */ {null, null, null, null, null, null, null, null},  // row 1
    /* row 2 */ {null, null, null, null, null, null, null, null},  // row 2
    /* row 3 */ {null,  Q2 , null,  Q1 , null, null, null, null},  // row 3
    /* row 4 */ {null, null, null,  B1 , null, null, null, null},  // row 4
    /* row 5 */ {null, null, null, null, null, null, null, null},  // row 5
    /* row 6 */ {null, null, null, null,  K  , null, null, null},  // row 6
    /* row 7 */ { k  , null, null,  n1 , null, null, null, null},  // row 7
    // columns :  0     1     2     3     4     5     6     7
        };
        setUpGame(pieces);
        gameState.setColorToMove(PieceColor.BLACK);

        Move testMove = new Move(new Square(7, 3), new Square(5, 2));
        Assert.assertTrue(game.isKnightAttackMove(testMove));
    }
    
}
