/*
Copyright (C) 2009, 2010 Alex Ignácio da Silva

This file is part of Haundrix Games.

Haundrix Games is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License version 3 as published by the
Free Software Foundation, with additional permission under section 7 to combine
it with independent works licensed under selected Free/Libre Open Source Software licenses.
Please see the full text of the license that is distributed along with this file.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
*/
package com.haundrix.games.chess.model;

import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.DataProvider;

import com.haundrix.games.model.Move;
import com.haundrix.games.model.MoveNotationTest;
import com.haundrix.games.model.Square;

public class ChessSimpleMoveNotationTest extends MoveNotationTest {

    private final Move[] moves1 = {
        new Move(new Square(6, 4), new Square(4, 4)),  // e4
        new Move(new Square(1, 3), new Square(3, 3)),  //            d5
        new Move(new Square(4, 4), new Square(3, 4)),  // e5
        new Move(new Square(1, 5), new Square(3, 5)),  //            f5
        new Move(new Square(3, 4), new Square(2, 5)),  // exf6 e.p.
        new Move(new Square(0, 1), new Square(2, 2)),  //            Nc6
        new Move(new Square(7, 6), new Square(5, 5)),  // Nf3
        new Move(new Square(0, 2), new Square(4, 6)),  //            Bg4
        new Move(new Square(7, 5), new Square(4, 2)),  // Bc4
        new Move(new Square(0, 3), new Square(2, 3)),  //            Qd6
        new Move(new Square(7, 4), new Square(7, 6)),  // 0-0
        new Move(new Square(0, 4), new Square(0, 2)),  //            0-0-0
        new Move(new Square(2, 5), new Square(1, 6)),  // fxg7
        new Move(new Square(3, 3), new Square(4, 2)),  //            dxc4
        new Move(new Square(6, 1), new Square(4, 1)),  // b4
        new Move(new Square(0, 6), new Square(2, 5)),  //            Nf6
        new PromotionMove(new Square(1, 6), new Square(0, 5), Queen.class),  // gxf8=Q
        new Move(new Square(0, 7), new Square(0, 5)),  //            Rhxf8
        new Move(new Square(6, 3), new Square(5, 3)),  // d3
        new Move(new Square(2, 3), new Square(5, 3)),  //            Qxd3
        new Move(new Square(7, 1), new Square(6, 3)),  // Nbd2
        new Move(new Square(4, 6), new Square(5, 5)),  //            Bxf3
        new Move(new Square(7, 3), new Square(5, 5)),  // Qxf3
        new Move(new Square(5, 3), new Square(7, 5)),  //            Qxf1+
        new Move(new Square(6, 3), new Square(7, 5)),  // Nxf1
        new Move(new Square(2, 2), new Square(4, 1)),  //            Nxb4
        new Move(new Square(5, 5), new Square(3, 5)),  // Qf5+
        new Move(new Square(0, 2), new Square(0, 1)),  //            Kb8
        new Move(new Square(7, 5), new Square(5, 6)),  // Ng3
        new Move(new Square(0, 5), new Square(0, 4)),  //            Rfe8
        new Move(new Square(5, 6), new Square(4, 4)),  // Ne4
        new Move(new Square(0, 3), new Square(7, 3)),  //            Rd1++
    };
    
    private final String[] moveDescriptions1 = {
        "e2e4", "d7d5",
        "e4e5", "f7f5",
        "e5f6", "b8c6",
        "g1f3", "c8g4",
        "f1c4", "d8d6",
        "e1g1", "e8c8",
        "f6g7", "d5c4",
        "b2b4", "g8f6",
        "g7f8q", "h8f8",
        "d2d3", "d6d3",
        "b1d2", "g4f3",
        "d1f3", "d3f1",
        "d2f1", "c6b4",
        "f3f5", "c8b8",
        "f1g3", "f8e8",
        "g3e4", "d8d1",
    };
        
    private final Move[] moves2 = {
        new Move(new Square(6, 7), new Square(4, 7)),  // h4
        new Move(new Square(1, 2), new Square(3, 2)),  // c5
        new Move(new Square(4, 7), new Square(3, 7)),  // h5
        new Move(new Square(3, 2), new Square(4, 2)),  // c4
        new Move(new Square(3, 7), new Square(2, 7)),  // h6
        new Move(new Square(4, 2), new Square(5, 2)),  // c3
        new Move(new Square(2, 7), new Square(1, 6)),  // hxg7
        new Move(new Square(5, 2), new Square(6, 1)),  // cxb2
        new PromotionMove(new Square(1, 6), new Square(0, 7), Bishop.class),  // gxh8=B
        new PromotionMove(new Square(6, 1), new Square(7, 0), Knight.class),  // bxa1=N
        new Move(new Square(0, 7), new Square(7, 0)),  // Bxa1
    };
    
    private final String[] moveDescriptions2 = {
        "h2h4", "c7c5",
        "h4h5", "c5c4",
        "h5h6", "c4c3",
        "h6g7", "c3b2",
        "g7h8b", "b2a1n",
        "h8a1",
    };
    
    @BeforeMethod
    public void setUp() {
        game = new ChessGame();
        notation = new ChessSimpleMoveNotation();
    }
    
    @AfterMethod
    public void tearDown() {
        game = null;
        notation = null;
    }
    
    @DataProvider(name=MoveNotationTest.MOVE_BY_MOVE_TEST_DATA_PROVIDER)
    public Object[][] getMoveByMoveTestData() {
        return new Object[][] {
            {moves1, moveDescriptions1, null},
            {moves2, moveDescriptions2, null}
        };
    }
    
    @DataProvider(name=MoveNotationTest.ALL_MOVES_TEST_DATA_PROVIDER)
    public Object[][] getAllMovesTestData() {
        return new Object[][] {
            {moves1, moveDescriptions1},
            {moves2, moveDescriptions2}
        };
    }

}
