/*
Copyright (C) 2009, 2010 Alex Ignácio da Silva

This file is part of Haundrix Games.

Haundrix Games is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License version 3 as published by the
Free Software Foundation, with additional permission under section 7 to combine
it with independent works licensed under selected Free/Libre Open Source Software licenses.
Please see the full text of the license that is distributed along with this file.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
*/
package com.haundrix.games.chess.model;

import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeMethod;

import com.haundrix.games.model.RectangularBoard;
import com.haundrix.games.model.Piece.PieceColor;

public abstract class ChessTest {

    protected RectangularBoard board;
    protected ChessGameState gameState;
    protected ChessGame game;
    protected Rook r1, r2, R1, R2;
    protected Knight n1, n2, N1, N2, N3, N4;
    protected Bishop b1, b2, B1, B2, B3, B4;
    protected Queen q1, Q1, Q2, Q3;
    protected King k, K;
    protected Pawn p1, p2, p3, p4, p5, p6, p7, p8, P1, P2, P3, P4, P5, P6, P7, P8, P9;
    
    @BeforeMethod
    public void setUpChessTest() {
        board = new RectangularBoard(ChessGame.ROW_COUNT, ChessGame.COLUMN_COUNT);
        gameState = new ChessGameState(board, PieceColor.WHITE, null);
        
        r1 = new Rook(PieceColor.BLACK, gameState, true);
        r2 = new Rook(PieceColor.BLACK, gameState, true);
        R1 = new Rook(PieceColor.WHITE, gameState, true);
        R2 = new Rook(PieceColor.WHITE, gameState, true);
        
        n1 = new Knight(PieceColor.BLACK, gameState);
        n2 = new Knight(PieceColor.BLACK, gameState);
        N1 = new Knight(PieceColor.WHITE, gameState);
        N2 = new Knight(PieceColor.WHITE, gameState);
        N3 = new Knight(PieceColor.WHITE, gameState);
        N4 = new Knight(PieceColor.WHITE, gameState);
        
        b1 = new Bishop(PieceColor.BLACK, gameState);
        b2 = new Bishop(PieceColor.BLACK, gameState);
        B1 = new Bishop(PieceColor.WHITE, gameState);
        B2 = new Bishop(PieceColor.WHITE, gameState);
        B3 = new Bishop(PieceColor.WHITE, gameState);
        B4 = new Bishop(PieceColor.WHITE, gameState);
        
        q1 = new Queen(PieceColor.BLACK, gameState);
        Q1 = new Queen(PieceColor.WHITE, gameState);
        Q2 = new Queen(PieceColor.WHITE, gameState);
        Q3 = new Queen(PieceColor.WHITE, gameState);
        
        k = new King(PieceColor.BLACK, gameState, true);
        K = new King(PieceColor.WHITE, gameState, true);
        
        p1 = new Pawn(PieceColor.BLACK, gameState);
        p2 = new Pawn(PieceColor.BLACK, gameState);
        p3 = new Pawn(PieceColor.BLACK, gameState);
        p4 = new Pawn(PieceColor.BLACK, gameState);
        p5 = new Pawn(PieceColor.BLACK, gameState);
        p6 = new Pawn(PieceColor.BLACK, gameState);
        p7 = new Pawn(PieceColor.BLACK, gameState);
        p8 = new Pawn(PieceColor.BLACK, gameState);
        P1 = new Pawn(PieceColor.WHITE, gameState);
        P2 = new Pawn(PieceColor.WHITE, gameState);
        P3 = new Pawn(PieceColor.WHITE, gameState);
        P4 = new Pawn(PieceColor.WHITE, gameState);
        P5 = new Pawn(PieceColor.WHITE, gameState);
        P6 = new Pawn(PieceColor.WHITE, gameState);
        P7 = new Pawn(PieceColor.WHITE, gameState);
        P8 = new Pawn(PieceColor.WHITE, gameState);
        P9 = new Pawn(PieceColor.WHITE, gameState);
    }
    
    @AfterMethod
    public void tearDownChessTest() {
        board = null;
        gameState = null;
        game = null;
        r1 = r2 = R1 = R2 = null;
        n1 = n2 = N1 = N2 = N3 = N4 = null;
        b1 = b2 = B1 = B2 = B3 = B4 = null;
        q1 = Q1 = Q2 = Q3 = null;
        k = K = null;
        p1 = p2 = p3 = p4 = p5 = p6 = p7 = p8 = P1 = P2 = P3 = P4 = P5 = P6 = P7 = P8 = P9 = null;
    }
    
    protected void setUpGame(ChessPiece[][] boardPieces) {
        for(int row = 0; row < ChessGame.ROW_COUNT; row++) {
            for(int column = 0; column < ChessGame.COLUMN_COUNT; column++) {
                board.setPiece(boardPieces[row][column], row, column);
            }
        }
        
        game = new ChessGame(gameState, 0, 0);
    }
    
}
