/*
Copyright (C) 2009, 2010 Alex Ignácio da Silva

This file is part of Haundrix Games.

Haundrix Games is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License version 3 as published by the
Free Software Foundation, with additional permission under section 7 to combine
it with independent works licensed under selected Free/Libre Open Source Software licenses.
Please see the full text of the license that is distributed along with this file.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
*/
package com.haundrix.games.chess.model;

import org.testng.Assert;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import com.haundrix.games.model.Move;
import com.haundrix.games.model.Square;

public class ForsythEdwardsNotationTest {

    private Move[] moves;
    private String[] positionDescriptions;
    
    private ChessGame game;
    
    @BeforeMethod
    public void setUp() {
        moves = new Move[] {
            new Move(new Square(6, 4), new Square(4, 4)),  //  1. e4
            new Move(new Square(1, 0), new Square(3, 0)),  //  1.      a5
            new Move(new Square(6, 3), new Square(4, 3)),  //  2. d4
            new Move(new Square(0, 0), new Square(2, 0)),  //  2.      Ra6
            new Move(new Square(4, 4), new Square(3, 4)),  //  3. e5
            new Move(new Square(1, 5), new Square(3, 5)),  //  3.      f5
            new Move(new Square(3, 4), new Square(2, 5)),  //  4. exf6 e.p.
            new Move(new Square(0, 6), new Square(2, 5)),  //  4.      Nxf6
            new Move(new Square(7, 5), new Square(4, 2)),  //  5. Bc4
            new Move(new Square(1, 4), new Square(3, 4)),  //  5.      e5
            new Move(new Square(7, 2), new Square(3, 6)),  //  6. Bg5
            new Move(new Square(0, 5), new Square(4, 1)),  //  6.      Bb4+
            new Move(new Square(7, 1), new Square(5, 2)),  //  7. Nc3
            new Move(new Square(1, 3), new Square(2, 3)),  //  7.      d6
            new Move(new Square(7, 3), new Square(4, 6)),  //  8. Qg4
            new Move(new Square(0, 2), new Square(4, 6)),  //  8.      Bxg4
            new Move(new Square(7, 6), new Square(5, 7)),  //  9. Nh3
            new Move(new Square(0, 1), new Square(2, 2)),  //  9.      Nc6
            new Move(new Square(3, 6), new Square(4, 7)),  // 10. Bh4
            new Move(new Square(2, 0), new Square(0, 0)),  // 10.      Ra8
            new Move(new Square(4, 7), new Square(5, 6)),  // 11. Bg3
            new Move(new Square(0, 3), new Square(1, 3)),  // 11.      Qd7
            new Move(new Square(7, 4), new Square(7, 6)),  // 12. 0-0
            new Move(new Square(0, 4), new Square(1, 4)),  // 12.      Ke7
            new Move(new Square(7, 5), new Square(7, 4)),  // 13. Rfe1
            new Move(new Square(1, 4), new Square(0, 4)),  // 13.      Ke8
        };
        
        positionDescriptions = new String[] {
            "rnbqkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNBQKBNR w KQkq - 0 1",
            "rnbqkbnr/pppppppp/8/8/4P3/8/PPPP1PPP/RNBQKBNR b KQkq e3 0 1",
            "rnbqkbnr/1ppppppp/8/p7/4P3/8/PPPP1PPP/RNBQKBNR w KQkq a6 0 2",
            "rnbqkbnr/1ppppppp/8/p7/3PP3/8/PPP2PPP/RNBQKBNR b KQkq d3 0 2",
            "1nbqkbnr/1ppppppp/r7/p7/3PP3/8/PPP2PPP/RNBQKBNR w KQk - 1 3",
            "1nbqkbnr/1ppppppp/r7/p3P3/3P4/8/PPP2PPP/RNBQKBNR b KQk - 0 3",
            "1nbqkbnr/1pppp1pp/r7/p3Pp2/3P4/8/PPP2PPP/RNBQKBNR w KQk f6 0 4",
            "1nbqkbnr/1pppp1pp/r4P2/p7/3P4/8/PPP2PPP/RNBQKBNR b KQk - 0 4",
            "1nbqkb1r/1pppp1pp/r4n2/p7/3P4/8/PPP2PPP/RNBQKBNR w KQk - 0 5",
            "1nbqkb1r/1pppp1pp/r4n2/p7/2BP4/8/PPP2PPP/RNBQK1NR b KQk - 1 5",
            "1nbqkb1r/1ppp2pp/r4n2/p3p3/2BP4/8/PPP2PPP/RNBQK1NR w KQk e6 0 6",
            "1nbqkb1r/1ppp2pp/r4n2/p3p1B1/2BP4/8/PPP2PPP/RN1QK1NR b KQk - 1 6",
            "1nbqk2r/1ppp2pp/r4n2/p3p1B1/1bBP4/8/PPP2PPP/RN1QK1NR w KQk - 2 7",
            "1nbqk2r/1ppp2pp/r4n2/p3p1B1/1bBP4/2N5/PPP2PPP/R2QK1NR b KQk - 3 7",
            "1nbqk2r/1pp3pp/r2p1n2/p3p1B1/1bBP4/2N5/PPP2PPP/R2QK1NR w KQk - 0 8",
            "1nbqk2r/1pp3pp/r2p1n2/p3p1B1/1bBP2Q1/2N5/PPP2PPP/R3K1NR b KQk - 1 8",
            "1n1qk2r/1pp3pp/r2p1n2/p3p1B1/1bBP2b1/2N5/PPP2PPP/R3K1NR w KQk - 0 9",
            "1n1qk2r/1pp3pp/r2p1n2/p3p1B1/1bBP2b1/2N4N/PPP2PPP/R3K2R b KQk - 1 9",
            "3qk2r/1pp3pp/r1np1n2/p3p1B1/1bBP2b1/2N4N/PPP2PPP/R3K2R w KQk - 2 10",
            "3qk2r/1pp3pp/r1np1n2/p3p3/1bBP2bB/2N4N/PPP2PPP/R3K2R b KQk - 3 10",
            "r2qk2r/1pp3pp/2np1n2/p3p3/1bBP2bB/2N4N/PPP2PPP/R3K2R w KQk - 4 11",
            "r2qk2r/1pp3pp/2np1n2/p3p3/1bBP2b1/2N3BN/PPP2PPP/R3K2R b KQk - 5 11",
            "r3k2r/1ppq2pp/2np1n2/p3p3/1bBP2b1/2N3BN/PPP2PPP/R3K2R w KQk - 6 12",
            "r3k2r/1ppq2pp/2np1n2/p3p3/1bBP2b1/2N3BN/PPP2PPP/R4RK1 b k - 7 12",
            "r6r/1ppqk1pp/2np1n2/p3p3/1bBP2b1/2N3BN/PPP2PPP/R4RK1 w - - 8 13",
            "r6r/1ppqk1pp/2np1n2/p3p3/1bBP2b1/2N3BN/PPP2PPP/R3R1K1 b - - 9 13",
            "r3k2r/1ppq2pp/2np1n2/p3p3/1bBP2b1/2N3BN/PPP2PPP/R3R1K1 w - - 10 14",
        };
        
        game = new ChessGame();
    }

    @Test
    public void testGetPositionDescription() {
        ForsythEdwardsNotation notation = ForsythEdwardsNotation.getInstance();
        Assert.assertEquals(notation.getPositionDescription(game), positionDescriptions[0]);
        for(int i = 0; i < moves.length; i++) {
            game.move(moves[i]);
            Assert.assertEquals(notation.getPositionDescription(game), positionDescriptions[i + 1]);
        }
    }
    
    @Test
    public void testGetGame() {
        ForsythEdwardsNotation notation = ForsythEdwardsNotation.getInstance();
        ForsythEdwardsNotationTest.assertEquals(notation.getGame(positionDescriptions[0]), game);
        for(int i = 0; i < moves.length; i++) {
            game.move(moves[i]);
            ForsythEdwardsNotationTest.assertEquals(notation.getGame(positionDescriptions[i + 1]), game);
        }
    }

    private static void assertEquals(ChessGame actualGame, ChessGame expectedGame) {
        Assert.assertTrue(actualGame.getState().equalsIgnoreMoved(expectedGame.getState()));
        Assert.assertEquals(actualGame.getCurrentPlyCount(), expectedGame.getCurrentPlyCount());
        Assert.assertEquals(actualGame.getCurrentPlyClock(), expectedGame.getCurrentPlyClock());
    }
    
}
