/*
Copyright (C) 2009, 2010 Alex Ignácio da Silva

This file is part of Haundrix Games.

Haundrix Games is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License version 3 as published by the
Free Software Foundation, with additional permission under section 7 to combine
it with independent works licensed under selected Free/Libre Open Source Software licenses.
Please see the full text of the license that is distributed along with this file.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
*/
package com.haundrix.games.chess.model;

import org.apache.commons.lang.ArrayUtils;
import org.testng.Assert;
import org.testng.annotations.Test;

import com.haundrix.games.model.Square;
import com.haundrix.games.model.Piece.PieceColor;

public class IsSquareAttackedTest extends ChessTest {

    @Test
    public void testIsSquareAttacked() {
        ChessPiece[][] pieces = {
    // columns :  0     1     2     3     4     5     6     7
    /* row 0 */ {null, null, null, null,  k  , null, null, null},  // row 0
    /* row 1 */ {null, null,  p1 , null, null, null, null, null},  // row 1
    /* row 2 */ {null, null, null,  p2 , null, null, null, null},  // row 2
    /* row 3 */ {null, null, null, null,  P3 ,  p3 , null, null},  // row 3
    /* row 4 */ { P1 , null,  n1 , null, null, null,  Q1 , null},  // row 4
    /* row 5 */ {null, null, null, null, null, null, null, null},  // row 5
    /* row 6 */ {null, null, null,  P2 , null, null, null, null},  // row 6
    /* row 7 */ { R1 , null, null, null,  K  ,  B1 , null, null},  // row 7
    // columns :  0     1     2     3     4     5     6     7
        };
        setUpGame(pieces);
        gameState.setColorToMove(PieceColor.WHITE);
        gameState.setEnPassantSquare(new Square(2, 5));
        
        Square[] allAttackedSquares = {
            new Square(6, 0), new Square(5, 0), new Square(4, 0), new Square(7, 1),
            new Square(7, 2), new Square(7, 3), new Square(7, 4),
            
            new Square(3, 1),
            
            new Square(5, 2), new Square(5, 4),
            
            new Square(7, 3), new Square(7, 5), new Square(6, 3), new Square(6, 4),
            new Square(6, 5),
            
            new Square(2, 3), new Square(2, 5),
            
            new Square(6, 4), new Square(5, 3), new Square(4, 2), new Square(6, 6),
            new Square(5, 7),
            
            new Square(4, 2), new Square(4, 3), new Square(4, 4), new Square(4, 5),
            new Square(4, 7), new Square(7, 6), new Square(6, 6), new Square(5, 6),
            new Square(3, 6), new Square(2, 6), new Square(1, 6), new Square(0, 6),
            new Square(3, 5), new Square(5, 7), new Square(7, 3), new Square(6, 4),
            new Square(5, 5), new Square(3, 7),
        };
        assertAllAttackedSquares(allAttackedSquares, gameState.getColorToMove());
        
        gameState.setColorToMove(PieceColor.BLACK);
        gameState.setEnPassantSquare(null);
        
        allAttackedSquares = new Square[] {
            new Square(2, 1), new Square(3, 0), new Square(5, 0), new Square(6, 1),
            new Square(6, 3), new Square(5, 4), new Square(3, 4), new Square(2, 3),
            
            new Square(2, 1), new Square(2, 3),
            
            new Square(3, 2), new Square(3, 4),
            
            new Square(4, 4), new Square(4, 6),
            
            new Square(0, 3), new Square(0, 5), new Square(1, 3), new Square(1, 4),
            new Square(1, 5),
        };
        assertAllAttackedSquares(allAttackedSquares, gameState.getColorToMove());
    }

    private void assertAllAttackedSquares(Square[] allAttackedSquares, PieceColor attackingPieceColor) {
        for(Square square : board) {
            Assert.assertEquals(game.isSquareAttacked(square, attackingPieceColor), ArrayUtils.contains(allAttackedSquares, square));
        }
    }
    
}
