/*
Copyright (C) 2009, 2010 Alex Ignácio da Silva

This file is part of Haundrix Games.

Haundrix Games is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License version 3 as published by the
Free Software Foundation, with additional permission under section 7 to combine
it with independent works licensed under selected Free/Libre Open Source Software licenses.
Please see the full text of the license that is distributed along with this file.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
*/
package com.haundrix.games.model;

import java.util.List;

import org.testng.Assert;
import org.testng.annotations.Test;

public abstract class MoveNotationTest {
    
    protected static final String MOVE_BY_MOVE_TEST_DATA_PROVIDER = "moveByMoveTestDataProvider";
    protected static final String ALL_MOVES_TEST_DATA_PROVIDER = "allMovesTestDataProvider";

    protected Game game;
    protected IMoveNotation notation;
    
    @Test(dataProvider=MOVE_BY_MOVE_TEST_DATA_PROVIDER)
    public void testMoveByMove(Move[] moves, String[] moveDescriptions, String[][] invalidMoveDescriptions) {
        for(int i = 0; i < moves.length; i++) {
            Assert.assertEquals(notation.getMoveDescription(moves[i], game), moveDescriptions[i]);
            Assert.assertEquals(notation.getMove(moveDescriptions[i], game), moves[i]);
            if(invalidMoveDescriptions != null) {
                for(String invalidMoveDescription : invalidMoveDescriptions[i]) {
                    boolean illegalArgumentExceptionThrown = false;
                    try {
                        notation.getMove(invalidMoveDescription, game);
                    } catch(IllegalArgumentException e) {
                        illegalArgumentExceptionThrown = true;
                    }
                    String errorMessage = "Failed to detect invalid move: " + invalidMoveDescription + ".";
                    Assert.assertTrue(illegalArgumentExceptionThrown, errorMessage);
                }
            }
            game.move(moves[i]);
            Assert.assertEquals(game.getLastMoveDescription(notation), moveDescriptions[i]);
        }
    }
    
    @Test(dataProvider=ALL_MOVES_TEST_DATA_PROVIDER)
    public void testAllMoves(Move[] moves, String[] moveDescriptions) {
        for(Move move : moves) {
            game.move(move);
        }
        
        List<String> actualMoveDescriptions = game.getMoveDescriptions(notation);
        Assert.assertEquals(actualMoveDescriptions.size(), moveDescriptions.length);
        for(int i = 0; i < actualMoveDescriptions.size(); i++) {
            Assert.assertEquals(actualMoveDescriptions.get(i), moveDescriptions[i]);
        }
    }
    
}
