/*
Copyright (C) 2009, 2010 Alex Ignácio da Silva

This file is part of Haundrix Games.

Haundrix Games is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License version 3 as published by the
Free Software Foundation, with additional permission under section 7 to combine
it with independent works licensed under selected Free/Libre Open Source Software licenses.
Please see the full text of the license that is distributed along with this file.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
*/
package com.haundrix.graphics;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Display;
import org.testng.Assert;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import com.haundrix.tests.TestGroups;

@Test(groups=TestGroups.GUI_TESTS)
public abstract class BorderTest {
    
    protected Display display;

    // The actual border implementation must be provided by the concrete subclass test.
    protected IBorder border;

    @BeforeMethod
    public void setUpBorderTests() {
        display = new Display();
    }

    @AfterMethod
    public void tearDownBorderTests() {
        display.dispose();
    }

    @Test(description="Tests whether a given border implementation does not draw over the region of its inset.")
    public void testInsetNotOverdrawn() {
        final int width = 100;
        final int height = 300;
        Image image = new Image(display, width, height);
        GC gc = new GC(image);
        
        gc.setBackground(display.getSystemColor(SWT.COLOR_WHITE));
        gc.fillRectangle(0, 0, width, height);
        int insetX = 23;
        int insetY = 18;
        int insetWidth = 17;
        int insetHeight = 51;
        border.setInset(new Rectangle(insetX, insetY, insetWidth, insetHeight));
        border.setThickness(7);
        border.draw(gc);
        
        ImageData imageData = image.getImageData();
        for(int row = insetY; row < insetY + insetHeight; row++) {
            for(int column = insetX; column < insetX + insetWidth; column++) {
                int pixel = imageData.getPixel(column, row);
                RGB pixelColor = imageData.palette.getRGB(pixel);
                Assert.assertEquals(pixelColor.red, 0xFF,
                        "Error! Incorrect inset pixel red component at row = " + row + " and column = " + column);
                Assert.assertEquals(pixelColor.green, 0xFF,
                        "Error! Incorrect inset pixel green component at row = " + row + " and column = " + column);
                Assert.assertEquals(pixelColor.blue, 0xFF,
                        "Error! Incorrect inset pixel blue component at row = " + row + " and column = " + column);
                Assert.assertEquals(imageData.getAlpha(column, row), 255,
                        "Error! The inset pixel is not opaque as originally set at row = " + row +
                        " and column = " + column);
            }
        }
        
        gc.dispose();
        image.dispose();
    }
    
}
