/*
Copyright (C) 2009, 2010 Alex Ignácio da Silva

This file is part of Haundrix Games.

Haundrix Games is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License version 3 as published by the
Free Software Foundation, with additional permission under section 7 to combine
it with independent works licensed under selected Free/Libre Open Source Software licenses.
Please see the full text of the license that is distributed along with this file.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
*/
package com.haundrix.tests;

import java.io.File;
import java.io.IOException;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import org.apache.commons.io.FileUtils;
import org.apache.commons.io.LineIterator;

public class HaundrixAssert {

    public static <T> void areEqualIgnoringOrder(T[] actual, T[] expected) {
        HaundrixAssert.areEqualIgnoringOrder(actual, expected, null);
    }

    public static <T> void areEqualIgnoringOrder(T[] actual, T[] expected, String errorMessage) {
        if(actual == expected) {
            return;
        }
        
        if(actual == null) {
            HaundrixAssert.fail(errorMessage, "Actual array is null.");
        }
        if(expected == null) {
            HaundrixAssert.fail(errorMessage, "Expected array is null.");
        }
        
        List<T> actualList = Arrays.asList(actual);
        List<T> expectedList = Arrays.asList(expected);
        for(T expectedElement : expectedList) {
            if(!actualList.contains(expectedElement)) {
                String message = "This element was expected, but did not appear in the actual array: " +
                    expectedElement + ".";
                HaundrixAssert.fail(errorMessage, message);
            }
        }
        for(T actualElement : actualList) {
            if(!expectedList.contains(actualElement)) {
                String message = "This element was not expected, but did appear in the actual array: " +
                    actualElement + ".";
                HaundrixAssert.fail(errorMessage, message);
            }
        }
        for(T expectedElement : expectedList) {
            int nExpected = Collections.frequency(expectedList, expectedElement);
            int nActual = Collections.frequency(actualList, expectedElement);
            if(nActual != nExpected) {
                String message = "The following element was expected to appear " + nExpected +
                    " times, but was found " + nActual + " time in the actual array: " + expectedElement;
                HaundrixAssert.fail(errorMessage, message);
            }
        }
    }

    public static void contentEqualsIgnoringLineTerminator(File actualFile, File benchmarkFile) throws IOException {
        String errorMessage = "Error comparing " + actualFile.getAbsolutePath() + " against " + benchmarkFile.getAbsolutePath() + ".\n";
        
        LineIterator actualLines = FileUtils.lineIterator(actualFile);
        LineIterator benchmarkLines = FileUtils.lineIterator(benchmarkFile);
        int lineNumber = 1;
        while(actualLines.hasNext()) {
            String actualLine = actualLines.nextLine();
            
            if(!benchmarkLines.hasNext()) {
                errorMessage += "Unexpected end of benchmark file at line " + lineNumber + ".";
                throw new AssertionError(errorMessage);
            }
            String benchmarkLine = benchmarkLines.nextLine();
            
            if(!actualLine.equals(benchmarkLine)) {
                errorMessage += "Files don't match at line " + lineNumber + ".\n";
                errorMessage += "Expected:\n";
                errorMessage += benchmarkLine + "\n";
                errorMessage += "but found:\n";
                errorMessage += actualLine;
                throw new AssertionError(errorMessage);
            }
            
            lineNumber++;
        }
        
        if(benchmarkLines.hasNext()) {
            errorMessage += "Unexpected end of actual file at line " + lineNumber + ".";
            throw new AssertionError(errorMessage);
        }
    }
    
    private static String fail(String customMessage, String standardMessage) {
        String errorMessage = customMessage == null ? standardMessage :
            customMessage + "\n" + standardMessage;
        throw new AssertionError(errorMessage);
    }
    
}
