/*
Copyright (C) 2009, 2010 Alex Ignácio da Silva

This file is part of Haundrix Games.

Haundrix Games is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License version 3 as published by the
Free Software Foundation, with additional permission under section 7 to combine
it with independent works licensed under selected Free/Libre Open Source Software licenses.
Please see the full text of the license that is distributed along with this file.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
*/
package com.haundrix.utils;

import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;

import javax.imageio.ImageIO;

import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.graphics.RGB;
import org.testng.Assert;
import org.testng.annotations.DataProvider;
import org.testng.annotations.Test;

import com.haundrix.tests.TestGroups;
import com.haundrix.tests.TestUtils;

@Test(groups=TestGroups.GUI_TESTS)
public class ImageUtilsTest {
    
    @DataProvider(name="imageFiles")
    public Object[][] getImageFiles() {
        final File imageDir = TestUtils.getDataDir(ImageUtilsTest.class);
        return new File[][] {
                {new File(imageDir, "whiteKnight.png")},
                {new File(imageDir, "screenshot.png")},
                {new File(imageDir, "screenshot.bmp")},
                {new File(imageDir, "screenshot.gif")},
                {new File(imageDir, "screenshotGrayscale.gif")},
                {new File(imageDir, "screenshot.jpeg")},
        };
    }

    @Test(dataProvider="imageFiles")
    public void testConvertToSwt(File testImageFile) throws IOException {
        BufferedImage awtImage = ImageIO.read(testImageFile);
        ImageData swtImage = ImageUtils.convertToSwt(awtImage);
        compareImages(awtImage, swtImage);
    }

    @Test(dataProvider="imageFiles")
    public void testConvertToAwt(File testImageFile) throws FileNotFoundException {
        ImageData swtImage = new ImageData(new FileInputStream(testImageFile));
        BufferedImage awtImage = ImageUtils.convertToAwt(swtImage);
        compareImages(awtImage, swtImage);
    }
    
    private void compareImages(BufferedImage awtImage, ImageData swtImage) {
        int awtWidth = awtImage.getWidth();
        int awtHeight = awtImage.getHeight();
        int swtWidth = swtImage.width;
        int swtHeight = swtImage.height;
        Assert.assertEquals(swtWidth, awtWidth);
        Assert.assertEquals(swtHeight, awtHeight);
        
        int[] awtPixels = new int[awtWidth * awtHeight];
        awtImage.getRGB(0, 0, awtWidth, awtHeight, awtPixels, 0, awtWidth);
        int[] swtPixels = new int[swtWidth * swtHeight];
        swtImage.getPixels(0, 0, swtPixels.length, swtPixels, 0);
        byte[] swtAlphas = new byte[swtWidth * swtHeight];
        swtImage.getAlphas(0, 0, swtAlphas.length, swtAlphas, 0);
        for(int i = 0; i < awtPixels.length; i++) {
            RGB swtRgb = swtImage.palette.getRGB(swtPixels[i]);
            
            int awtRed = (awtPixels[i] >> 16) & 0xFF;
            int swtRed = swtRgb.red;
            Assert.assertEquals(swtRed, awtRed, "Error! Red component of image pixels do not match.");
            
            int awtGreen = (awtPixels[i] >> 8) & 0xFF;
            int swtGreen = swtRgb.green;
            Assert.assertEquals(swtGreen, awtGreen, "Error! Green component of image pixels do not match.");
            
            int awtBlue = awtPixels[i] & 0xFF;
            int swtBlue = swtRgb.blue;
            Assert.assertEquals(swtBlue, awtBlue, "Error! Blue component of image pixels do not match.");
            
            byte awtAlpha = (byte) ((awtPixels[i] >> 24) & 0xFF);
            Assert.assertEquals(swtAlphas[i], awtAlpha, "Error! Alpha component of image pixels do not match.");
        }
    }

}
