/*
Copyright (C) 2009, 2010 Alex Ignácio da Silva

This file is part of Haundrix Games.

Haundrix Games is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License version 3 as published by the
Free Software Foundation, with additional permission under section 7 to combine
it with independent works licensed under selected Free/Libre Open Source Software licenses.
Please see the full text of the license that is distributed along with this file.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
*/
package com.haundrix.animation;

import com.haundrix.utils.IDisposable;

public class CompositeMoveCallback extends MoveCallback {

    private MoveCallback moveCallback1;
    private MoveCallback moveCallback2;
    
    // The contract is that callback1 is called before callback2.
    public CompositeMoveCallback(MoveCallback moveCallback1, MoveCallback moveCallback2) {
        this.moveCallback1 = moveCallback1;
        this.moveCallback2 = moveCallback2;
    }

    @Override
    public void started(Mover mover) {
        moveCallback1.started(mover);
        moveCallback2.started(mover);
    }
    
    @Override
    public void movableDisposed(Mover mover, IDisposable disposable) {
        moveCallback1.movableDisposed(mover, disposable);
        moveCallback2.movableDisposed(mover, disposable);
    }

    @Override
    public void moved(Mover mover) {
        moveCallback1.moved(mover);
        moveCallback2.moved(mover);
    }

    @Override
    public void aboutToEnd(Mover mover) {
        moveCallback1.aboutToEnd(mover);
        moveCallback2.aboutToEnd(mover);
    }

    @Override
    public void ended(Mover mover) {
        moveCallback1.ended(mover);
        moveCallback2.ended(mover);
    }

}
