/*
Copyright (C) 2009, 2010 Alex Ignácio da Silva

This file is part of Haundrix Games.

Haundrix Games is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License version 3 as published by the
Free Software Foundation, with additional permission under section 7 to combine
it with independent works licensed under selected Free/Libre Open Source Software licenses.
Please see the full text of the license that is distributed along with this file.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
 */
package com.haundrix.animation;

import org.jdesktop.animation.timing.TimingTarget;

public class ConstantAccelerationTimingTarget implements TimingTarget {

    private float acceleration;
    private float deceleration;

    private TimingTarget timingTarget;

    public ConstantAccelerationTimingTarget(float acceleration, float deceleration, TimingTarget timingTarget) {
        this.acceleration = acceleration;
        this.deceleration = deceleration;
        this.timingTarget = timingTarget;
    }

    public float getAcceleration() {
        return acceleration;
    }

    public float getDeceleration() {
        return deceleration;
    }

    public TimingTarget getTimingTarget() {
        return timingTarget;
    }

    @Override
    public void begin() {
        timingTarget.begin();
    }

    @Override
    public void end() {
        timingTarget.end();
    }

    @Override
    public void repeat() {
        timingTarget.repeat();
    }

    // This code has been stolen from the Timing Framework library.
    // (see org.jdesktop.animation.timing.Animator.timingEventPreprocessor(float fraction))
    @Override
    public void timingEvent(float fraction) {
        // first, take care of acceleration/deceleration factors
        if(acceleration != 0 || deceleration != 0.0f) {
            // see the SMIL 2.0 specification for details on this calculation
            float runRate = 1.0f / (1.0f - acceleration / 2.0f - deceleration / 2.0f);
            if(fraction < acceleration) {
                float averageRunRate = runRate * (fraction / acceleration) / 2;
                fraction *= averageRunRate;
            } else if(fraction > (1.0f - deceleration)) {
                // time spent in deceleration portion
                float tdec = fraction - (1.0f - deceleration);
                // proportion of tdec to total deceleration time
                float pdec = tdec / deceleration;
                fraction = runRate * (1.0f - (acceleration / 2) - deceleration + tdec * (2 - pdec) / 2);
            } else {
                fraction = runRate * (fraction - (acceleration / 2));
            }
            // clamp fraction to [0,1] since above calculations may cause rounding errors
            if(fraction < 0) {
                fraction = 0;
            } else if(fraction > 1.0f) {
                fraction = 1.0f;
            }
        }

        timingTarget.timingEvent(fraction);
    }

}
