/*
Copyright (C) 2009, 2010 Alex Ignácio da Silva

This file is part of Haundrix Games.

Haundrix Games is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License version 3 as published by the
Free Software Foundation, with additional permission under section 7 to combine
it with independent works licensed under selected Free/Libre Open Source Software licenses.
Please see the full text of the license that is distributed along with this file.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
*/
package com.haundrix.animation;

import org.jdesktop.animation.timing.TimingTarget;

import com.haundrix.utils.IDisposable;

public class DisposableAwareTimingTarget implements TimingTarget {

    private IDisposable disposable;
    private TimingTarget timingTarget;
    
    public DisposableAwareTimingTarget(IDisposable disposable, TimingTarget timingTarget) {
        this.disposable = disposable;
        this.timingTarget = timingTarget;
    }

    public IDisposable getDisposable() {
        return disposable;
    }

    public TimingTarget getTimingTarget() {
        return timingTarget;
    }

    @Override
    public void begin() {
        if(!disposable.isDisposed()) {
            timingTarget.begin();
        }
    }

    @Override
    public void end() {
        if(!disposable.isDisposed()) {
            timingTarget.end();
        }
    }

    @Override
    public void repeat() {
        if(!disposable.isDisposed()) {
            timingTarget.repeat();
        }
    }

    @Override
    public void timingEvent(float fraction) {
        if(!disposable.isDisposed()) {
            timingTarget.timingEvent(fraction);
        }
    }

}
