/*
Copyright (C) 2009, 2010 Alex Ignácio da Silva

This file is part of Haundrix Games.

Haundrix Games is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License version 3 as published by the
Free Software Foundation, with additional permission under section 7 to combine
it with independent works licensed under selected Free/Libre Open Source Software licenses.
Please see the full text of the license that is distributed along with this file.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
*/
package com.haundrix.animation;

import org.eclipse.swt.widgets.Display;
import org.jdesktop.animation.timing.TimingTarget;
import org.jdesktop.animation.timing.TimingTargetAdapter;
import org.jdesktop.animation.timing.interpolation.PropertySetter;

import com.haundrix.utils.IDisposeListener;
import com.haundrix.utils.IDisposable;

public class Fader {

    private SwtAnimator animator;
    
    public Fader(Display display, final IFadable fadable, float alpha, int duration,
            final FadeCallback fadeCallback) {
        PropertySetter propertySetter = new PropertySetter(fadable, "alpha", alpha);
        animator = new SwtAnimator(display, duration, propertySetter);
        final FadeCallback usedFadeCallback = fadeCallback != null ? fadeCallback : new FadeCallback();
        
        final IDisposeListener disposeListener = new IDisposeListener() {
            @Override
            public void disposed(IDisposable disposable) {
                animator.cancel();
                usedFadeCallback.fadableDisposed(disposable);
            }
        };
        if(fadable instanceof IDisposable) {
            ((IDisposable) fadable).addDisposeListener(disposeListener);
        }
        
        animator.addTarget(new TimingTargetAdapter() {

            @Override
            public void begin() {
                usedFadeCallback.started();
            }

            @Override
            public void end() {
                if(fadable instanceof IDisposable) {
                    ((IDisposable) fadable).removeDisposeListener(disposeListener);
                }
                usedFadeCallback.ended();
            }
            
        });
    }
    
    public static TimingTarget createTimingTarget(IFadable fadable, float alpha, float acceleration, float deceleration) {
        PropertySetter propertySetter = new PropertySetter(fadable, "alpha", alpha);
        TimingTarget timingTarget = new ConstantAccelerationTimingTarget(acceleration, deceleration, propertySetter);
        return fadable instanceof IDisposable ?
                new DisposableAwareTimingTarget((IDisposable) fadable, timingTarget) :
                timingTarget;
    }
    
    public float getAcceleration() {
        return animator.getAcceleration();
    }
    
    public void setAcceleration(float acceleration) {
        animator.setAcceleration(acceleration);
    }
    
    public float getDeceleration() {
        return animator.getDeceleration();
    }
    
    public void setDeceleration(float deceleration) {
        animator.setDeceleration(deceleration);
    }
    
    public void fade() {
        animator.start();
    }
    
}
