/*
Copyright (C) 2009, 2010 Alex Ignácio da Silva

This file is part of Haundrix Games.

Haundrix Games is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License version 3 as published by the
Free Software Foundation, with additional permission under section 7 to combine
it with independent works licensed under selected Free/Libre Open Source Software licenses.
Please see the full text of the license that is distributed along with this file.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
*/
package com.haundrix.animation;

import org.eclipse.swt.widgets.Display;
import org.jdesktop.animation.timing.TimingTargetAdapter;
import org.jdesktop.animation.timing.Animator.RepeatBehavior;
import org.jdesktop.animation.timing.interpolation.Interpolator;

import com.haundrix.graphics.Highlight;
import com.haundrix.utils.IDisposeListener;
import com.haundrix.utils.IDisposable;

public class Highlighter {

    private SwtAnimator animator;
    
    public Highlighter(Display display, final IHighlightable highlightable, final Highlight highlight,
            int duration) {
        animator = new SwtAnimator(display, duration);
        
        final IDisposeListener disposeListener = new IDisposeListener() {
            @Override
            public void disposed(IDisposable disposable) {
                animator.cancel();
            }
        };
        if(highlightable instanceof IDisposable) {
            ((IDisposable) highlightable).addDisposeListener(disposeListener);
        }
        
        animator.addTarget(new TimingTargetAdapter() {

            @Override
            public void begin() {
                highlightable.setHighlight(highlight);
                highlightable.setHighlightIntensity(0.0F);
            }

            @Override
            public void timingEvent(float fraction) {
                highlightable.setHighlightIntensity(highlight.getIntensity() * fraction);
            }
            
            @Override
            public void end() {
                if(highlightable instanceof IDisposable) {
                    ((IDisposable) highlightable).removeDisposeListener(disposeListener);
                }
            }

        });
    }
    
    public float getAcceleration() {
        return animator.getAcceleration();
    }
    
    public void setAcceleration(float acceleration) {
        animator.setAcceleration(acceleration);
    }
    
    public float getDeceleration() {
        return animator.getDeceleration();
    }
    
    public void setDeceleration(float deceleration) {
        animator.setDeceleration(deceleration);
    }
    
    public Interpolator getInterpolator() {
        return animator.getInterpolator();
    }

    public void setInterpolator(Interpolator interpolator) {
        animator.setInterpolator(interpolator);
    }
    
    public void highlight() {
        animator.start();
    }

    public void pulse(int nPulses) {
        animator.setRepeatBehavior(RepeatBehavior.REVERSE);
        animator.setRepeatCount(nPulses * 2.0);
        animator.start();
    }
    
}
