/*
Copyright (C) 2009, 2010 Alex Ignácio da Silva

This file is part of Haundrix Games.

Haundrix Games is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License version 3 as published by the
Free Software Foundation, with additional permission under section 7 to combine
it with independent works licensed under selected Free/Libre Open Source Software licenses.
Please see the full text of the license that is distributed along with this file.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
 */
package com.haundrix.games.chess.model;

import com.haundrix.games.model.AlgebraicNotation;
import com.haundrix.utils.BidirectionalMap;

public class ChessNotation extends AlgebraicNotation {

    private BidirectionalMap<Class<? extends ChessPiece>, Character> pieceCharacters;

    public ChessNotation() {
        super(ChessGame.ROW_COUNT, ChessGame.COLUMN_COUNT);
        
        pieceCharacters = new BidirectionalMap<Class<? extends ChessPiece>, Character>();
        pieceCharacters.put(Rook.class, 'R');
        pieceCharacters.put(Knight.class, 'N');
        pieceCharacters.put(Bishop.class, 'B');
        pieceCharacters.put(Queen.class, 'Q');
        pieceCharacters.put(King.class, 'K');
        pieceCharacters.put(Pawn.class, 'P');
    }
    
    public ChessNotation(BidirectionalMap<Class<? extends ChessPiece>, Character> pieceCharacters) {
        super(ChessGame.ROW_COUNT, ChessGame.COLUMN_COUNT);
        
        this.pieceCharacters = pieceCharacters;
    }
    
    public Character getPieceDescription(Class<? extends ChessPiece> pieceClass) {
        return pieceCharacters.getValue(pieceClass);
    }

    public Class<? extends ChessPiece> getPieceClass(Character pieceDescription) {
        return pieceCharacters.getKey(pieceDescription);
    }

}