/*
Copyright (C) 2009, 2010 Alex Ignácio da Silva

This file is part of Haundrix Games.

Haundrix Games is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License version 3 as published by the
Free Software Foundation, with additional permission under section 7 to combine
it with independent works licensed under selected Free/Libre Open Source Software licenses.
Please see the full text of the license that is distributed along with this file.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
*/
package com.haundrix.games.chess.model;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import com.haundrix.games.model.Game;
import com.haundrix.games.model.IMoveNotation;
import com.haundrix.games.model.Move;
import com.haundrix.games.model.SimpleMoveNotation;
import com.haundrix.utils.BidirectionalMap;

public class ChessSimpleMoveNotation extends SimpleMoveNotation implements IMoveNotation {
    
    private static final BidirectionalMap<Class<? extends PromoteablePiece>, String> PROMOTION_PIECES =
        new BidirectionalMap<Class<? extends PromoteablePiece>, String>();
    static {
        PROMOTION_PIECES.put(Rook.class, "r");
        PROMOTION_PIECES.put(Knight.class, "n");
        PROMOTION_PIECES.put(Bishop.class, "b");
        PROMOTION_PIECES.put(Queen.class, "q");
    }
    
    private static final Pattern MOVE_DESCRIPTION_PATTERN = Pattern.compile("([a-z]\\d+[a-z]\\d+)([rnbq])?");

    public ChessSimpleMoveNotation() {
        super(ChessGame.ROW_COUNT, ChessGame.COLUMN_COUNT);
    }

    @Override
    public Move getMove(String description, Game game) {
        Matcher matcher = MOVE_DESCRIPTION_PATTERN.matcher(description);
        if(!matcher.matches()) {
            throw new IllegalArgumentException("Error! Invalid move description: " + description + ".");
        }
        
        Move move = super.getMove(matcher.group(1), game);
        if(matcher.group(2) != null) {
            Class<? extends PromoteablePiece> promotedPiece = PROMOTION_PIECES.getKey(matcher.group(2));
            move = new PromotionMove(move, promotedPiece);
        }
        
        return move;
    }

    @Override
    public String getMoveDescription(Move move, Game game) {
        String moveDescription = super.getMoveDescription(move, game);
        if(move instanceof PromotionMove) {
            Class<? extends PromoteablePiece> promotedPiece = ((PromotionMove) move).getPromotedPiece();
            moveDescription += PROMOTION_PIECES.getValue(promotedPiece);
        }
        
        return moveDescription;
    }

}
