/*
Copyright (C) 2009, 2010 Alex Ignácio da Silva

This file is part of Haundrix Games.

Haundrix Games is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License version 3 as published by the
Free Software Foundation, with additional permission under section 7 to combine
it with independent works licensed under selected Free/Libre Open Source Software licenses.
Please see the full text of the license that is distributed along with this file.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
*/
package com.haundrix.games.chess.model;

import java.util.ArrayList;
import java.util.List;

import com.haundrix.games.model.Move;
import com.haundrix.games.model.Square;

public class Queen extends PromoteablePiece {

    public Queen(PieceColor color, ChessGameState gameState) {
        super(color, gameState);
    }

    @Override
    public boolean canMove(Move move) {
        Square origin = move.getOrigin();
        Square destination = move.getDestination();
        return getBoard().isFreeColumn(origin, destination) || getBoard().isFreeRank(origin, destination) ||
            getBoard().isFreeDiagonal(origin, destination);
    }

    @Override
    public List<Square> getDestinationsFrom(Square origin) {
        List<Square> destinations = new ArrayList<Square>();
        destinations.addAll(getBoard().getFreeColumnSquaresFrom(origin));
        destinations.addAll(getBoard().getFreeRankSquaresFrom(origin));
        destinations.addAll(getBoard().getFreeDiagonalSquaresFrom(origin));
        return destinations;
    }

}
