/*
Copyright (C) 2009, 2010 Alex Ignácio da Silva

This file is part of Haundrix Games.

Haundrix Games is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License version 3 as published by the
Free Software Foundation, with additional permission under section 7 to combine
it with independent works licensed under selected Free/Libre Open Source Software licenses.
Please see the full text of the license that is distributed along with this file.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
*/
package com.haundrix.games.chess.model.playing;

enum CECPFeature {
    PING("ping", Boolean.class),
    SET_BOARD("setboard", Boolean.class),
    PLAY_OTHER("playother", Boolean.class),
    SAN("san", Boolean.class),
    USER_MOVE("usermove", Boolean.class),
    TIME("time", Boolean.class),
    DRAW("draw", Boolean.class),
    SIG_INT("sigint", Boolean.class),
    SIG_TERM("sigterm", Boolean.class),
    REUSE("reuse", Boolean.class),
    ANALYZE("analyze", Boolean.class),
    MY_NAME("myname", String.class),
    VARIANTS("variants", String.class),
    COLORS("colors", Boolean.class),
    ICS("ics", Boolean.class),
    NAME("name", Boolean.class),
    PAUSE("pause", Boolean.class),
    NPS("nps", Boolean.class),
    DEBUG("debug", Boolean.class),
    MEMORY("memory", Boolean.class),
    SMP("smp", Boolean.class),
    EGT("egt", String.class),
    OPTION("option", String.class),
    DONE("done", Integer.class);
    
    private String name;
    private Class<?> type;
    
    private CECPFeature(String name, Class<?> type) {
        this.name = name;
        this.type = type;
    }

    public String getName() {
        return name;
    }
    
    public Class<?> getType() {
        return type;
    }

    public String getFriendlyTypeName() {
        if(type == String.class) {
            return "string";
        } else if(type == Integer.class) {
            return "integer";
        } else if(type == Boolean.class) {
            return "boolean";
        } else {
            return "unknown";
        }
    }
    
    public boolean validate(Object value) {
        if(type == String.class) {
            return value instanceof String;
        } else if(type == Integer.class) {
            return value instanceof Integer;
        } else if(type == Boolean.class) {
            if(!(value instanceof Integer)) {
                return false;
            }
            
            Integer integerValue = (Integer) value;
            return (integerValue == 0) || (integerValue == 1);
        } else {
            return false;
        }
    }
    
    public static CECPFeature getFeature(String name) {
        for(CECPFeature feature : CECPFeature.values()) {
            if(feature.getName().equals(name)) {
                return feature;
            }
        }
        
        return null;
    }
    
}
