/*
Copyright (C) 2009, 2010 Alex Ignácio da Silva

This file is part of Haundrix Games.

Haundrix Games is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License version 3 as published by the
Free Software Foundation, with additional permission under section 7 to combine
it with independent works licensed under selected Free/Libre Open Source Software licenses.
Please see the full text of the license that is distributed along with this file.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
*/
package com.haundrix.games.chess.model.playing;

import com.haundrix.games.model.playing.Engine;
import com.haundrix.games.model.playing.EngineConfig;
import com.haundrix.games.model.playing.ICommandLine;

public class CraftyEngine extends Engine {

    private static final String PROMPT = "White(1): ";

    /*
     * Crafty is forced to move if its time falls below
     * 20% of the initially allotted time for the move.
     * (also respecting the min/max bounds below)
     */
    private static final double TIME_FRACTIONAL_THRESHOLD = 0.20;  // 20%
    private static final long MIN_TIME_THRESHOLD =  100L;  // in milliseconds
    private static final long MAX_TIME_THRESHOLD = 2000L;  // in milliseconds

    @Override
    public void postConfigure(EngineConfig config, ICommandLine commandLine) {
        if(!(config instanceof CraftyEngineConfig)) {
            throw new IllegalArgumentException("Error! Invalid engine config.");
        }
        
        int skill = ((CraftyEngineConfig) config).getSkill();
        commandLine.outputAtPrompt(PROMPT, "skill " + skill);
        
        commandLine.outputAtPrompt(PROMPT, "xboard");
    }

    @Override
    public long calculateTimeThreshold(long availableTime) {
        long timeThreshold = (long) (availableTime * TIME_FRACTIONAL_THRESHOLD);
        timeThreshold = Math.max(timeThreshold, MIN_TIME_THRESHOLD);
        timeThreshold = Math.min(timeThreshold, MAX_TIME_THRESHOLD);
        
        return timeThreshold;
    }

}
