/*
Copyright (C) 2009, 2010 Alex Ignácio da Silva

This file is part of Haundrix Games.

Haundrix Games is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License version 3 as published by the
Free Software Foundation, with additional permission under section 7 to combine
it with independent works licensed under selected Free/Libre Open Source Software licenses.
Please see the full text of the license that is distributed along with this file.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
*/
package com.haundrix.games.chess.pgn;

import com.haundrix.games.model.GameOutcome;

enum PgnGameOutcome {

    BLACK_WINS("0-1") {
        @Override
        public GameOutcome getGameOutcome() {
            return GameOutcome.BLACK_WINS;
        }
    },
    WHITE_WINS("1-0") {
        @Override
        public GameOutcome getGameOutcome() {
            return GameOutcome.WHITE_WINS;
        }
    },
    DRAW("1/2-1/2") {
        @Override
        public GameOutcome getGameOutcome() {
            return GameOutcome.DRAW;
        }
    },
    OTHER("*") {
        @Override
        public GameOutcome getGameOutcome() {
            return null;
        }
    };
    
    private String description;
    
    private PgnGameOutcome(String description) {
        this.description = description;
    }
    
    public String getDescription() {
        return description;
    }
    
    public abstract GameOutcome getGameOutcome();
    
    public static PgnGameOutcome fromGameOutcome(GameOutcome gameOutcome) {
        if(gameOutcome == GameOutcome.BLACK_WINS) {
            return BLACK_WINS;
        } else if(gameOutcome == GameOutcome.WHITE_WINS) {
            return WHITE_WINS;
        } else if(gameOutcome == GameOutcome.DRAW) {
            return DRAW;
        } else {
            return OTHER;
        }
    }
    
    public static PgnGameOutcome parse(String description) {
        for(PgnGameOutcome gameOutcome : PgnGameOutcome.values()) {
            if(gameOutcome.getDescription().equals(description)) {
                return gameOutcome;
            }
        }
        
        throw new IllegalArgumentException("Error! Unknown PgnGameOutcome description: " + description + ".");
    }
    
}
