/*
Copyright (C) 2009, 2010 Alex Ignácio da Silva

This file is part of Haundrix Games.

Haundrix Games is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License version 3 as published by the
Free Software Foundation, with additional permission under section 7 to combine
it with independent works licensed under selected Free/Libre Open Source Software licenses.
Please see the full text of the license that is distributed along with this file.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
*/
package com.haundrix.games.controller;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowData;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Text;

import com.haundrix.games.model.GameInfo;
import com.haundrix.swt.CenteredDialog;
import com.haundrix.swt.CompositeDateControl;
import com.haundrix.swt.SwtUtils;

public class GameInfoDialog extends CenteredDialog {
    
    private static final int PLAYER_TEXT_LENGTH = 30;
    private static final int PLAYER_RATING_SPACING = 10;

    private GameInfo initialGameInfo;
    
    private GameInfo gameInfo;
    
    private Text whiteNameText;
    private Spinner whiteEloSpinner;
    private Text blackNameText;
    private Spinner blackEloSpinner;
    private Text eventText;
    private Text siteText;
    private CompositeDateControl dateControl;
    private Spinner roundSpinner;

    public GameInfoDialog(Shell parent, GameInfo gameInfo) {
        super(parent);
        
        initialGameInfo = gameInfo;
    }

    public GameInfo getGameInfo() {
        openAndRunLoop();
        
        return gameInfo;
    }
    
    @Override
    protected void createWidgets() {
        dialog.setLayout(new GridLayout(4, false));
        
        Label whiteNameLabel = new Label(dialog, SWT.NONE);
        whiteNameLabel.setLayoutData(new GridData(GridData.END, GridData.CENTER, false, false));
        whiteNameLabel.setText("White Name");  // TODO externalize this
        
        whiteNameText = new Text(dialog, SWT.SINGLE | SWT.BORDER);
        GridData whiteNameTextLayoutData = new GridData();
        whiteNameTextLayoutData.horizontalAlignment = GridData.BEGINNING;
        whiteNameTextLayoutData.verticalAlignment = GridData.CENTER;
        whiteNameTextLayoutData.widthHint  = SwtUtils.computeWidth(whiteNameText, PLAYER_TEXT_LENGTH);
        whiteNameText.setLayoutData(whiteNameTextLayoutData);
        
        Label whiteEloLabel = new Label(dialog, SWT.NONE);
        whiteEloLabel.setText("White ELO");  // TODO externalize this
        GridData whiteEloLabelLayoutData = new GridData();
        whiteEloLabelLayoutData.horizontalAlignment = GridData.END;
        whiteEloLabelLayoutData.verticalAlignment = GridData.CENTER;
        whiteEloLabelLayoutData.horizontalIndent = PLAYER_RATING_SPACING;
        whiteEloLabel.setLayoutData(whiteEloLabelLayoutData);
        
        whiteEloSpinner = new Spinner(dialog, SWT.BORDER);
        configureEloSpinner(whiteEloSpinner);
        whiteEloSpinner.setSelection(0);
        whiteEloSpinner.setLayoutData(new GridData(GridData.BEGINNING, GridData.CENTER, false, false));
        
        Label blackNameLabel = new Label(dialog, SWT.NONE);
        blackNameLabel.setLayoutData(new GridData(GridData.END, GridData.CENTER, false, false));
        blackNameLabel.setText("Black Name");  // TODO externalize this
        
        blackNameText = new Text(dialog, SWT.SINGLE | SWT.BORDER);
        GridData blackNameTextLayoutData = new GridData();
        blackNameTextLayoutData.horizontalAlignment = GridData.BEGINNING;
        blackNameTextLayoutData.verticalAlignment = GridData.CENTER;
        blackNameTextLayoutData.widthHint  = SwtUtils.computeWidth(whiteNameText, PLAYER_TEXT_LENGTH);
        blackNameText.setLayoutData(blackNameTextLayoutData);
        
        Label blackEloLabel = new Label(dialog, SWT.NONE);
        blackEloLabel.setText("Black ELO");  // TODO externalize this
        GridData blackEloLabelLayoutData = new GridData();
        blackEloLabelLayoutData.horizontalAlignment = GridData.END;
        blackEloLabelLayoutData.verticalAlignment = GridData.CENTER;
        blackEloLabelLayoutData.horizontalIndent = PLAYER_RATING_SPACING;
        blackEloLabel.setLayoutData(blackEloLabelLayoutData);
        
        blackEloSpinner = new Spinner(dialog, SWT.BORDER);
        configureEloSpinner(blackEloSpinner);
        blackEloSpinner.setSelection(0);
        blackEloSpinner.setLayoutData(new GridData(GridData.BEGINNING, GridData.CENTER, false, false));
        
        Label eventLabel = new Label(dialog, SWT.NONE);
        eventLabel.setText("Event");  // TODO externalize this
        eventLabel.setLayoutData(new GridData(GridData.END, GridData.CENTER, false, false));
        
        eventText = new Text(dialog, SWT.SINGLE | SWT.BORDER);
        GridData eventTextLayoutData = new GridData();
        eventTextLayoutData.horizontalAlignment = GridData.FILL;
        eventTextLayoutData.verticalAlignment = GridData.CENTER;
        eventTextLayoutData.horizontalSpan = 3;
        eventText.setLayoutData(eventTextLayoutData);
        
        Label siteLabel = new Label(dialog, SWT.NONE);
        siteLabel.setText("Site");  // TODO externalize this
        siteLabel.setLayoutData(new GridData(GridData.END, GridData.CENTER, false, false));
        
        siteText = new Text(dialog, SWT.SINGLE | SWT.BORDER);
        GridData siteTextLayoutData = new GridData();
        siteTextLayoutData.horizontalAlignment = GridData.FILL;
        siteTextLayoutData.verticalAlignment = GridData.CENTER;
        siteTextLayoutData.horizontalSpan = 3;
        siteText.setLayoutData(siteTextLayoutData);
        
        Label dateLabel = new Label(dialog, SWT.NONE);
        dateLabel.setText("Date");  // TODO externalize this
        dateLabel.setLayoutData(new GridData(GridData.END, GridData.CENTER, false, false));
        
        dateControl = new CompositeDateControl(dialog);
        dateControl.setLayoutData(new GridData(GridData.BEGINNING, GridData.CENTER, false, false));
        
        Label roundLabel = new Label(dialog, SWT.NONE);
        roundLabel.setText("Round");  // TODO externalize this
        roundLabel.setLayoutData(new GridData(GridData.END, GridData.CENTER, false, false));
        
        roundSpinner = new Spinner(dialog, SWT.BORDER);
        roundSpinner.setMinimum(0);
        roundSpinner.setMaximum(100);
        roundSpinner.setIncrement(1);
        roundSpinner.setPageIncrement(5);
        roundSpinner.setSelection(0);
        roundSpinner.setLayoutData(new GridData(GridData.BEGINNING, GridData.CENTER, false, false));
        
        Composite buttonsPanel = new Composite(dialog, SWT.NONE);
        buttonsPanel.setLayoutData(new GridData(GridData.END, GridData.CENTER, false, false, 4, 1));
        fillButtonsPanel(buttonsPanel);
        
        initializeWidgets();
    }

    private void configureEloSpinner(Spinner eloSpinner) {
        eloSpinner.setMinimum(0);
        eloSpinner.setMaximum(9999);
        eloSpinner.setIncrement(10);
        eloSpinner.setPageIncrement(100);
    }
    
    private void fillButtonsPanel(Composite buttonsPanel) {
        RowLayout buttonsPanelLayout = new RowLayout();
        buttonsPanelLayout.pack = false;
        buttonsPanelLayout.spacing = 15;
        buttonsPanel.setLayout(buttonsPanelLayout);
        
        Button cancelButton = new Button(buttonsPanel, SWT.PUSH);
        cancelButton.setText("&Cancel");  // TODO externalize this
        cancelButton.setLayoutData(new RowData());
        cancelButton.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                dispose();
            }
        });
        
        Button okButton = new Button(buttonsPanel, SWT.PUSH);
        okButton.setText("&OK");  // TODO externalize this
        okButton.setLayoutData(new RowData());
        okButton.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                readGameInfo();
                dispose();
            }
        });
    }

    private void initializeWidgets() {
        if(initialGameInfo != null) {
            if(initialGameInfo.getWhitePlayer() != null) {
                whiteNameText.setText(initialGameInfo.getWhitePlayer());
            }
            if(initialGameInfo.getWhiteElo() != -1) {
                whiteEloSpinner.setSelection(initialGameInfo.getWhiteElo());
            } else {
                whiteEloSpinner.setSelection(0);
            }
            if(initialGameInfo.getBlackPlayer() != null) {
                blackNameText.setText(initialGameInfo.getBlackPlayer());
            }
            if(initialGameInfo.getBlackElo() != -1) {
                blackEloSpinner.setSelection(initialGameInfo.getBlackElo());
            } else {
                blackEloSpinner.setSelection(0);
            }
            if(initialGameInfo.getEvent() != null) {
                eventText.setText(initialGameInfo.getEvent());
            }
            if(initialGameInfo.getSite() != null) {
                siteText.setText(initialGameInfo.getSite());
            }
            if(initialGameInfo.getDate() != null) {
                dateControl.setDate(initialGameInfo.getDate());
            }
            if(initialGameInfo.getRound() != -1) {
                roundSpinner.setSelection(initialGameInfo.getRound());
            } else {
                roundSpinner.setSelection(0);
            }
        }
    }

    private void readGameInfo() {
        gameInfo = new GameInfo();
        
        gameInfo.setWhitePlayer(whiteNameText.getText().trim());
        if(whiteEloSpinner.getSelection() > 0) {
            gameInfo.setWhiteElo(whiteEloSpinner.getSelection());
        }
        gameInfo.setBlackPlayer(blackNameText.getText().trim());
        if(blackEloSpinner.getSelection() > 0) {
            gameInfo.setBlackElo(blackEloSpinner.getSelection());
        }
        gameInfo.setEvent(eventText.getText().trim());
        gameInfo.setSite(siteText.getText().trim());
        gameInfo.setDate(dateControl.getDate());
        if(roundSpinner.getSelection() > 0) {
            gameInfo.setRound(roundSpinner.getSelection());
        }
    }

}
