/*
Copyright (C) 2009, 2010 Alex Ignácio da Silva

This file is part of Haundrix Games.

Haundrix Games is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License version 3 as published by the
Free Software Foundation, with additional permission under section 7 to combine
it with independent works licensed under selected Free/Libre Open Source Software licenses.
Please see the full text of the license that is distributed along with this file.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
*/
package com.haundrix.games.controller;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;

import com.haundrix.games.model.Game;

public class GameNavigationKeyListener extends KeyAdapter implements KeyListener {

    private static final int SHORT_STEP_SIZE = 2;
    private static final int LONG_STEP_SIZE = 10;
    
    private Game game;
    
    public GameNavigationKeyListener() {
    }

    public GameNavigationKeyListener(Game game) {
        this.game = game;
    }

    public Game getGame() {
        return game;
    }

    public void setGame(Game game) {
        this.game = game;
    }

    @Override
    public void keyPressed(KeyEvent e) {
        if(game != null) {
            int currentPosition = game.getCurrentPosition();
            switch(e.keyCode) {
            case(SWT.ARROW_LEFT) :
                if(!game.isAtBegin()) {
                    game.goTo(currentPosition - 1);
                }
                break;
            case(SWT.ARROW_RIGHT) :
                if(!game.isAtEnd()) {
                    game.goTo(currentPosition + 1);
                }
                break;
            case(SWT.ARROW_UP) :
                if(!game.isAtBegin()) {
                    int nextPosition = Math.max(currentPosition - SHORT_STEP_SIZE, 0);
                    game.goTo(nextPosition);
                }
                break;
            case(SWT.ARROW_DOWN) :
                if(!game.isAtEnd()) {
                    int nextPosition = Math.min(currentPosition + SHORT_STEP_SIZE, game.getLastPosition());
                    game.goTo(nextPosition);
                }
                break;
            case(SWT.PAGE_UP) :
                if(!game.isAtBegin()) {
                    int nextPosition = Math.max(currentPosition - LONG_STEP_SIZE, 0);
                    game.goTo(nextPosition);
                }
                break;
            case(SWT.PAGE_DOWN) :
                if(!game.isAtEnd()) {
                    int nextPosition = Math.min(currentPosition + LONG_STEP_SIZE, game.getLastPosition());
                    game.goTo(nextPosition);
                }
                break;
            case(SWT.HOME) :
                if(!game.isAtBegin()) {
                    game.goToBegin();
                }
                break;
            case(SWT.END) :
                if(!game.isAtEnd()) {
                    game.goToEnd();
                }
                break;
            }
        }
    }

}
