/*
Copyright (C) 2009, 2010 Alex Ignácio da Silva

This file is part of Haundrix Games.

Haundrix Games is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License version 3 as published by the
Free Software Foundation, with additional permission under section 7 to combine
it with independent works licensed under selected Free/Libre Open Source Software licenses.
Please see the full text of the license that is distributed along with this file.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
*/
package com.haundrix.games.controller;

import java.util.Comparator;

import com.haundrix.games.model.Square;

// Compares two given squares based on their angles relative to a reference square.
// If both squares are at the same angle, then they are compared according to their distance
// from the reference square.
// By definition, a square at the noon position has a relative angle of zero.
class SquareComparator implements Comparator<Square> {
    
    private Square referenceSquare;
    
    public SquareComparator(Square referenceSquare) {
        this.referenceSquare = referenceSquare;
    }

    @Override
    public int compare(Square square1, Square square2) {
        int result = getAngle(square1).compareTo(getAngle(square2));
        if(result != 0) {
            return result;
        } else {
            return getDistance(square1).compareTo(getDistance(square2));
        }
    }
    
    private Double getAngle(Square square) {
        double x = referenceSquare.getRow() - square.getRow();
        double y = square.getColumn() - referenceSquare.getColumn();
        double angle = Math.atan2(y, x);
        if(angle < 0.0) {
            angle += 2.0 * Math.PI;
        }
        return angle;
    }

    private Double getDistance(Square square) {
        double x = square.getColumn() - referenceSquare.getColumn();
        double y = referenceSquare.getRow() - square.getRow();
        return Math.sqrt(x * x + y * y);
    }

}
