/*
Copyright (C) 2009, 2010 Alex Ignácio da Silva

This file is part of Haundrix Games.

Haundrix Games is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License version 3 as published by the
Free Software Foundation, with additional permission under section 7 to combine
it with independent works licensed under selected Free/Libre Open Source Software licenses.
Please see the full text of the license that is distributed along with this file.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
*/
package com.haundrix.games.controller;

import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.widgets.Control;
import org.jdesktop.animation.timing.TimingEventListener;
import org.jdesktop.animation.timing.TimingSource;

import com.haundrix.animation.SwtTimingSource;

public class WaitingCursorAnimator {
    
    private static final int FRAME_DURATION = 1000;  // in milliseconds

    private SwtTimingSource timingSource;
    private int frameNumber;
    
    public WaitingCursorAnimator(final CursorTheme cursorTheme, final Control control) {
        timingSource = new SwtTimingSource(control.getDisplay(), FRAME_DURATION);
        
        timingSource.addEventListener(new TimingEventListener() {
            @Override
            public void timingSourceEvent(TimingSource timingSource) {
                if(!control.isDisposed()) {
                    control.setCursor(cursorTheme.getWaitingCursor(frameNumber));
                    frameNumber++;
                    if(frameNumber >= cursorTheme.getWaitingCursorFrameCount()) {
                        frameNumber = 0;
                    }
                }
            }
        });
        
        control.addDisposeListener(new DisposeListener() {
            @Override
            public void widgetDisposed(DisposeEvent e) {
                if(timingSource.isRunning()) {
                    timingSource.stop();
                }
            }
        });
    }
    
    public void start() {
        frameNumber = 0;
        timingSource.start();
    }
    
    public void stop() {
        timingSource.stop();
    }

    public boolean isRunning() {
        return timingSource.isRunning();
    }
    
}
