/*
Copyright (C) 2009, 2010 Alex Ignácio da Silva

This file is part of Haundrix Games.

Haundrix Games is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License version 3 as published by the
Free Software Foundation, with additional permission under section 7 to combine
it with independent works licensed under selected Free/Libre Open Source Software licenses.
Please see the full text of the license that is distributed along with this file.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
*/
package com.haundrix.games.model;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

// Note: this implementation does not support boards with more than 26 columns.
public class AlgebraicNotation {
    
    private static final int MAX_BOARD_COLUMN_COUNT = 26;
    
    private static final Pattern SQUARE_DESCRIPTION_PATTERN = Pattern.compile("([a-z])(\\d+)");
    
    private int boardRowCount;
    private int boardColumnCount;

    public AlgebraicNotation(int boardRowCount, int boardColumnCount) {
        if(boardRowCount < 1) {
            throw new IllegalArgumentException("Error! Board row count out of range.");
        }
        if((boardColumnCount < 1) || (boardColumnCount > MAX_BOARD_COLUMN_COUNT)) {
            throw new IllegalArgumentException("Error! Board column count out of range.");
        }
        
        this.boardRowCount = boardRowCount;
        this.boardColumnCount = boardColumnCount;
    }

    public String getColumnDescription(Square square) {
        int column = square.getColumn();
        if((column < 0) || (column >= boardColumnCount)) {
            throw new IllegalArgumentException("Error! Square column out of range.");
        }
        
        return Character.toString((char) ('a' + column));
    }
    
    public int getColumn(String columnDescription) {
        return (int) (columnDescription.toCharArray()[0] - 'a');
    }

    public String getRowDescription(Square square) {
        int row = square.getRow();
        if((row < 0) || (row >= boardRowCount)) {
            throw new IllegalArgumentException("Error! Square row out of range.");
        }
        
        return Integer.toString(boardRowCount - row);
    }

    public int getRow(String rowDescription) {
        return boardRowCount - Integer.parseInt(rowDescription);
    }

    public String getSquareDescription(Square square) {
        return getColumnDescription(square) + getRowDescription(square);
    }

    public Square getSquare(String squareDescription) {
        Matcher matcher = SQUARE_DESCRIPTION_PATTERN.matcher(squareDescription);
        if(!matcher.matches()) {
            throw new IllegalArgumentException("Error! Invalid square description.");
        }
        
        int column = getColumn(matcher.group(1));
        int row = getRow(matcher.group(2));
        return new Square(row, column);
    }

}
