/*
Copyright (C) 2009, 2010 Alex Ignácio da Silva

This file is part of Haundrix Games.

Haundrix Games is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License version 3 as published by the
Free Software Foundation, with additional permission under section 7 to combine
it with independent works licensed under selected Free/Libre Open Source Software licenses.
Please see the full text of the license that is distributed along with this file.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
*/
package com.haundrix.games.model;

import org.joda.time.LocalDate;

import com.haundrix.utils.Utils;

public class GameInfo {

    private String whitePlayer;
    private String blackPlayer;
    private int whiteElo = -1;
    private int blackElo = -1;
    
    private String event;
    private String site;
    private LocalDate date;
    private int round = -1;
    
    private Game game;
    
    public String getWhitePlayer() {
        return whitePlayer;
    }

    public void setWhitePlayer(String whitePlayer) {
        this.whitePlayer = whitePlayer;
        
        triggerChanged();
    }

    public String getBlackPlayer() {
        return blackPlayer;
    }

    public void setBlackPlayer(String blackPlayer) {
        this.blackPlayer = blackPlayer;
        
        triggerChanged();
    }

    public int getWhiteElo() {
        return whiteElo;
    }

    public void setWhiteElo(int whiteElo) {
        this.whiteElo = whiteElo;
        
        triggerChanged();
    }

    public int getBlackElo() {
        return blackElo;
    }

    public void setBlackElo(int blackElo) {
        this.blackElo = blackElo;
        
        triggerChanged();
    }

    public String getEvent() {
        return event;
    }

    public void setEvent(String event) {
        this.event = event;
        
        triggerChanged();
    }

    public String getSite() {
        return site;
    }

    public void setSite(String site) {
        this.site = site;
        
        triggerChanged();
    }

    public LocalDate getDate() {
        return date;
    }

    public void setDate(LocalDate date) {
        this.date = date;
        
        triggerChanged();
    }

    public int getRound() {
        return round;
    }

    public void setRound(int round) {
        this.round = round;
        
        triggerChanged();
    }

    @Override
    public boolean equals(Object otherObject) {
        if((otherObject == null) || !(otherObject instanceof GameInfo)) {
            return false;
        }
        
        GameInfo otherGameInfo = (GameInfo) otherObject;
        
        if(!Utils.nullableEquals(whitePlayer, otherGameInfo.getWhitePlayer())) {
            return false;
        }
        
        if(whiteElo != otherGameInfo.getWhiteElo()) {
            return false;
        }
        
        if(!Utils.nullableEquals(blackPlayer, otherGameInfo.getBlackPlayer())) {
            return false;
        }
        
        if(blackElo != otherGameInfo.getBlackElo()) {
            return false;
        }
        
        if(!Utils.nullableEquals(event, otherGameInfo.getEvent())) {
            return false;
        }
        
        if(!Utils.nullableEquals(site, otherGameInfo.getSite())) {
            return false;
        }
        
        if(!Utils.nullableEquals(date, otherGameInfo.getDate())) {
            return false;
        }
        
        return round == otherGameInfo.getRound();
    }

    Game getGame() {
        return game;
    }

    void setGame(Game game) {
        this.game = game;
    }
    
    private void triggerChanged() {
        if(game != null) {
            game.notifyInfoChanged();
        }
    }
    
}
