/*
Copyright (C) 2009, 2010 Alex Ignácio da Silva

This file is part of Haundrix Games.

Haundrix Games is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License version 3 as published by the
Free Software Foundation, with additional permission under section 7 to combine
it with independent works licensed under selected Free/Libre Open Source Software licenses.
Please see the full text of the license that is distributed along with this file.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
*/
package com.haundrix.games.model;

/*
 * This class is immutable by contract.
 */
public class GameResult {
    
    private GameOutcome outcome;
    private IGameResultReason resultReason;
    
    public GameResult(GameOutcome outcome) {
        this(outcome, null);
    }

    public GameResult(GameOutcome outcome, IGameResultReason resultReason) {
        if(outcome == null) {
            throw new IllegalArgumentException("Error! Game outcome can't be null.");
        }
        
        this.outcome = outcome;
        this.resultReason = resultReason;
    }

    public GameOutcome getOutcome() {
        return outcome;
    }

    public IGameResultReason getResultReason() {
        return resultReason;
    }

    @Override
    public boolean equals(Object otherObject) {
        if((otherObject == null) || !(otherObject instanceof GameResult)) {
            return false;
        }
        
        GameResult otherResult = (GameResult) otherObject;
        
        if(!outcome.equals(otherResult.getOutcome())) {
            return false;
        }
        
        return resultReason == null ? otherResult.getResultReason() == null :
            resultReason.equals(otherResult.getResultReason());
    }

}
