/*
Copyright (C) 2009, 2010 Alex Ignácio da Silva

This file is part of Haundrix Games.

Haundrix Games is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License version 3 as published by the
Free Software Foundation, with additional permission under section 7 to combine
it with independent works licensed under selected Free/Libre Open Source Software licenses.
Please see the full text of the license that is distributed along with this file.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
*/
package com.haundrix.games.model;

import com.haundrix.games.model.Piece.PieceColor;

public class GameState implements Cloneable {

    protected RectangularBoard board;
    protected PieceColor colorToMove;

    public GameState(RectangularBoard board, PieceColor colorToMove) {
        this.board = board;
        this.colorToMove = colorToMove;
    }

    public RectangularBoard getBoard() {
        return board;
    }
    
    public PieceColor getColorToMove() {
        return colorToMove;
    }

    public void setColorToMove(PieceColor colorToMove) {
        this.colorToMove = colorToMove;
    }

    public void switchColorToMove() {
        colorToMove = colorToMove.getOppositeColor();
    }
    
    @Override
    public boolean equals(Object otherObject) {
        if((otherObject == null) || !(otherObject instanceof GameState)) {
            return false;
        }
        
        GameState otherGameState = (GameState) otherObject;
        
        return otherGameState.board.equals(board) && otherGameState.colorToMove.equals(colorToMove);
    }

    @Override
    public GameState clone() {
        GameState clone;
        try {
            clone = (GameState) super.clone();
        } catch(CloneNotSupportedException e) {
            throw new RuntimeException(e);
        }
        
        RectangularBoard clonedBoard = new RectangularBoard(board.getRowCount(), board.getColumnCount());
        for(Square square : board) {
            Piece piece = board.getPiece(square);
            if(piece != null) {
                clonedBoard.setPiece(piece.clone(), square);
            }
        }
        clone.board = clonedBoard;
        
        return clone;
    }

}
