/*
Copyright (C) 2009, 2010 Alex Ignácio da Silva

This file is part of Haundrix Games.

Haundrix Games is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License version 3 as published by the
Free Software Foundation, with additional permission under section 7 to combine
it with independent works licensed under selected Free/Libre Open Source Software licenses.
Please see the full text of the license that is distributed along with this file.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
*/
package com.haundrix.games.model;

import java.util.HashMap;
import java.util.Map;

public class Move implements Cloneable {

    private Square origin;
    private Square destination;

    private Map<Object, Object> unmoveInfos = new HashMap<Object, Object>();
    
    public Move(Square origin, Square destination) {
        this.origin = origin;
        this.destination = destination;
    }

    public Square getOrigin() {
        return origin;
    }

    public Square getDestination() {
        return destination;
    }

    @Override
    public String toString() {
        return "from (" + origin.getRow() + "," + origin.getColumn() +
            ") to (" + destination.getRow() + "," + destination.getColumn() + ")";
    }

    @Override
    public boolean equals(Object object) {
        if((object == null) || !(object instanceof Move)) {
            return false;
        }
        
        Move move = (Move) object;
        return origin.equals(move.getOrigin()) && destination.equals(move.getDestination());
    }

    @Override
    public Move clone() {
        Move clone;
        try {
            clone = (Move) super.clone();
        } catch(CloneNotSupportedException e) {
            throw new RuntimeException(e);
        }
        clone.unmoveInfos = new HashMap<Object, Object>();
        
        return clone;
    }

    public void addUnmoveInfo(Object unmoveInfoProperty, Object unmoveInfo) {
        unmoveInfos.put(unmoveInfoProperty, unmoveInfo);
    }

    public Object getUnmoveInfo(Object unmoveInfoProperty) {
        return unmoveInfos.get(unmoveInfoProperty);
    }

    public void clearUnmoveInfos() {
        unmoveInfos.clear();
    }
    
}
