/*
Copyright (C) 2009, 2010 Alex Ignácio da Silva

This file is part of Haundrix Games.

Haundrix Games is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License version 3 as published by the
Free Software Foundation, with additional permission under section 7 to combine
it with independent works licensed under selected Free/Libre Open Source Software licenses.
Please see the full text of the license that is distributed along with this file.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
*/
package com.haundrix.games.model;


public abstract class Piece implements Cloneable {

    // By definition, the white color is the color of the side that makes the initial move.
    // It may actually be a different color in some games, but that's a presentation issue
    // that doesn't concern the business layer.
    public enum PieceColor {
        
        BLACK {
            @Override
            public PieceColor getOppositeColor() {
                return WHITE;
            }
        },
        
        WHITE {
            @Override
            public PieceColor getOppositeColor() {
                return BLACK;
            }
        };

        public abstract PieceColor getOppositeColor();
        
    }

    protected PieceColor color;
    
    public Piece(PieceColor color) {
        this.color = color;
    }

    public PieceColor getColor() {
        return color;
    }

    @Override
    public boolean equals(Object object) {
        if((object == null) || !(object instanceof Piece)) {
            return false;
        }
        
        Piece piece = (Piece) object;
        return piece.getColor().equals(color);
    }

    @Override
    public Piece clone() {
        try {
            return (Piece) super.clone();
        } catch(CloneNotSupportedException e) {
            throw new RuntimeException(e);
        }
    }
    
}
