/*
Copyright (C) 2009, 2010 Alex Ignácio da Silva

This file is part of Haundrix Games.

Haundrix Games is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License version 3 as published by the
Free Software Foundation, with additional permission under section 7 to combine
it with independent works licensed under selected Free/Libre Open Source Software licenses.
Please see the full text of the license that is distributed along with this file.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
*/
package com.haundrix.games.model;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

/*
 * A simple move notation that specifies the origin 
 * and destination squares in algebraic notation.
 * Examples: e2e4, e7e5, g1f3, etc.
 */
public class SimpleMoveNotation extends AlgebraicNotation implements IMoveNotation {
    
    private static final Pattern MOVE_DESCRIPTION_PATTERN = Pattern.compile("([a-z]\\d+)([a-z]\\d+)");

    public SimpleMoveNotation(int boardRowCount, int boardColumnCount) {
        super(boardRowCount, boardColumnCount);
    }

    @Override
    public Move getMove(String description, Game game) {
        Matcher matcher = MOVE_DESCRIPTION_PATTERN.matcher(description);
        if(!matcher.matches()) {
            throw new IllegalArgumentException("Error! Invalid move description: " + description + ".");
        }
        
        Square origin = getSquare(matcher.group(1));
        Square destination = getSquare(matcher.group(2));
        return new Move(origin, destination);
    }

    @Override
    public String getMoveDescription(Move move, Game game) {
        return getSquareDescription(move.getOrigin()) + getSquareDescription(move.getDestination());
    }

}
