/*
Copyright (C) 2009, 2010 Alex Ignácio da Silva

This file is part of Haundrix Games.

Haundrix Games is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License version 3 as published by the
Free Software Foundation, with additional permission under section 7 to combine
it with independent works licensed under selected Free/Libre Open Source Software licenses.
Please see the full text of the license that is distributed along with this file.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
*/
package com.haundrix.games.model.playing;

import java.util.ArrayList;
import java.util.Deque;
import java.util.LinkedList;
import java.util.List;

public class EngineLog {
    
    public interface ILogListener {
        void messageLogged(String message);
    }

    public static final int MAX_SIZE = 300;

    private Deque<String> deque = new LinkedList<String>();
    
    private List<ILogListener> listeners = new ArrayList<ILogListener>();
    
    public synchronized void log(String message) {
        if(deque.size() > MAX_SIZE) {
            deque.removeFirst();
        }
        deque.addLast(message);
        
        synchronized(listeners) {
            for(ILogListener listener : listeners) {
                listener.messageLogged(message);
            }
        }
    }
    
    public synchronized String[] getMessages() {
        return deque.toArray(new String[0]);
    }
    
    // Atomically gets the current log and subscribes to subsequent messages.
    public synchronized String[] addListenerAndGetMessages(ILogListener listener) {
        synchronized(listeners) {
            listeners.add(listener);
        }
        
        return getMessages();
    }
    
    public void removeListener(ILogListener listener) {
        synchronized(listeners) {
            listeners.remove(listener);
        }
    }
    
}
