/*
Copyright (C) 2009, 2010 Alex Ignácio da Silva

This file is part of Haundrix Games.

Haundrix Games is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License version 3 as published by the
Free Software Foundation, with additional permission under section 7 to combine
it with independent works licensed under selected Free/Libre Open Source Software licenses.
Please see the full text of the license that is distributed along with this file.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
*/
package com.haundrix.games.model.playing;

import com.haundrix.games.model.GameResult;
import com.haundrix.games.model.Move;

public interface IPlayer {

    void go();
    
    void move(Move move);
    
    void setTime(ClockTime time);
    
    /**
     * Calculates the time threshold for forcing the player to move.
     * Players that take longer to respond to the <code>moveNow()</code>
     * method should return larger values, so that the GUI may call
     * the <code>moveNow()</code> method in a timely fashion to prevent
     * the player from forfeiting on time.
     * @param availableTime The available time for making the next move, in milliseconds.
     * @return The time threshold for forcing the player to move, in milliseconds.
     */
    long calculateTimeThreshold(long availableTime);
    
    void moveNow();
    
    void setResult(GameResult result);
    
    boolean isDisposed();
    
    void dispose();
    
}
