/*
Copyright (C) 2009, 2010 Alex Ignácio da Silva

This file is part of Haundrix Games.

Haundrix Games is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License version 3 as published by the
Free Software Foundation, with additional permission under section 7 to combine
it with independent works licensed under selected Free/Libre Open Source Software licenses.
Please see the full text of the license that is distributed along with this file.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
*/
package com.haundrix.games.rcp.editors;

import java.io.InputStream;

import org.apache.commons.io.IOUtils;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.TabFolder;
import org.eclipse.swt.widgets.TabItem;
import org.eclipse.swt.widgets.Text;

import com.haundrix.games.rcp.HaundrixImages;
import com.haundrix.games.rcp.HaundrixLog;

public class AboutDialog extends Dialog {

    private static final String LICENSE_FILE = "/rsc/branding/aboutLicense.txt";
    private static final String CREDITS_FILE = "/rsc/branding/aboutCredits.txt";

    public AboutDialog(Shell parent) {
        super(parent, SWT.APPLICATION_MODAL | SWT.TITLE | SWT.CLOSE | SWT.BORDER);
    }
    
    public void open() {
        Shell parent = getParent();
        
        Shell dialog = new Shell(parent, getStyle());
        dialog.setText("About Haundrix Chess");  // TODO externalize this
        dialog.setLayout(new FillLayout());
        
        TabFolder tabFolder = new TabFolder(dialog, SWT.TOP);
        createInfoTab(tabFolder);
        dialog.pack();
        createTextTab(tabFolder, "License", LICENSE_FILE);  // TODO externalize this
        createTextTab(tabFolder, "Credits", CREDITS_FILE);  // TODO externalize this
        tabFolder.setSelection(0);
        
        dialog.open();
        Display display = parent.getDisplay();
        while(!dialog.isDisposed()) {
            if(!display.readAndDispatch()) {
                display.sleep();
            }
        }
    }

    private void createInfoTab(TabFolder tabFolder) {
        TabItem tabItem = new TabItem(tabFolder, SWT.NONE);
        tabItem.setText("&Info");  // TODO externalize this
        
        Label label = new Label(tabFolder, SWT.HORIZONTAL);
        final Image aboutImage = HaundrixImages.ABOUT_IMAGE.createImage();
        label.setImage(aboutImage);
        label.addDisposeListener(new DisposeListener() {
            @Override
            public void widgetDisposed(DisposeEvent e) {
                aboutImage.dispose();
            }
        });
        
        tabItem.setControl(label);
    }

    private void createTextTab(TabFolder tabFolder, String title, String textFileName) {
        try {
            TabItem tabItem = new TabItem(tabFolder, SWT.NONE);
            tabItem.setText(title);
            
            Text text = new Text(tabFolder, SWT.MULTI | SWT.WRAP | SWT.V_SCROLL | SWT.READ_ONLY);
            InputStream textFile = AboutDialog.class.getResourceAsStream(textFileName);
            text.setText(IOUtils.toString(textFile, "UTF-8"));
            textFile.close();
            
            tabItem.setControl(text);
        } catch(Exception exception) {
            HaundrixLog.logError("Error creating about dialog.", exception);
        }
    }

}
