/*
Copyright (C) 2009, 2010 Alex Ignácio da Silva

This file is part of Haundrix Games.

Haundrix Games is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License version 3 as published by the
Free Software Foundation, with additional permission under section 7 to combine
it with independent works licensed under selected Free/Libre Open Source Software licenses.
Please see the full text of the license that is distributed along with this file.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
*/
package com.haundrix.games.rcp.editors;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Spinner;

import com.haundrix.games.model.playing.FischerClock;
import com.haundrix.games.model.playing.IClock;
import com.haundrix.games.model.playing.MovesPerTimeClock;
import com.haundrix.games.model.playing.TimePerMoveClock;
import com.haundrix.swt.FlipLayout;

class ClockControl extends Composite {
    
    private Combo timeControlCombo;
    
    // Fixed moves per time clock controls.
    private Spinner movesCountSpinner;
    private Spinner totalTimeSpinner;

    // Fischer clock controls.
    private Spinner fixedTimeSpinner;
    private Spinner timeIncrementSpinner;

    // Fixed time per move controls.
    private Spinner timePerMoveSpinner;

    public ClockControl(Composite parent) {
        super(parent, SWT.NONE);
        
        setLayout(new RowLayout(SWT.VERTICAL));
        
        timeControlCombo = new Combo(this, SWT.DROP_DOWN | SWT.READ_ONLY);
        timeControlCombo.add("Untimed");  // TODO externalize this
        timeControlCombo.add("Fixed moves per time");  // TODO externalize this
        timeControlCombo.add("Fixed time plus increment");  // TODO externalize this
        timeControlCombo.add("Fixed time per move");  // TODO externalize this
        timeControlCombo.select(0);
        
        Composite timeConfigurationPanel = new Composite(this, SWT.NONE);
        final FlipLayout flipLayout = new FlipLayout();
        timeConfigurationPanel.setLayout(flipLayout);
        
        new Composite(timeConfigurationPanel, SWT.NONE) {
            @Override
            public Point computeSize(int wHint, int hHint, boolean changed) {
                return new Point(1, 1);
            }
        };
        
        Composite page1 = new Composite(timeConfigurationPanel, SWT.NONE);
        fillPage1(page1);
        
        Composite page2 = new Composite(timeConfigurationPanel, SWT.NONE);
        fillPage2(page2);
        
        Composite page3 = new Composite(timeConfigurationPanel, SWT.NONE);
        fillPage3(page3);
        
        flipLayout.setSelection(0);
        timeControlCombo.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                flipLayout.setSelection(timeControlCombo.getSelectionIndex());
                // Attention:
                // Make sure you test on Windows if you change the lines below!
                getShell().pack(true);
                layout(true, true);
            }
        });
    }
    
    public IClock getClock() {
        switch(timeControlCombo.getSelectionIndex()) {
        case 0:
            return null;
        case 1:
            return new MovesPerTimeClock(movesCountSpinner.getSelection(), totalTimeSpinner.getSelection());
        case 2:
            return new FischerClock(fixedTimeSpinner.getSelection(), timeIncrementSpinner.getSelection());
        case 3:
            return new TimePerMoveClock(timePerMoveSpinner.getSelection());
        default:
            throw new RuntimeException("Error! Invalid time control selection in time control combo box.");
        }
    }

    private void fillPage1(Composite page1) {
        RowLayout layout = new RowLayout();
        layout.center = true;
        page1.setLayout(layout);
        
        movesCountSpinner = new Spinner(page1, SWT.BORDER);
        movesCountSpinner.setMinimum(1);
        movesCountSpinner.setMaximum(999);
        movesCountSpinner.setIncrement(1);
        movesCountSpinner.setPageIncrement(10);
        movesCountSpinner.setSelection(40);
        
        Label movesInLabel = new Label(page1, SWT.HORIZONTAL | SWT.LEFT);
        movesInLabel.setText("moves in");  // TODO externalize this
        
        totalTimeSpinner = new Spinner(page1, SWT.BORDER);
        totalTimeSpinner.setMinimum(1);
        totalTimeSpinner.setMaximum(999);
        totalTimeSpinner.setIncrement(1);
        totalTimeSpinner.setPageIncrement(10);
        totalTimeSpinner.setSelection(60);
        
        Label minutesLabel = new Label(page1, SWT.HORIZONTAL | SWT.LEFT);
        minutesLabel.setText("minutes");  // TODO externalize this
    }

    private void fillPage2(Composite page2) {
        RowLayout layout = new RowLayout();
        layout.center = true;
        page2.setLayout(layout);
        
        fixedTimeSpinner = new Spinner(page2, SWT.BORDER);
        fixedTimeSpinner.setMinimum(1);
        fixedTimeSpinner.setMaximum(999);
        fixedTimeSpinner.setIncrement(1);
        fixedTimeSpinner.setPageIncrement(10);
        fixedTimeSpinner.setSelection(30);
        
        Label minutesPlusLabel = new Label(page2, SWT.HORIZONTAL | SWT.LEFT);
        minutesPlusLabel.setText("minutes plus");  // TODO externalize this
        
        timeIncrementSpinner = new Spinner(page2, SWT.BORDER);
        timeIncrementSpinner.setMinimum(0);
        timeIncrementSpinner.setMaximum(999);
        timeIncrementSpinner.setIncrement(1);
        timeIncrementSpinner.setPageIncrement(10);
        timeIncrementSpinner.setSelection(0);
        
        Label secondsPerMoveLabel = new Label(page2, SWT.HORIZONTAL | SWT.LEFT);
        secondsPerMoveLabel.setText("seconds per move");  // TODO externalize this
    }

    private void fillPage3(Composite page3) {
        RowLayout layout = new RowLayout();
        layout.center = true;
        page3.setLayout(layout);
        
        timePerMoveSpinner = new Spinner(page3, SWT.BORDER);
        timePerMoveSpinner.setMinimum(1);
        timePerMoveSpinner.setMaximum(999);
        timePerMoveSpinner.setIncrement(1);
        timePerMoveSpinner.setPageIncrement(10);
        timePerMoveSpinner.setSelection(60);
        
        Label secondsPerMoveLabel = new Label(page3, SWT.HORIZONTAL | SWT.LEFT);
        secondsPerMoveLabel.setText("seconds per move");  // TODO externalize this
    }

}
