/*
Copyright (C) 2009, 2010 Alex Ignácio da Silva

This file is part of Haundrix Games.

Haundrix Games is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License version 3 as published by the
Free Software Foundation, with additional permission under section 7 to combine
it with independent works licensed under selected Free/Libre Open Source Software licenses.
Please see the full text of the license that is distributed along with this file.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
*/
package com.haundrix.games.rcp.editors;

import java.util.Map;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.Command;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.commands.ICommandService;
import org.eclipse.ui.commands.IElementUpdater;
import org.eclipse.ui.handlers.HandlerUtil;
import org.eclipse.ui.handlers.RegistryToggleState;
import org.eclipse.ui.menus.UIElement;
import org.eclipse.ui.services.IServiceLocator;

import com.haundrix.games.rcp.HaundrixCommands;
import com.haundrix.games.rcp.HaundrixPlugin;
import com.haundrix.utils.SoundPlayer;

public class ConfigureSoundHandler extends AbstractHandler implements IElementUpdater {
    
    private static final String CONTRIBUTION_TYPE_PARAMETER = "contributionType";
    private static final String MENU_CONTRIBUTION_TYPE_PARAMETER_VALUE = "menu";
    private static final String TOOLBAR_CONTRIBUTION_TYPE_PARAMETER_VALUE = "toolbar";
    
    private static final String SOUND_ENABLED_NAME = "Sound Enabled";  // TODO externalize this
    private static final String SOUND_ENABLED_DESCRIPTION = "Sound Enabled";  // TODO externalize this
    private static final String SOUND_ENABLED_MENU_IMAGE = "rsc/commandIcons/soundEnabled16.png";
    private static final String SOUND_ENABLED_TOOLBAR_IMAGE = "rsc/commandIcons/soundEnabled24.png";
    
    private static final String SOUND_DISABLED_NAME = "Sound Disabled";  // TODO externalize this
    private static final String SOUND_DISABLED_DESCRIPTION = "Sound Disabled";  // TODO externalize this
    private static final String SOUND_DISABLED_MENU_IMAGE = "rsc/commandIcons/soundDisabled16.png";
    private static final String SOUND_DISABLED_TOOLBAR_IMAGE = "rsc/commandIcons/soundDisabled24.png";

    public ConfigureSoundHandler() {
        getSoundPlayer().setMuted(!isChecked());
    }
    
    @Override
    public Object execute(ExecutionEvent event) throws ExecutionException {
        HandlerUtil.toggleCommandState(event.getCommand());
        
        getSoundPlayer().setMuted(!isChecked());
        
        return null;
    }

    @SuppressWarnings("unchecked")
    @Override
    public void updateElement(UIElement element, Map parameters) {
        String contributionType = (String) parameters.get(CONTRIBUTION_TYPE_PARAMETER);
        
        if(isChecked()) {
            element.setText(SOUND_ENABLED_NAME);
            element.setTooltip(SOUND_ENABLED_DESCRIPTION);
            String image = null;
            if(MENU_CONTRIBUTION_TYPE_PARAMETER_VALUE.equals(contributionType)) {
                image = SOUND_ENABLED_MENU_IMAGE;
            } else if(TOOLBAR_CONTRIBUTION_TYPE_PARAMETER_VALUE.equals(contributionType)) {
                image = SOUND_ENABLED_TOOLBAR_IMAGE;
            }
            element.setIcon(image != null ? HaundrixPlugin.getImageDescriptor(image) : null);
        } else {
            element.setText(SOUND_DISABLED_NAME);
            element.setTooltip(SOUND_DISABLED_DESCRIPTION);
            String image = null;
            if(MENU_CONTRIBUTION_TYPE_PARAMETER_VALUE.equals(contributionType)) {
                image = SOUND_DISABLED_MENU_IMAGE;
            } else if(TOOLBAR_CONTRIBUTION_TYPE_PARAMETER_VALUE.equals(contributionType)) {
                image = SOUND_DISABLED_TOOLBAR_IMAGE;
            }
            element.setIcon(image != null ? HaundrixPlugin.getImageDescriptor(image) : null);
        }
    }
    
    private boolean isChecked() {
        IServiceLocator serviceLocator = PlatformUI.getWorkbench();
        ICommandService commandService = (ICommandService) serviceLocator.getService(ICommandService.class);
        Command configureSoundCommand = commandService.getCommand(HaundrixCommands.CONFIGURE_SOUND.getId());
        return (Boolean) configureSoundCommand.getState(RegistryToggleState.STATE_ID).getValue();
    }

    private SoundPlayer getSoundPlayer() {
        return HaundrixPlugin.getInstance().getSoundPlayer();
    }

}
