/*
Copyright (C) 2009, 2010 Alex Ignácio da Silva

This file is part of Haundrix Games.

Haundrix Games is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License version 3 as published by the
Free Software Foundation, with additional permission under section 7 to combine
it with independent works licensed under selected Free/Libre Open Source Software licenses.
Please see the full text of the license that is distributed along with this file.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
*/
package com.haundrix.games.rcp.editors;

import java.util.Arrays;

import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.menus.WorkbenchWindowControlContribution;

import com.haundrix.games.rcp.HaundrixPlugin;
import com.haundrix.games.rcp.managers.PieceSetManager;
import com.haundrix.games.rcp.managers.PluginPieceSet;
import com.haundrix.games.rcp.preferences.HaundrixPreferences;
import com.haundrix.utils.NameComparator;

public class SelectPieceSetControl extends WorkbenchWindowControlContribution {

    private PluginPieceSet[] pieceSets;
    
    private Combo combo;
    
    private IPropertyChangeListener preferencesListener = new IPropertyChangeListener() {
        @Override
        public void propertyChange(PropertyChangeEvent event) {
            if(event.getProperty().equals(HaundrixPreferences.CHESS_BOARD_PIECE_SET)) {
                updateSelectedPieceSet();
            }
        }
    };
    
    public SelectPieceSetControl() {
    }

    public SelectPieceSetControl(String id) {
        super(id);
    }

    @Override
    protected Control createControl(Composite parent) {
        pieceSets = PieceSetManager.getInstance().getAvailablePieceSets();
        Arrays.sort(pieceSets, NameComparator.getInstance());
        
        combo = new Combo(parent, SWT.DROP_DOWN | SWT.READ_ONLY);
        for(PluginPieceSet pieceSet : pieceSets) {
            combo.add(pieceSet.getName());
        }
        
        // Workaround to prevent the combo from being displayed with incorrect height on Linux x86.
        // See:
        // https://bugs.eclipse.org/bugs/show_bug.cgi?id=293870
        parent.setSize(combo.computeSize(SWT.DEFAULT, SWT.DEFAULT));

        updateSelectedPieceSet();
        
        combo.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                PluginPieceSet selectedPieceSet = pieceSets[combo.getSelectionIndex()];
                getPreferenceStore().setValue(HaundrixPreferences.CHESS_BOARD_PIECE_SET, selectedPieceSet.getId());
                HaundrixPlugin.getInstance().savePreferences();
            }
        });
        
        getPreferenceStore().addPropertyChangeListener(preferencesListener);
        
        return combo;
    }
    
    @Override
    public void dispose() {
        getPreferenceStore().removePropertyChangeListener(preferencesListener);
        
        super.dispose();
    }

    private IPreferenceStore getPreferenceStore() {
        return HaundrixPlugin.getInstance().getPreferenceStore();
    }

    private void updateSelectedPieceSet() {
        String pieceSetId = getPreferenceStore().getString(HaundrixPreferences.CHESS_BOARD_PIECE_SET);
        for(int i = 0; i < pieceSets.length; i++) {
            if(pieceSets[i].getId().equals(pieceSetId)) {
                combo.select(i);
                break;
            }
        }
    }

}
