/*
Copyright (C) 2009, 2010 Alex Ignácio da Silva

This file is part of Haundrix Games.

Haundrix Games is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License version 3 as published by the
Free Software Foundation, with additional permission under section 7 to combine
it with independent works licensed under selected Free/Libre Open Source Software licenses.
Please see the full text of the license that is distributed along with this file.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
*/
package com.haundrix.games.rcp.editors;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.IHandler;
import org.eclipse.ui.handlers.HandlerUtil;

import com.haundrix.games.chess.model.ChessGameResult;
import com.haundrix.games.model.GameOutcome;

public class SetResultHandler extends AbstractHandler implements IHandler {
    
    private static final String RESULT_PARAMETER = "result";
    
    private static final String WHITE_WINS_RESULT_PARAMETER_VALUE = "whiteWins";
    private static final String BLACK_WINS_RESULT_PARAMETER_VALUE = "blackWins";
    private static final String DRAW_RESULT_PARAMETER_VALUE = "draw";

    @Override
    public Object execute(ExecutionEvent event) throws ExecutionException {
        String resultParameterValue = event.getParameter(RESULT_PARAMETER);
        GameOutcome outcome;
        if(resultParameterValue.equals(WHITE_WINS_RESULT_PARAMETER_VALUE)) {
            outcome = GameOutcome.WHITE_WINS;
        } else if(resultParameterValue.equals(BLACK_WINS_RESULT_PARAMETER_VALUE)) {
            outcome = GameOutcome.BLACK_WINS;
        } else if(resultParameterValue.equals(DRAW_RESULT_PARAMETER_VALUE)) {
            outcome = GameOutcome.DRAW;
        } else {
            throw new IllegalArgumentException("Error! Invalid result parameter value: " + resultParameterValue + ".");
        }
        
        ChessGameResult result = new ChessGameResult(outcome);
        RecordedChessGameEditor recordedGameEditor = (RecordedChessGameEditor) HandlerUtil.getActiveEditor(event);
        recordedGameEditor.setGameResult(result);
        
        return null;
    }

}
