/*
Copyright (C) 2009, 2010 Alex Ignácio da Silva

This file is part of Haundrix Games.

Haundrix Games is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License version 3 as published by the
Free Software Foundation, with additional permission under section 7 to combine
it with independent works licensed under selected Free/Libre Open Source Software licenses.
Please see the full text of the license that is distributed along with this file.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
*/
package com.haundrix.games.rcp.editors;

import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.image.BufferedImage;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.IHandler;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.ui.handlers.HandlerUtil;

import com.haundrix.utils.ImageSelection;
import com.haundrix.utils.ImageUtils;

public class TakeScreenshotHandler extends AbstractHandler implements IHandler {

    @Override
    public Object execute(ExecutionEvent event) throws ExecutionException {
        ImageData boardImage = ((ChessGameEditor) HandlerUtil.getActiveEditor(event)).getBoardImage();

        // The code below uses the SWT implementation of copying an image to the clipboard,
        // which is seriously flawed because:
        // 1. It doesn't work on Linux 64:
        // https://bugs.eclipse.org/bugs/show_bug.cgi?id=283960
        // 2. It transfers the image using JPEG, instead of a lossless format such as PNG:
        // https://bugs.eclipse.org/bugs/show_bug.cgi?id=279052
//        Display display = HandlerUtil.getActiveShell(event).getDisplay();
//        Clipboard clipboard = new Clipboard(display);
//        clipboard.setContents(new Object[] {boardImage}, new Transfer[] {ImageTransfer.getInstance()});
//        clipboard.dispose();
        
        // while the SWT bugs mentioned above aren't fixed, we need to convert the image to
        // a Java image and use Java's standard image copy mechanism instead:
        BufferedImage awtBoardImage = ImageUtils.convertToAwt(boardImage);
        Clipboard clipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
        clipboard.setContents(new ImageSelection(awtBoardImage), null);
        
        return null;
    }

}
