/*
Copyright (C) 2009, 2010 Alex Ignácio da Silva

This file is part of Haundrix Games.

Haundrix Games is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License version 3 as published by the
Free Software Foundation, with additional permission under section 7 to combine
it with independent works licensed under selected Free/Libre Open Source Software licenses.
Please see the full text of the license that is distributed along with this file.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
*/
package com.haundrix.games.rcp.managers;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Scale;

import com.haundrix.games.chess.model.playing.CraftyEngineConfig;
import com.haundrix.games.model.playing.EngineConfig;

public class CraftyEngineOptionsGuiConfigurer extends EngineOptionsGuiConfigurer
    implements IEngineOptionsGuiConfigurer {

    private static final String MIN_SKILL_DESCRIPTION = "Looser";  // TODO externalize this
    private static final String MAX_SKILL_DESCRIPTION = "Unbeatable";  // TODO externalize this
    
    // TODO externalize these
    private final String[] SKILL_DESCRIPTIONS = {"Fool", "Innocent", "Peaceful",
        "Careful", "Competitive", "Strong", "Aggressive",
        "Violent", "Ruthless", "Blood Thirsty"
    };
    
    private Composite engineSkillPanel;
    private Scale skillScale;
    private Label skillDescriptionLabel;

    @Override
    public void createWidgets(Composite parent) {
        super.createWidgets(parent);
        
        engineSkillPanel = new Composite(parent, SWT.NONE);
        fillEngineSkillPanel();
    }

    @Override
    public EngineConfig getConfig() {
        return new CraftyEngineConfig(ponderButton.getSelection(), skillScale.getSelection());
    }

    private void fillEngineSkillPanel() {
        GridLayout layout = new GridLayout(2, false);
        layout.horizontalSpacing = 10;
        layout.verticalSpacing = 0;
        engineSkillPanel.setLayout(layout);
        
        Label label = new Label(engineSkillPanel, SWT.HORIZONTAL | SWT.LEFT);
        label.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false, false));
        label.setText("Skill:");  // TODO externalize this
        
        skillScale = new Scale(engineSkillPanel, SWT.HORIZONTAL);
        GridData layoutData = new GridData(SWT.FILL, SWT.CENTER, true, false);
        layoutData.widthHint = 150;
        skillScale.setLayoutData(layoutData);
        skillScale.setMinimum(1);
        skillScale.setMaximum(100);
        skillScale.setIncrement(1);
        skillScale.setPageIncrement(10);
        skillScale.setSelection(50);
        skillScale.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                updateSkillDescription();
            }
        });
        
        // an empty panel just to fill the cell space
        Composite emptyPanel = new Composite(engineSkillPanel, SWT.NONE);
        GridData emptyPanelLayoutData = new GridData(SWT.LEFT, SWT.CENTER, false, false);
        emptyPanelLayoutData.widthHint = 0;
        emptyPanelLayoutData.heightHint = 0;
        emptyPanel.setLayoutData(emptyPanelLayoutData);
        
        skillDescriptionLabel = new Label(engineSkillPanel, SWT.HORIZONTAL | SWT.LEFT);
        skillDescriptionLabel.setLayoutData(new GridData(SWT.CENTER, SWT.CENTER, true, false));
        updateSkillDescription();
    }

    private void updateSkillDescription() {
        int skill = skillScale.getSelection();
        String skillDescription;
        if(skill == 1) {
            skillDescription = MIN_SKILL_DESCRIPTION;
        } else if(skill == 100) {
            skillDescription = MAX_SKILL_DESCRIPTION;
        } else {
            skillDescription = SKILL_DESCRIPTIONS[skill / 10];
        }
        
        skillDescription = skill + "% " + skillDescription;
        skillDescriptionLabel.setText(skillDescription);
        
        engineSkillPanel.layout(true);
    }

}
