/*
Copyright (C) 2009, 2010 Alex Ignácio da Silva

This file is part of Haundrix Games.

Haundrix Games is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License version 3 as published by the
Free Software Foundation, with additional permission under section 7 to combine
it with independent works licensed under selected Free/Libre Open Source Software licenses.
Please see the full text of the license that is distributed along with this file.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
 */
package com.haundrix.games.rcp.preferences;

import java.util.Arrays;
import java.util.Comparator;

import org.eclipse.jface.preference.ComboFieldEditor;
import org.eclipse.jface.preference.FieldEditorPreferencePage;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;

import com.haundrix.games.rcp.HaundrixPlugin;
import com.haundrix.games.rcp.managers.FigurineFontManager;
import com.haundrix.games.rcp.managers.PluginFigurineFont;
import com.haundrix.rcp.SpinnerFieldEditor;

public class GameHistoryPreferencePage extends FieldEditorPreferencePage implements IWorkbenchPreferencePage {

    public GameHistoryPreferencePage() {
        super(FieldEditorPreferencePage.GRID);
        setPreferenceStore(HaundrixPlugin.getInstance().getPreferenceStore());
        setDescription("Game History view settings:");  // TODO externalize this
    }

    @Override
    public void init(IWorkbench workbench) {
    }

    @Override
    protected void createFieldEditors() {
        SpinnerFieldEditor headerFontSizePreference =
            new SpinnerFieldEditor(HaundrixPreferences.GAME_HISTORY_HEADER_FONT_SIZE, "Header font size:",
            getFieldEditorParent());  // TODO externalize this
        headerFontSizePreference.setMinValue(1);
        addField(headerFontSizePreference);
        
        PluginFigurineFont[] figurineFonts = FigurineFontManager.getInstance().getAvailableFigurineFonts();
        Arrays.sort(figurineFonts, new Comparator<PluginFigurineFont>() {
            @Override
            public int compare(PluginFigurineFont figurineFont1, PluginFigurineFont figurineFont2) {
                return figurineFont1.getName().compareTo(figurineFont2.getName());
            }
        });
        String[][] figurineFontChoices = new String[figurineFonts.length][2];
        for(int i = 0; i < figurineFonts.length; i++) {
            figurineFontChoices[i][0] = figurineFonts[i].getName();
            figurineFontChoices[i][1] = figurineFonts[i].getId();
        }
        ComboFieldEditor movesFigurineFontIdPreference =
            new ComboFieldEditor(HaundrixPreferences.GAME_HISTORY_MOVES_FIGURINE_FONT, "Moves font:",
            figurineFontChoices, getFieldEditorParent());  // TODO externalize this
        addField(movesFigurineFontIdPreference);
        
        SpinnerFieldEditor movesFontSizePreference =
            new SpinnerFieldEditor(HaundrixPreferences.GAME_HISTORY_MOVES_FONT_SIZE, "Moves font size:",
            getFieldEditorParent());  // TODO externalize this
        movesFontSizePreference.setMinValue(1);
        addField(movesFontSizePreference);
    }

}
