/*
Copyright (C) 2009, 2010 Alex Ignácio da Silva

This file is part of Haundrix Games.

Haundrix Games is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License version 3 as published by the
Free Software Foundation, with additional permission under section 7 to combine
it with independent works licensed under selected Free/Libre Open Source Software licenses.
Please see the full text of the license that is distributed along with this file.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
*/
package com.haundrix.games.rcp.views;

import java.util.Arrays;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.ActionContributionItem;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IContributionItem;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.ui.actions.CompoundContributionItem;

import com.haundrix.games.rcp.HaundrixPlugin;
import com.haundrix.games.rcp.managers.FigurineFontManager;
import com.haundrix.games.rcp.managers.PluginFigurineFont;
import com.haundrix.games.rcp.preferences.HaundrixPreferences;
import com.haundrix.utils.NameComparator;

public class DynamicFormatFigurineFontTypeContributionItem extends CompoundContributionItem {

    private FormatFigurineFontTypeAction[] actions;
    private FormatFigurineFontTypeAction previousSelectedAction;
    
    private IContributionItem[] contributionItems;
    
    private IPropertyChangeListener preferencesListener = new IPropertyChangeListener() {
        @Override
        public void propertyChange(PropertyChangeEvent event) {
            if(event.getProperty().equals(HaundrixPreferences.GAME_HISTORY_MOVES_FIGURINE_FONT)) {
                updateSelectedAction();
            }
        }
    };

    public DynamicFormatFigurineFontTypeContributionItem() {
    }

    public DynamicFormatFigurineFontTypeContributionItem(String id) {
        super(id);
    }

    @Override
    public void dispose() {
        IPreferenceStore preferences = HaundrixPlugin.getInstance().getPreferenceStore();
        preferences.removePropertyChangeListener(preferencesListener);
        
        super.dispose();
    }

    @Override
    protected IContributionItem[] getContributionItems() {
        if(contributionItems == null) {
            createContributionItems();
        }
        
        return contributionItems;
    }

    private void createContributionItems() {
        PluginFigurineFont[] figurineFonts = FigurineFontManager.getInstance().getAvailableFigurineFonts();
        Arrays.sort(figurineFonts, NameComparator.getInstance());
        
        actions = new FormatFigurineFontTypeAction[figurineFonts.length];
        contributionItems = new IContributionItem[figurineFonts.length];
        for(int i = 0; i < figurineFonts.length; i++) {
            PluginFigurineFont figurineFont = figurineFonts[i];
            FormatFigurineFontTypeAction action = new FormatFigurineFontTypeAction(figurineFont);
            actions[i] = action;
            contributionItems[i] = new ActionContributionItem(action);
        }
        
        updateSelectedAction();
        
        IPreferenceStore preferences = HaundrixPlugin.getInstance().getPreferenceStore();
        preferences.addPropertyChangeListener(preferencesListener);
    }
    
    private void updateSelectedAction() {
        if(previousSelectedAction != null) {
            previousSelectedAction.setChecked(false);
        }
        
        IPreferenceStore preferences = HaundrixPlugin.getInstance().getPreferenceStore();
        String selectedFigurineFontId = preferences.getString(HaundrixPreferences.GAME_HISTORY_MOVES_FIGURINE_FONT);
        for(FormatFigurineFontTypeAction action : actions) {
            if(action.getFigurineFont().getId().equals(selectedFigurineFontId)) {
                action.setChecked(true);
                previousSelectedAction = action;
                break;
            }
        }
    }

    private class FormatFigurineFontTypeAction extends Action {
        
        private PluginFigurineFont figurineFont;

        public FormatFigurineFontTypeAction(PluginFigurineFont figurineFont) {
            super(figurineFont.getName(), IAction.AS_RADIO_BUTTON);
            
            this.figurineFont = figurineFont;
        }

        public PluginFigurineFont getFigurineFont() {
            return figurineFont;
        }

        @Override
        public void run() {
            if(isChecked()) {
                IPreferenceStore preferences = HaundrixPlugin.getInstance().getPreferenceStore();
                preferences.setValue(HaundrixPreferences.GAME_HISTORY_MOVES_FIGURINE_FONT, figurineFont.getId());
                HaundrixPlugin.getInstance().savePreferences();
            }
        }

    }

}
