/*
Copyright (C) 2009, 2010 Alex Ignácio da Silva

This file is part of Haundrix Games.

Haundrix Games is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License version 3 as published by the
Free Software Foundation, with additional permission under section 7 to combine
it with independent works licensed under selected Free/Libre Open Source Software licenses.
Please see the full text of the license that is distributed along with this file.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
*/
package com.haundrix.games.rcp.views;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Spinner;

import com.haundrix.swt.CenteredDialog;

class FontSizeDialog extends CenteredDialog {
    
    private final int initialFontSize;
    
    private Integer fontSize;

    private Spinner spinner;

    public FontSizeDialog(Shell parent, int initialFontSize) {
        super(parent);
        
        this.initialFontSize = initialFontSize;
    }

    public Integer getFontSize() {
        openAndRunLoop();
        
        return fontSize;
    }
    
    @Override
    protected void createWidgets() {
        dialog.setLayout(new GridLayout(1, true));
        
        spinner = new Spinner(dialog, SWT.BORDER);
        spinner.setMinimum(1);
        spinner.setSelection(initialFontSize);
        spinner.setLayoutData(new GridData(GridData.CENTER, GridData.CENTER, true, true));
        
        Composite buttonsPanel = new Composite(dialog, SWT.NONE);
        buttonsPanel.setLayoutData(new GridData(GridData.CENTER, GridData.CENTER, true, false));
        fillButtonsPanel(buttonsPanel);
    }

    private void fillButtonsPanel(Composite buttonsPanel) {
        RowLayout rowLayout = new RowLayout();
        rowLayout.center = true;
        rowLayout.fill = true;
        rowLayout.pack = false;
        buttonsPanel.setLayout(rowLayout);
        
        Button cancelButton = new Button(buttonsPanel, SWT.PUSH);
        cancelButton.setText("&Cancel");  // TODO externalize this
        cancelButton.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                dispose();
            }
        });

        Button okButton = new Button(buttonsPanel, SWT.PUSH);
        okButton.setText("&OK");  // TODO externalize this
        okButton.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                fontSize = spinner.getSelection();
                dispose();
            }
        });
    }

}
