/*
Copyright (C) 2009, 2010 Alex Ignácio da Silva

This file is part of Haundrix Games.

Haundrix Games is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License version 3 as published by the
Free Software Foundation, with additional permission under section 7 to combine
it with independent works licensed under selected Free/Libre Open Source Software licenses.
Please see the full text of the license that is distributed along with this file.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
*/
package com.haundrix.games.rcp.views;

import org.eclipse.jface.action.IContributionItem;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.menus.WorkbenchWindowControlContribution;

import com.haundrix.games.model.Piece.PieceColor;
import com.haundrix.rcp.PartAdapter;

public class SelectEngineColorControl extends WorkbenchWindowControlContribution {
    
    private Combo engineColorCombo;
    
    private EngineView engineView;
    
    private InternalPartListener partListener = new InternalPartListener();

    public SelectEngineColorControl() {
    }

    public SelectEngineColorControl(String id) {
        super(id);
    }

    @Override
    public void dispose() {
        if(!PlatformUI.getWorkbench().isClosing()) {
            getWorkbenchWindow().getActivePage().removePartListener(partListener);
        }
        
        super.dispose();
    }

    @Override
    protected Control createControl(Composite parent) {
        engineColorCombo = new Combo(parent, SWT.DROP_DOWN | SWT.READ_ONLY);
        engineColorCombo.add("White");  // TODO externalize this
        engineColorCombo.add("Black");  // TODO externalize this
        
        // Workaround to prevent the combo from being displayed with incorrect height on Linux x86.
        // See:
        // https://bugs.eclipse.org/bugs/show_bug.cgi?id=293870
        parent.setSize(engineColorCombo.computeSize(SWT.DEFAULT, SWT.DEFAULT));

        engineColorCombo.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                PieceColor engineColor = engineColorCombo.getSelectionIndex() == 0 ? PieceColor.WHITE : PieceColor.BLACK;
                engineView.setEngineColor(engineColor);
            }
        });

        getWorkbenchWindow().getActivePage().addPartListener(partListener);
        
        return engineColorCombo;
    }

    void updateEngineColor() {
        int comboSelectionIndex = engineView.getEngineColor() == PieceColor.WHITE ? 0 : 1;
        engineColorCombo.select(comboSelectionIndex);
    }

    private class InternalPartListener extends PartAdapter {

        @Override
        public void partOpened(IWorkbenchPart part) {
            if(part instanceof EngineView) {
                EngineView view = (EngineView) part;
                for(IContributionItem toolbarItem : view.getViewSite().getActionBars().getToolBarManager().getItems()) {
                    if(toolbarItem == SelectEngineColorControl.this) {
                        engineView = view;
                        engineView.setEngineColorControl(SelectEngineColorControl.this);
                        updateEngineColor();
                        getWorkbenchWindow().getActivePage().removePartListener(this);
                        break;
                    }
                }
            }
        }
        
    }

}
