/*
Copyright (C) 2009, 2010 Alex Ignácio da Silva

This file is part of Haundrix Games.

Haundrix Games is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License version 3 as published by the
Free Software Foundation, with additional permission under section 7 to combine
it with independent works licensed under selected Free/Libre Open Source Software licenses.
Please see the full text of the license that is distributed along with this file.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
*/
package com.haundrix.games.view;

import java.awt.image.BufferedImage;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;

import com.haundrix.swt.CenteredDialog;
import com.haundrix.utils.ImageUtils;

public class PieceSelectorDialog<P extends IPieceType> extends CenteredDialog {
    
    private RectangularBoardControl<P> boardControl;
    private P[] selectionPieces;
    private int pieceSize;
    
    private P selectedPiece;
    
    public PieceSelectorDialog(RectangularBoardControl<P> boardControl, P[] selectionPieces, int pieceSize) {
        super(boardControl.getShell(), false);
        
        this.boardControl = boardControl;
        this.selectionPieces = selectionPieces;
        this.pieceSize = pieceSize;
    }
    
    public P getPiece() {
        openAndRunLoop();
        
        return selectedPiece;
    }

    @Override
    protected void createWidgets() {
        dialog.setLayout(new GridLayout(selectionPieces.length, true));
        
        PieceSet<P> pieceSet = boardControl.getPieceSet();
        for(final P selectionPiece : selectionPieces) {
            Button button = new Button(dialog, SWT.PUSH);
            
            BufferedImage awtImage = pieceSet.loadPieceImage(selectionPiece);
            awtImage = ImageUtils.fastAndGoodScale(awtImage, pieceSize, pieceSize);
            ImageData imageData = ImageUtils.convertToSwt(awtImage);
            final Image image = new Image(dialog.getDisplay(), imageData);
            button.setImage(image);
            button.addDisposeListener(new DisposeListener() {
                @Override
                public void widgetDisposed(DisposeEvent e) {
                    image.dispose();
                }
            });
            
            GridData gridData = new GridData(GridData.CENTER, GridData.CENTER, true, true);
            button.setLayoutData(gridData);
            
            button.addSelectionListener(new SelectionAdapter() {
                @Override
                public void widgetSelected(SelectionEvent e) {
                    selectedPiece = selectionPiece;
                    dispose();
                }
            });
        }
    }
    
}
