/*
Copyright (C) 2009, 2010 Alex Ignácio da Silva

This file is part of Haundrix Games.

Haundrix Games is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License version 3 as published by the
Free Software Foundation, with additional permission under section 7 to combine
it with independent works licensed under selected Free/Libre Open Source Software licenses.
Please see the full text of the license that is distributed along with this file.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
*/
package com.haundrix.games.view;

import java.awt.image.BufferedImage;
import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Map;

import javax.imageio.ImageIO;

public class PieceSet<P extends IPieceType> {

    private Map<P, IPieceImageDescriptor> pieceImages = new HashMap<P, IPieceImageDescriptor>();

    public PieceSet(Map<P, IPieceImageDescriptor> pieceImages) {
        this.pieceImages = pieceImages;
    }
    
    public IPieceImageDescriptor getPieceImageDescriptor(P piece) {
        return pieceImages.get(piece);
    }
    
    public BufferedImage loadPieceImage(P piece) {
        BufferedImage image = null;
        
        InputStream imageStream = getPieceImageDescriptor(piece).getPieceImageAsStream();
        try {
            image = ImageIO.read(imageStream);
        } catch(IOException e) {
            // TODO load an alternate image instead of failing
            throw new RuntimeException(e);
        } finally {
            try {
                imageStream.close();
            } catch(IOException e) {
                // never mind
            }
        }
        
        return image;
    }
    
}
