/*
Copyright (C) 2009, 2010 Alex Ignácio da Silva

This file is part of Haundrix Games.

Haundrix Games is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License version 3 as published by the
Free Software Foundation, with additional permission under section 7 to combine
it with independent works licensed under selected Free/Libre Open Source Software licenses.
Please see the full text of the license that is distributed along with this file.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
*/
package com.haundrix.games.view;

import org.eclipse.swt.graphics.Point;

public class ResizeBoardTransform implements IBoardTransform {

    private int originalTopMargin;
    private int originalLeftMargin;
    
    private int transformedTopMargin;
    private int transformedLeftMargin;
    
    private double squareSizesRatio;
    
    public ResizeBoardTransform(int originalTopMargin, int originalLeftMargin, int originalSquareSize,
            int transformedTopMargin, int transformedLeftMargin, int transformedSquareSize) {
        this.originalTopMargin = originalTopMargin;
        this.originalLeftMargin = originalLeftMargin;
        
        this.transformedTopMargin = transformedTopMargin;
        this.transformedLeftMargin = transformedLeftMargin;

        squareSizesRatio = ((double) transformedSquareSize) / ((double) originalSquareSize);
    }
    
    // The transform is carried out so that the quantity (coordinate - margin) / squareSize is kept constant.
    @Override
    public Point transform(Point originalPoint) {
        int x = ((int) ((originalPoint.x - originalLeftMargin) * squareSizesRatio)) + transformedLeftMargin;
        int y = ((int) ((originalPoint.y - originalTopMargin) * squareSizesRatio)) + transformedTopMargin;
        return new Point(x, y);
    }

}
