/*
Copyright (C) 2009, 2010 Alex Ignácio da Silva

This file is part of Haundrix Games.

Haundrix Games is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License version 3 as published by the
Free Software Foundation, with additional permission under section 7 to combine
it with independent works licensed under selected Free/Libre Open Source Software licenses.
Please see the full text of the license that is distributed along with this file.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
 */
package com.haundrix.graphics;

import java.awt.image.BufferedImage;
import java.awt.image.ConvolveOp;
import java.awt.image.Kernel;

public class BlurRenderer {

    private int radius;
    
    private ConvolveOp horizontalBlur;
    private ConvolveOp verticalBlur;

    public BlurRenderer(int radius) {
        if(radius < 1) {
            throw new IllegalArgumentException("Blurring radius must be >= 1 .");
        }

        this.radius = radius;
        horizontalBlur = getGaussianBlurFilter(true);
        verticalBlur = getGaussianBlurFilter(false);
    }

    public int getRadius() {
        return radius;
    }

    public BufferedImage getBlurredImage(BufferedImage image) {
        BufferedImage horizontalBlurredImage = horizontalBlur.filter(image, null);
        return verticalBlur.filter(horizontalBlurredImage, null);
    }

    // Based on code taken from the book:
    // Filthy Rich Clients
    // Chet Haase and Romain Guy
    // The Java Series
    // 2008, Addison-Wesley
    // chapter 16, GaussianBlur demo
    private ConvolveOp getGaussianBlurFilter(boolean horizontal) {
        int size = radius * 2 + 1;
        float[] data = new float[size];

        float sigma = radius / 3.0F;
        float twoSigmaSquare = 2.0F * sigma * sigma;
        float total = 0.0F;
        for(int i = -radius; i <= radius; i++) {
            float distance = i * i;
            int index = i + radius;
            data[index] = (float) Math.exp(-distance / twoSigmaSquare);
            total += data[index];
        }
        for(int i = 0; i < data.length; i++) {
            data[i] /= total;
        }

        Kernel kernel = horizontal ? new Kernel(size, 1, data) : new Kernel(1, size, data);
        return new ConvolveOp(kernel, ConvolveOp.EDGE_NO_OP, null);
    }

}
