/*
Copyright (C) 2009, 2010 Alex Ignácio da Silva

This file is part of Haundrix Games.

Haundrix Games is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License version 3 as published by the
Free Software Foundation, with additional permission under section 7 to combine
it with independent works licensed under selected Free/Libre Open Source Software licenses.
Please see the full text of the license that is distributed along with this file.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
*/
package com.haundrix.graphics;

import org.eclipse.swt.graphics.RGB;

public class Highlight {

    private RGB color;
    private float intensity;
    private float glowFracThickness;
    
    public Highlight(RGB color, float intensity, float glowFracThickness) {
        if(color == null) {
            throw new IllegalArgumentException("Highlight color cannot be null.");
        }
        if((intensity < 0.0F) || (intensity > 1.0F)) {
            throw new IllegalArgumentException("Highlight intensity must be a number within 0 and 1.");
        }
        if((glowFracThickness < 0.0F) || (glowFracThickness > 1.0F)) {
            throw new IllegalArgumentException("Glow fractional thickness must be a number within 0 and 1.");
        }
        
        this.color = color;
        this.intensity = intensity;
        this.glowFracThickness = glowFracThickness;
    }

    public RGB getColor() {
        return color;
    }

    public float getIntensity() {
        return intensity;
    }

    public Highlight derive(float intensity) {
        if((intensity < 0.0F) || (intensity > 1.0F)) {
            throw new IllegalArgumentException("Highlight intensity must be a number within 0 and 1.");
        }

        return new Highlight(new RGB(color.red, color.green, color.blue), intensity, glowFracThickness);
    }
    
    public float getGlowFracThickness() {
        return glowFracThickness;
    }

    @Override
    public boolean equals(Object object) {
        if((object == null) || !(object instanceof Highlight)) {
            return false;
        }
        
        Highlight otherHighlight = (Highlight) object;
        return otherHighlight.color.equals(color) && (otherHighlight.intensity == intensity) &&
            (otherHighlight.glowFracThickness == glowFracThickness);
    }

}
