/*
Copyright (C) 2009, 2010 Alex Ignácio da Silva

This file is part of Haundrix Games.

Haundrix Games is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License version 3 as published by the
Free Software Foundation, with additional permission under section 7 to combine
it with independent works licensed under selected Free/Libre Open Source Software licenses.
Please see the full text of the license that is distributed along with this file.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
*/
package com.haundrix.graphics;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Device;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.graphics.Rectangle;

public class SimpleBorder implements IBorder {
    
    private Device device;
    private Rectangle inset;
    private int thickness;
    private Color color;

    public SimpleBorder(Device device, Rectangle inset, int thickness, RGB color) {
        this.device = device;
        this.inset = inset;
        this.thickness = thickness;
        this.color = new Color(device, color);
    }

    @Override
    public Rectangle getInset() {
        return inset;
    }
    
    @Override
    public void setInset(Rectangle inset) {
        this.inset = inset;
    }
    
    @Override
    public int getThickness() {
        return thickness;
    }
    
    @Override
    public void setThickness(int thickness) {
        this.thickness = thickness;
    }
    
    @Override
    public RGB getColor() {
        return color.getRGB();
    }

    @Override
    public void setColor(RGB color) {
        this.color.dispose();
        this.color = new Color(device, color);
    }

    @Override
    public void draw(GC gc) {
        int antialias = gc.getAntialias();
        gc.setAntialias(SWT.OFF);  // we don't want the borders to overlap
        Color backgroundColor = gc.getBackground();
        gc.setBackground(color);
        
        Rectangle clipping = gc.getClipping();
        
        // left border
        if(clipping.x < inset.x) {
            gc.fillRectangle(inset.x - thickness, inset.y - thickness, thickness, inset.height + 2 * thickness);
        }
        
        // top border
        if(clipping.y < inset.y) {
            gc.fillRectangle(inset.x, inset.y - thickness, inset.width, thickness);
        }
        
        // right border
        if(clipping.x + clipping.width > inset.x + inset.width) {
            gc.fillRectangle(inset.x + inset.width, inset.y - thickness, thickness, inset.height + 2 * thickness);
        }
        
        // bottom border
        if(clipping.y + clipping.height > inset.y + inset.height) {
            gc.fillRectangle(inset.x, inset.y + inset.height, inset.width, thickness);
        }
        
        gc.setBackground(backgroundColor);
        gc.setAntialias(antialias);
    }
    
    @Override
    public void dispose() {
        color.dispose();
    }

}
