/*
Copyright (C) 2009, 2010 Alex Ignácio da Silva

This file is part of Haundrix Games.

Haundrix Games is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License version 3 as published by the
Free Software Foundation, with additional permission under section 7 to combine
it with independent works licensed under selected Free/Libre Open Source Software licenses.
Please see the full text of the license that is distributed along with this file.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
*/
package com.haundrix.rcp;

import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IPartListener;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PlatformUI;

public class ActiveEditorListener<T extends IEditorPart> extends PartAdapter implements IPartListener {

    private IEditorAware<T> editorAwareObject;

    public ActiveEditorListener(IEditorAware<T> editorAwareObject){
        this.editorAwareObject = editorAwareObject;
    }
    
    @Override
    public void partActivated(IWorkbenchPart part) {
        if(part instanceof IEditorPart) {
            IEditorPart editor = (IEditorPart) part;
            Class<T> editorClass = editorAwareObject.getEditorClass();
            T activeEditor =  isValidEditor(editor) ? editorClass.cast(editor) : null;
            editorAwareObject.setActiveEditor(activeEditor);
        }
    }

    @Override
    public void partClosed(IWorkbenchPart part) {
        if(PlatformUI.getWorkbench().isClosing()) {
            editorAwareObject.setActiveEditor(null);
            return;
        }
        
        if(part instanceof IEditorPart) {
            if(editorAwareObject.getActiveEditor() == part) {
                editorAwareObject.setActiveEditor(null);
            }
        }
    }
    
    public boolean isValidEditor(IEditorPart editor) {
        Class<T> editorClass = editorAwareObject.getEditorClass();
        return editorClass.isInstance(editor) && !(editor.getEditorInput() instanceof InvalidEditorInput);
    }

}
