/*
Copyright (C) 2009, 2010 Alex Ignácio da Silva

This file is part of Haundrix Games.

Haundrix Games is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License version 3 as published by the
Free Software Foundation, with additional permission under section 7 to combine
it with independent works licensed under selected Free/Libre Open Source Software licenses.
Please see the full text of the license that is distributed along with this file.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
*/
package com.haundrix.rcp;

import java.io.File;

import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.IPersistableElement;

public class FileEditorInput implements IEditorInput, IPersistableElement {
    
    public static final String PATH_NAME_KEY = "pathName";
    
    private File file;

    public FileEditorInput(String pathName) {
        file = new File(pathName);
    }
    
    @Override
    public boolean exists() {
        return file.exists();
    }

    @Override
    public ImageDescriptor getImageDescriptor() {
        return ImageDescriptor.getMissingImageDescriptor();
    }

    @Override
    public String getName() {
        return file.getName();
    }

    @Override
    public IPersistableElement getPersistable() {
        return this;
    }

    @Override
    public String getToolTipText() {
        return file.getAbsolutePath();
    }

    @SuppressWarnings("unchecked")
    @Override
    public Object getAdapter(Class adapter) {
        if(adapter == IPersistableElement.class) {
            return this;
        } else {
            return Platform.getAdapterManager().getAdapter(this, adapter);
        }
    }

    @Override
    public String getFactoryId() {
        return FileEditorInputElementFactory.ID;
    }

    @Override
    public void saveState(IMemento memento) {
        memento.putString(PATH_NAME_KEY, file.getAbsolutePath());
    }

    @Override
    public boolean equals(Object obj) {
        if(this == obj) {
            return true;
        }
        if(obj == null) {
            return false;
        }
        if(getClass() != obj.getClass()) {
            return false;
        }
        
        FileEditorInput other = (FileEditorInput) obj;
        if(file == null) {
            if(other.file != null) {
                return false;
            }
        } else if(!file.equals(other.file)) {
            return false;
        }
        
        return true;
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + ((file == null) ? 0 : file.hashCode());
        return result;
    }
    
    public File getFile() {
        return file;
    }

}
