/*
Copyright (C) 2009, 2010 Alex Ignácio da Silva

This file is part of Haundrix Games.

Haundrix Games is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License version 3 as published by the
Free Software Foundation, with additional permission under section 7 to combine
it with independent works licensed under selected Free/Libre Open Source Software licenses.
Please see the full text of the license that is distributed along with this file.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
*/
package com.haundrix.rcp;

import org.eclipse.jface.preference.FieldEditor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Spinner;

public class SpinnerFieldEditor extends FieldEditor {

    private Spinner spinner;
    
    private int oldValue;

    public SpinnerFieldEditor(String name, String labelText, Composite parent) {
        super(name, labelText, parent);
    }

    public int getMinValue() {
        return spinner.getMinimum();
    }
    
    public void setMinValue(int minValue) {
        spinner.setMinimum(minValue);
    }
    
    public int getMaxValue() {
        return spinner.getMaximum();
    }
    
    public void setMaxValue(int maxValue) {
        spinner.setMaximum(maxValue);
    }
    
    public int getIncrement() {
        return spinner.getIncrement();
    }
    
    public void setIncrement(int increment) {
        spinner.setIncrement(increment);
    }
    
    public int getPageIncrement() {
        return spinner.getPageIncrement();
    }
    
    public void setPageIncrement(int pageIncrement) {
        spinner.setPageIncrement(pageIncrement);
    }

    @Override
    public int getNumberOfControls() {
        return 2;
    }
    
    @Override
    public void setFocus() {
        spinner.setFocus();
    }

    @Override
    protected void doFillIntoGrid(Composite parent, int numColumns) {
        Label label = getLabelControl(parent);
        label.setLayoutData(new GridData());
        
        createSpinner(parent);
        spinner.setLayoutData(new GridData());
    }

    @Override
    protected void adjustForNumColumns(int numColumns) {
    }

    @Override
    protected void doLoad() {
        int value = getPreferenceStore().getInt(getPreferenceName());
        spinner.setSelection(value);
        oldValue = value;
    }

    @Override
    protected void doLoadDefault() {
        int defaultValue = getPreferenceStore().getDefaultInt(getPreferenceName());
        spinner.setSelection(defaultValue);
        oldValue = defaultValue;
    }

    @Override
    protected void doStore() {
        getPreferenceStore().setValue(getPreferenceName(), spinner.getSelection());
    }

    private void createSpinner(Composite parent) {
        spinner = new Spinner(parent, SWT.BORDER);
        spinner.setFont(parent.getFont());
        spinner.addModifyListener(new ModifyListener() {
            @Override
            public void modifyText(ModifyEvent e) {
                int newValue = spinner.getSelection();
                setPresentsDefaultValue(false);
                fireValueChanged(FieldEditor.VALUE, oldValue, newValue);
                oldValue = newValue;
            }
        });
    }

}
