/*
Copyright (C) 2009, 2010 Alex Ignácio da Silva

This file is part of Haundrix Games.

Haundrix Games is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License version 3 as published by the
Free Software Foundation, with additional permission under section 7 to combine
it with independent works licensed under selected Free/Libre Open Source Software licenses.
Please see the full text of the license that is distributed along with this file.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
 */
package com.haundrix.swt;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ShellAdapter;
import org.eclipse.swt.events.ShellEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;

public abstract class CenteredDialog extends Dialog {
    
    private boolean closeable;

    protected Shell dialog;
    
    public CenteredDialog(Shell parent) {
        this(parent, true);
    }

    public CenteredDialog(Shell parent, boolean closeable) {
        super(parent, SWT.APPLICATION_MODAL | SWT.TITLE | SWT.BORDER);
        
        this.closeable = closeable;
    }

    public void dispose() {
        if(dialog != null) {
            dialog.dispose();
        }
    }

    protected abstract void createWidgets();
    
    protected void openAndRunLoop() {
        Shell parent = getParent();
        
        dialog = new Shell(parent, getStyle());
        dialog.setText(getText());
        if(!closeable) {
            dialog.addShellListener(new ShellAdapter() {
                @Override
                public void shellClosed(ShellEvent e) {
                    e.doit = false;
                }
            });
        }
        
        createWidgets();
        dialog.pack();
        centerInParent();
        dialog.open();
        
        Display display = parent.getDisplay();
        while(!dialog.isDisposed()) {
            if(!display.readAndDispatch()) {
                display.sleep();
            }
        }
    }
    
    private void centerInParent() {
        Shell parent = getParent();
        Point parentSize = parent.getSize();
        Point dialogSize = dialog.getSize();
        int x = parentSize.x / 2 - dialogSize.x / 2;
        int y = parentSize.y / 2 - dialogSize.y / 2;
        Point dialogLocation = parent.toDisplay(x, y);
        dialog.setLocation(dialogLocation);
    }

}