/*
Copyright (C) 2009, 2010 Alex Ignácio da Silva

This file is part of Haundrix Games.

Haundrix Games is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License version 3 as published by the
Free Software Foundation, with additional permission under section 7 to combine
it with independent works licensed under selected Free/Libre Open Source Software licenses.
Please see the full text of the license that is distributed along with this file.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
*/
package com.haundrix.swt;

import java.io.IOException;
import java.io.InputStream;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.RowData;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.DateTime;
import org.joda.time.LocalDate;

public class CompositeDateControl extends Composite {
    
    private static final String IMAGE_FILE_NAME = "compositeDateControlImage.png"; 

    private DateTime dateControl;

    public CompositeDateControl(Composite parent) {
        super(parent, SWT.NONE);
        
        RowLayout layout = new RowLayout();
        layout.fill = true;
        setLayout(layout);
        
        dateControl = new DateTime(this, SWT.DATE | SWT.MEDIUM | SWT.BORDER);
        dateControl.setLayoutData(new RowData());
        
        InputStream stream = getClass().getResourceAsStream(IMAGE_FILE_NAME);
        final Image buttonImage = new Image(getDisplay(), stream);
        try {
            stream.close();
        } catch(IOException e) {
            e.printStackTrace();
        }
        
        Button button = new Button(this, SWT.PUSH);
        button.setImage(buttonImage);
        button.setToolTipText("Pick up date...");  // TODO externalize this
        button.addDisposeListener(new DisposeListener() {
            @Override
            public void widgetDisposed(DisposeEvent e) {
                buttonImage.dispose();
            }
        });
        button.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                LocalDate newDate = new DateDialog(getShell(), getDate()).getDate();
                if(newDate != null) {
                    setDate(newDate);
                }
            }
        });
    }
 
    public LocalDate getDate() {
        int day = dateControl.getDay();
        int month = dateControl.getMonth() + 1;
        int year = dateControl.getYear();
        return new LocalDate(year, month, day);
    }
    
    public void setDate(LocalDate newDate) {
        dateControl.setDate(newDate.getYear(), newDate.getMonthOfYear() - 1, newDate.getDayOfMonth());
    }
    
}
