/*
Copyright (C) 2009, 2010 Alex Ignácio da Silva

This file is part of Haundrix Games.

Haundrix Games is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License version 3 as published by the
Free Software Foundation, with additional permission under section 7 to combine
it with independent works licensed under selected Free/Libre Open Source Software licenses.
Please see the full text of the license that is distributed along with this file.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
*/
package com.haundrix.swt;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowData;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.DateTime;
import org.eclipse.swt.widgets.Shell;
import org.joda.time.LocalDate;

public class DateDialog extends CenteredDialog {
    
    private LocalDate initialDate;
    
    private LocalDate date;

    private DateTime dateControl;

    public DateDialog(Shell parent, LocalDate initialDate) {
        super(parent);
        
        this.initialDate = initialDate;
    }
    
    public LocalDate getDate() {
        openAndRunLoop();
        
        return date;
    }

    @Override
    protected void createWidgets() {
        dialog.setLayout(new GridLayout(1, false));
        
        dateControl = new DateTime(dialog, SWT.CALENDAR);
        dateControl.setDate(initialDate.getYear(), initialDate.getMonthOfYear() - 1, initialDate.getDayOfMonth());
        dateControl.setLayoutData(new GridData());
        
        Composite buttonsPanel = new Composite(dialog, SWT.NONE);
        buttonsPanel.setLayoutData(new GridData(GridData.END, GridData.CENTER, false, false));
        fillButtonsPanel(buttonsPanel);
    }

    private void fillButtonsPanel(Composite buttonsPanel) {
        RowLayout layout = new RowLayout();
        layout.pack = false;
        layout.spacing = 10;
        buttonsPanel.setLayout(layout);
        
        Button cancelButton = new Button(buttonsPanel, SWT.PUSH);
        cancelButton.setText("&Cancel");  // TODO externalize this
        cancelButton.setLayoutData(new RowData());
        cancelButton.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                dispose();
            }
        });
        
        Button okButton = new Button(buttonsPanel, SWT.PUSH);
        okButton.setText("&OK");  // TODO externalize this
        okButton.setLayoutData(new RowData());
        okButton.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                int day = dateControl.getDay();
                int month = dateControl.getMonth() + 1;
                int year = dateControl.getYear();
                date = new LocalDate(year, month, day);
                
                dispose();
            }
        });
    }

}
