/*
Copyright (C) 2009, 2010 Alex Ignácio da Silva

This file is part of Haundrix Games.

Haundrix Games is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License version 3 as published by the
Free Software Foundation, with additional permission under section 7 to combine
it with independent works licensed under selected Free/Libre Open Source Software licenses.
Please see the full text of the license that is distributed along with this file.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
*/
package com.haundrix.swt;

import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Layout;

// Notes:
// - if the selection is out of bounds then the first page is selected.
// - since each page may have a different size, the client needs to call Control.layout()
//   in the top level container in order to resize all children.
public class FlipLayout extends Layout {
    
    private int selection = 0;

    public int getSelection() {
        return selection;
    }

    public void setSelection(int selection) {
        if(selection >= 0) {
            this.selection = selection;
        }
    }

    @Override
    protected Point computeSize(Composite composite, int wHint, int hHint, boolean flushCache) {
        Control[] pages = composite.getChildren();
        
        if((pages == null) || (pages.length == 0)) {
            return new Point(0, 0);
        }
        
        Control selectedPage = selection < pages.length ? pages[selection] : pages[0];
        return selectedPage.computeSize(wHint, hHint, flushCache);
    }

    @Override
    protected void layout(Composite composite, boolean flushCache) {
        Control[] pages = composite.getChildren();
        
        if((pages == null) || (pages.length == 0)) {
            return;
        }
        
        Control selectedPage = selection < pages.length ? pages[selection] : pages[0];
        selectedPage.setBounds(composite.getClientArea());
        for(Control page : pages) {
            page.setVisible(page == selectedPage);
        }
    }

    @Override
    protected boolean flushCache(Control control) {
        return true;
    }

}
