/*
Copyright (C) 2009, 2010 Alex Ignácio da Silva

This file is part of Haundrix Games.

Haundrix Games is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License version 3 as published by the
Free Software Foundation, with additional permission under section 7 to combine
it with independent works licensed under selected Free/Libre Open Source Software licenses.
Please see the full text of the license that is distributed along with this file.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
*/
package com.haundrix.swt;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ControlAdapter;
import org.eclipse.swt.events.ControlEvent;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Pattern;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;

public class GradientComposite extends Composite {
    
    private Color topBackgroundColor;
    private Color bottomBackgroundColor;
    
    private Image backgroundImage;

    public GradientComposite(Composite parent) {
        super(parent, SWT.NONE);
        
        setBackgroundMode(SWT.INHERIT_FORCE);
        
        Display display = parent.getDisplay();
        topBackgroundColor = display.getSystemColor(SWT.COLOR_WIDGET_BACKGROUND);
        bottomBackgroundColor = display.getSystemColor(SWT.COLOR_WIDGET_BACKGROUND);
        
        updateBackground();
        
        addControlListener(new ControlAdapter() {
            @Override
            public void controlResized(ControlEvent e) {
                updateBackground();
            }
        });
        
        addDisposeListener(new DisposeListener() {
            @Override
            public void widgetDisposed(DisposeEvent e) {
                backgroundImage.dispose();
            }
        });
    }

    public Color getTopBackgroundColor() {
        return topBackgroundColor;
    }

    public void setTopBackgroundColor(Color topBackgroundColor) {
        this.topBackgroundColor = topBackgroundColor;
        
        updateBackground();
    }

    public Color getBottomBackgroundColor() {
        return bottomBackgroundColor;
    }

    public void setBottomBackgroundColor(Color bottomBackgroundColor) {
        this.bottomBackgroundColor = bottomBackgroundColor;
        
        updateBackground();
    }

    private void updateBackground() {
        int height = Math.max(getClientArea().height, 1);
        Image backgroundImage = new Image(getDisplay(), 1, height);
        
        GC gc = new GC(backgroundImage);
        gc.setAdvanced(true);
        gc.setAntialias(SWT.ON);
        Pattern gradient = new Pattern(getDisplay(), 0.0F, 0.0F, 0.0F, height, topBackgroundColor,
                bottomBackgroundColor);
        gc.setBackgroundPattern(gradient);
        gc.fillRectangle(0, 0, 1, height);
        gradient.dispose();
        gc.dispose();
        
        Image oldBackgroundImage = this.backgroundImage;
        this.backgroundImage = backgroundImage;
        setBackgroundImage(this.backgroundImage);
        if(oldBackgroundImage != null) {
            oldBackgroundImage.dispose();
        }
    }

}
