/*
Copyright (C) 2009, 2010 Alex Ignácio da Silva

This file is part of Haundrix Games.

Haundrix Games is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License version 3 as published by the
Free Software Foundation, with additional permission under section 7 to combine
it with independent works licensed under selected Free/Libre Open Source Software licenses.
Please see the full text of the license that is distributed along with this file.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
*/
package com.haundrix.utils;

import java.io.File;
import java.io.IOException;

import javax.sound.sampled.AudioFileFormat;
import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.UnsupportedAudioFileException;

public class SoundUtils {

    private static final long UNSPECIFIED = -1L;

    public static long getSoundDuration(File soundFile, File scratchDir) throws UnsupportedAudioFileException,
            IOException {
        AudioFileFormat fileFormat = AudioSystem.getAudioFileFormat(soundFile);
        long duration = getSoundDuration(fileFormat);
        if(duration != UNSPECIFIED) {
            return duration;
        }

        // converts the file to WAV format
        AudioInputStream audioStream = AudioSystem.getAudioInputStream(soundFile);
        AudioFormat format = audioStream.getFormat();
        AudioFormat newFormat = new AudioFormat(AudioFormat.Encoding.PCM_SIGNED, format.getSampleRate(),
                16, format.getChannels(), format.getChannels() * 2, format.getSampleRate(), false);
        AudioInputStream convertedStream = AudioSystem.getAudioInputStream(newFormat, audioStream);
        String convertedFileName = Utils.changeExtension(soundFile.getName(), "wav");
        File convertedFile = new File(scratchDir, convertedFileName);
        AudioSystem.write(convertedStream, AudioFileFormat.Type.WAVE, convertedFile);
        
        AudioFileFormat convertedFileFormat = AudioSystem.getAudioFileFormat(convertedFile);
        return getSoundDuration(convertedFileFormat);
    }

    private static long getSoundDuration(AudioFileFormat fileFormat) {
        int frameLength = fileFormat.getFrameLength();
        float frameRate = fileFormat.getFormat().getFrameRate();
        if((frameLength != AudioSystem.NOT_SPECIFIED) && (frameRate > 0.0F)) {
            return (long) (((double) frameLength) / ((double) frameRate) * 1000.0);
        } else {
            return UNSPECIFIED;
        }
    }
    
}
