/*
Copyright (C) 2009, 2010 Alex Ignácio da Silva

This file is part of Haundrix Games.

Haundrix Games is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License version 3 as published by the
Free Software Foundation, with additional permission under section 7 to combine
it with independent works licensed under selected Free/Libre Open Source Software licenses.
Please see the full text of the license that is distributed along with this file.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
*/
package com.haundrix.utils;

public class StopWatch {

    private boolean on = false;
    private long startTime;
    private long time = 0L;
    private int nRuns = 0;

    public StopWatch() {
    }

    public void on() {
        if(on) {
            throw new IllegalStateException("Error! The stopwatch is already on.");
        }

        startTime = System.currentTimeMillis();
        on = true;
    }

    public void off() {
        if(!on) {
            throw new IllegalStateException("Error! The stopwatch is not on.");
        }

        time += System.currentTimeMillis() - startTime;
        nRuns++;
        on = false;
    }

    public long getTotalTime() {
        return time;
    }

    public int getNRuns() {
        return nRuns;
    }

    public double getAverageTime() {
        return ((double) time) / ((double) nRuns);
    }

    public void reset() {
        if(on) {
            throw new IllegalStateException("Error! Cannot reset the stopwatch while it is on.");
        }

        time = 0L;
        nRuns = 0;
    }
}
